C   13/11/85 607072054  MEMBER NAME  DCPMOM   (S)        M  FORTRAN
      SUBROUTINE CTPMOM(ITRACK,PMOM,ICHARG,IFLAG)
*.
*...CTPMOM   calculate track momentum from information in the 
*.           'CTTRACK' bank
*.
*. INPUT     : ITRACK  track number in the 'CTTRACK' bank
*. OUTPUT    : PMOM(5) momentum vector
*.           :      1  px
*.           :      2  py
*.           :      3  pz
*.           :      4  |p|
*.           :      5  pt
*.           : ICHARG  charge of the track
*.           : IFLAG   reconstruction flag
*.           :   = -1   no track is reconstructed 
*.           :   =  0   track is reconstructed only in r-phi plane
*.           :   =  1   track is fully reconstructed
*.
*. SEQUENCE  :
*. COMMON    : 
*. CALLS     : KZCBR1 KZPUT1 KZGET1
*. CALLED    : GUDIGI
*.
*. BANKS L   : 
*. BANKS U   :
*. BANKS M   :
*. BANKS D   : 
*.
*. REPORT CONDITIONS
*.
*. AUTHOR    : Junichi Kanzaki
*. VERSION   : 1.00
*. CREATED   : 13-Mar-1996
*. LAST MOD  : 13-Mar-1996
*.
*. MODIFICATION LOG
*. HISTORY    
*. 13-Nov-1985  Created by Junichi Kanzaki for VENUS analysis
*
      IMPLICIT NONE

      INTEGER ITRACK
      INTEGER NDPMOM
      PARAMETER (NDPMOM=5)
      REAL    PMOM(NDPMOM)
      INTEGER ICHARG,IFLAG

*--   Parameter for bank 'HEADER'
      INTEGER NWMHED
      PARAMETER (NWMHED=20)
      INTEGER NDAHED,NMXHED
      PARAMETER (NMXHED=NWMHED)
      INTEGER   IDTHED(NMXHED)
      REAL      DATHED(NMXHED)
      EQUIVALENCE (IDTHED(1),DATHED(1))


*--   Parameter for bank 'CTTRACK'
      INTEGER LSECTR
c      PARAMETER (CNACTR='CTTRACK')
      PARAMETER (LSECTR=44)
      INTEGER NDACTR,NMXCTR
      PARAMETER (NMXCTR=LSECTR)
      REAL RDACTR(NMXCTR)
      INTEGER IDACTR(NMXCTR)
      EQUIVALENCE (RDACTR(1),IDACTR(1))

      REAL AK
      PARAMETER (AK= 2.997925E-4)

      REAL BKG,CURV

      REAL PX,PY,PZ,PT,PABS,COTTH
      REAL PHI0
      INTEGER NHITRP,NHITRZ
      
      INTEGER I


      CALL VZERO(PMOM,NDPMOM)
      ICHARG=0
      IFLAG=0

      CALL KZGET('HEADER',0,NDAHED,IDTHED)
      BKG=DATHED(5)

      CALL KZGET('CTTRACK',ITRACK,NDACTR,IDACTR)
      NHITRP=IDACTR(5)
      IF (NHITRP.LE.2) THEN
         IFLAG = -1
         ICHARG = 0
         DO 10 I=1,NDPMOM
            PMOM(I) = 0.
 10      CONTINUE
      ELSE
         CURV=RDACTR(3)
         ICHARG=INT(SIGN(1.,CURV)*SIGN(1.,BKG))
         IF (CURV.NE.0) THEN 
            PT=AK*ABS(BKG/CURV)
         ELSE
            PT=AK*1.0E+20
         ENDIF   
         PMOM(5)=PT
         PHI0=RDACTR(2)
         PX=PT*COS(PHI0)
         PY=PT*SIN(PHI0)
         PMOM(1)=PX
         PMOM(2)=PY
         NHITRZ=IDACTR(9)
         IF (NHITRZ.EQ.0) THEN
            IFLAG=0
            PMOM(4)=PT
            RETURN
         ELSE
            IFLAG=1
            COTTH=RDACTR(7)
            PABS=PT*SQRT(1.+COTTH**2)
            PMOM(4)=PABS
            PZ=PABS*COTTH/SQRT(1.+COTTH**2)
            PMOM(3)=PZ
         ENDIF
      ENDIF
      
      RETURN
      END
