      SUBROUTINE BCDIGI
*.
*...BCDIGI   Compute and store DIGIts for Barrel calorimeter
*.
*. INPUT     : none
*. OUTPUT    : none
*.
*. SEQUENCE  :
*. COMMON    : 
*. CALLS     : 
*. CALLED    : GUDIGI
*.
*. BANKS L   :
*. BANKS U   :
*. BANKS M   :
*. BANKS D   :
*.
*. REPORT CONDITIONS
*.
*. AUTHOR    : Nakamura Isamu
*. VERSION   : 1.00
*. CREATED   : 17-May-1994
*. LAST MOD  : 21-Aug-1996
*.
*. MODIFICATION LOG
*.    02-Mar-1999  K.Ishii  Buffer extended for overlap region (smearing mode)
*.    17-Feb-1999  K.Ishii  Added new bank for smearing mode
*.    17-Feb-1999  K.Ishii  EEMCUT=0.0, EHDCUT=0.0 (temporary!)
*.    09-Feb-1999  Naoko KANAYA bug fixed
*.    08-Nov-1998  J.Kanzaki Apply cuts on cell energies
*.    21-Aug-1996, Y.Fujii   Hadron Bank IDATA(5) was ToF => now #ofTrack
*.    21-Apr-1995 : Nakamura Isamu
*.                Bank Configuration is completely changed
*.    20-Jul-1994 : Nakamura Isamu
*.                Add MCBCHITS ZBS Bank.
*.                
*.
*.
*.**********************************************************************
*.
*.*********************************************************************
*.     +------------------------+                                     *
*.     | Bank Name  :  MCBESMH  |                                     *
*.     +------------------------+                                     *
*.     Same as MCBEHIT (Barrel ECAL), but for smearing mode           *
*.*********************************************************************
*.     +------------------------+                                     *
*.     | Bank Name  :  MCBHSMH  |                                     *
*.     +------------------------+                                     *
*.     Same as MCBHHIT (Barrel HCAL), but for smearing mode           *
*.*********************************************************************
*.     +------------------------+                                     *
*.     | Bank Name  :  MCBEHIT  |                                     *
*.     +------------------------+                                     *
*.                                                                    *
*.     Contents    : List of Barrel Calorimeter hits on tracks which  *
*.                   the JIM generated.                               *
*.     Record Type : 1-dim. variable length segment                   *
*.     Version     : Release 1.0  1995-Apr-21                         *
*.                                                                    *
*.*********************************************************************
*.
*.(1) Contents in the Master Segment
*.----------------------------------
*.
*.----------------------------------------------------------------------
*.  DATA                            CONTENTS
*.----------------------------------------------------------------------
*. IDATA(1)   Total number of hit blocks ( EM part )
*.----------------------------------------------------------------------
*.
*.(Note)
*.   Master segment = segment # 0
*.
*.
*.(2) Contents in the Data Segment
*.--------------------------------
*.
*.----------------------------------------------------------------------
*.  DATA                            CONTENTS
*.----------------------------------------------------------------------
*. IDATA(1)      Cell Id # (1000*ITHETA+IPHI)
*.  DATA(2)      Total Energy of the Cell ( GeV )
*.  DATA(3)      Time of Flight ( psec )
*. IDATA(4)      # of Contributed Track
*.
*. IDATA(3*I+2)  Contributed Track #  |
*.  DATA(3*I+3)  Deposit Energy       + I = 1, 5
*.  DATA(3*I+4)  Time of Flight       |       Variable Length
*.----------------------------------------------------------------------
*.
*.(Note)
*.   Segment # = hits #
*.
*.---------------------------------------------------------------------
*.*********************************************************************
*.     +------------------------+                                     *
*.     | Bank Name  :  MCBHHIT  |                                     *
*.     +------------------------+                                     *
*.                                                                    *
*.     Contents    : List of Barrel Calorimeter hits on tracks which  *
*.                   the JIM generated.                               *
*.     Record Type : 1-dim. fixed length segment                      *
*.     Version     : Release 1.0  1995-Apr-21                         *
*.                                                                    *
*.*********************************************************************
*.
*.(1) Contents in the Master Segment
*.----------------------------------
*.
*.----------------------------------------------------------------------
*.  DATA                            CONTENTS
*.----------------------------------------------------------------------
*. IDATA(1)   Total number of hit blocks 
*.----------------------------------------------------------------------
*.
*.(Note)
*.   Master segment = segment # 0
*.
*.
*.(2) Contents in the Data Segment
*.--------------------------------
*.
*.----------------------------------------------------------------------
*.  DATA                            CONTENTS
*.----------------------------------------------------------------------
*. IDATA(1)        Cell # (1000*ITHETA+IPHI)
*.  DATA(2)        Deposit Energy in PMT1 ( GeV )
*.  DATA(3)        Deposit Energy in PMT2 ( GeV )
*.  DATA(4)        Time of Flight ( nsec )
*. IDATA(5)        # of Contributed Track
*.
*. IDATA(4*I+2)    Contributed Track #     |
*.  DATA(4*I+3)    Deposit Energy (PMT1)   + I = 1,5 
*.  DATA(4*I+4)    Deposit Energy (PMT2)   |    Variable Length
*.  DATA(4*I+5)    Time of Flight          |
*.----------------------------------------------------------------------
*.
*.(Note)
*.   Segment # = hits #
*.
*.---------------------------------------------------------------------
      IMPLICIT NONE

#include "geant321/gccuts.inc"
#include "jmdet.inc"
#include "bcgeo.inc"
#include "bchit.inc"
#include "bcsmdt.inc"

      INTEGER LNGBKE,LNGBKH
      PARAMETER(LNGBKE = 19)
      PARAMETER(LNGBKH = 25)
      INTEGER ITRAE(NHMXBC),NUMBVE(NDSVBC,NHMXBC)
      INTEGER ITRAH(NHMXBC),NUMBVH(NDSVBC,NHMXBC)
      INTEGER NUMVS(NDSVBC)
      INTEGER ISTART
      REAL HITSE(NDHTBC,NHMXBC)
      REAL HITSH(NDHTBC,NHMXBC)

      INTEGER IDATA(25)
      REAL RDATA(25)
      EQUIVALENCE(IDATA(1),RDATA(1))

      REAL BKEDT(LNGBKE,NHMXBC),BKHDT(LNGBKH,NHMXBC)
      DATA NUMVS  /NDSVBC*0/

      REAL EEMCUT,EHDCUT
      PARAMETER (EEMCUT=0.0, EHDCUT=0.0)

CKI 100 MeV CUT case
CKI      PARAMETER (EEMCUT=0.1*0.028, EHDCUT=0.1*0.024)

      INTEGER NHHITS,NEHITS
      INTEGER NHITSE,NHITSH

      INTEGER NHCEL,NCELLS

      INTEGER IX,IY,IC
      INTEGER IB,ISEG,JSEG,IOFFS,NECEL
      INTEGER I,J,IERR

      INTEGER NECELL,NHCELL
      INTEGER NPECEL,NPHCEL
      PARAMETER (NPECEL=2000, NPHCEL=2000)
      INTEGER IECELL(2,NPECEL),IHCELL(2,NPHCEL)
      REAL    EECELL(NPECEL),EHCELL(NPHCEL)

*-- Parameters for bank
*-    for EM hits
      INTEGER NWMBEH,NSMBEH,LSGBEH
      PARAMETER (NWMBEH=1, NSMBEH=4, LSGBEH=3)
      CHARACTER*32 FMSBEH,FSMBEH,FSGBEH
      PARAMETER (FMSBEH='1I', FSMBEH='1I 2F 1I', FSGBEH='1I 2F')
*-    for HD hits
      INTEGER NWMBHH,NSMBHH,LSGBHH
      PARAMETER (NWMBHH=1, NSMBHH=5, LSGBHH=4)
      CHARACTER*32 FMSBHH,FSMBHH,FSGBHH
      PARAMETER (FMSBHH='1I', FSMBHH='1I 3F 1I', FSGBHH='1I 3F')

      CHARACTER*80 CMSG
      INTEGER LNBLNK


*-- Initialize HIT ID#
      NEHITS = 0
      NHHITS = 0
      DO 10 I=1,NHMXBC
         DO 11 J = 1,LNGBKE
            BKEDT(J,I)=0
 11      CONTINUE
         DO 12 J = 1,LNGBKH
            BKHDT(J,I)=0
 12      CONTINUE
 10   CONTINUE
      
      DO 100 I=1,NDTHBC
         
*-- Retrieve ECAL Hit Informations from JHITS
         CALL GFHITS('BC  ',VNEBC(I),NDSVBC,NDHTBC,
     &        NHMXBC,0,NUMVS,ITRAE,NUMBVE,HITSE,NHITSE)
         IF (NHITSE.GT.NHMXBC) THEN
            WRITE(CMSG,'(''NHITSE('',I8,'') > '',I8)') NHITSE,NHMXBC
            CALL MSGOUT('BCDIGI',CMSG(1:LNBLNK(CMSG)),1)
            NHITSE = NHMXBC
         ENDIF      
         
*-- EM Part Calculation
         DO 110 J=1,NHITSE
            DO 111 IX = 1,2
               DO 112 IY = 1,2
                  IF (HITSE(2*(IX-1)+IY+1,J).GT.EEMCUT) THEN
                     NEHITS = NEHITS+1
                     
*-- Calculate Data to be Stored to BANK
                     BKEDT(1,NEHITS) = 1000*(2*(NUMBVE(2,J)-1)+IY)
     &                    +(2*(NUMBVE(1,J)-1)+IX)
                     BKEDT(2,NEHITS) = HITSE(2*(IX-1)+IY+1,J)
                     BKEDT(3,NEHITS) = HITSE(1,J)*1.E9
                     BKEDT(4,NEHITS) = 1
                     BKEDT(5,NEHITS) = ITRAE(J)
                     BKEDT(6,NEHITS) = HITSE(2*(IX-1)+IY+1,J)
                     BKEDT(7,NEHITS) = HITSE(1,J)*1.E9
                  ENDIF
 112           CONTINUE
 111        CONTINUE
 110     CONTINUE

*-- Retrieve HCAL Hit Informations from JHITS
         CALL GFHITS ('BC  ',VNHBC(I),NDSVBC,NDHTBC,
     &        NHMXBC,0,NUMVS,ITRAH,NUMBVH,HITSH,NHITSH)
         IF (NHITSH.GT.NHMXBC) THEN
            WRITE(CMSG,'(''NHITSH('',I8,'') > '',I8)') NHITSH,NHMXBC
            CALL MSGOUT('BCDIGI',CMSG(1:LNBLNK(CMSG)),2)
            NHITSH = NHMXBC
         ENDIF      

*-- Hadron Part Calculation
         DO 120 J=1,NHITSH
            IF ((HITSH(2,J)+HITSH(3,J)).GT.EHDCUT) THEN
               NHHITS = NHHITS+1
*-- Calculate Data to be Stored to BANK
               BKHDT(1,NHHITS) = NUMBVH(1,J)+1000*NUMBVH(2,J)
               BKHDT(2,NHHITS) = HITSH(2,J)
               BKHDT(3,NHHITS) = HITSH(3,J)
               BKHDT(4,NHHITS) = HITSH(1,J)*1.E9
               BKHDT(5,NHHITS) = 1
               BKHDT(6,NHHITS) = ITRAH(J)
               BKHDT(7,NHHITS) = HITSH(2,J)
               BKHDT(8,NHHITS) = HITSH(3,J)
               BKHDT(9,NHHITS) = HITSH(1,J)*1.E9
            ENDIF
 120     CONTINUE

 100  CONTINUE

*-- Merge EM Hit DATA
      ISTART = 0
 200  CONTINUE
      ISTART = ISTART + 1
      IF (ISTART.GT.NEHITS) GOTO 220
      IF (BKEDT(1,ISTART).EQ.0) GOTO 200

      DO 210 IB = ISTART+1,NEHITS
         IF (BKEDT(1,IB).EQ.BKEDT(1,ISTART)) THEN
            IF (BKEDT(4,ISTART) .LT.5) THEN
               IOFFS = INT(3*BKEDT(4,ISTART))
               BKEDT(2,ISTART) = BKEDT(2,ISTART)+BKEDT(2,IB)
               BKEDT(4,ISTART) = BKEDT(4,ISTART) + 1
               BKEDT(IOFFS+5,ISTART) = BKEDT(5,IB)
               BKEDT(IOFFS+6,ISTART) = BKEDT(6,IB)
               BKEDT(IOFFS+7,ISTART) = BKEDT(7,IB)
               BKEDT(1,IB) = 0
            ENDIF
         ENDIF

 210  CONTINUE
      GOTO 200
 220  CONTINUE

      NECEL = 0
      DO 230 ISEG = 1,NEHITS
         IF (BKEDT(1,ISEG).GT.0) THEN
            NECEL = NECEL + 1 
            DO 231 JSEG = 1,LNGBKE
               BKEDT(JSEG,NECEL)=BKEDT(JSEG,ISEG)
 231        CONTINUE
         ENDIF
 230  CONTINUE

*-- Merge HAD Hit DATA
      ISTART = 0
 300  CONTINUE
      ISTART = ISTART + 1
      IF (ISTART.GT.NHHITS) GOTO 320
      IF (BKHDT(1,ISTART).EQ.0) GOTO 300

      DO 310 IB = ISTART+1,NHHITS
         IF (BKHDT(1,IB).EQ.BKHDT(1,ISTART)) THEN
            IF (BKHDT(5,ISTART) .LT.5) THEN
               IOFFS = INT(4*BKHDT(5,ISTART))
               BKHDT(2,ISTART) = BKHDT(2,ISTART)+BKHDT(2,IB)
               BKHDT(3,ISTART) = BKHDT(3,ISTART)+BKHDT(3,IB)
               BKHDT(5,ISTART) = BKHDT(5,ISTART) + 1
               BKHDT(IOFFS+6,ISTART) = BKHDT(6,IB)
               BKHDT(IOFFS+7,ISTART) = BKHDT(7,IB)
               BKHDT(IOFFS+8,ISTART) = BKHDT(8,IB)
               BKHDT(IOFFS+9,ISTART) = BKHDT(9,IB)
               BKHDT(1,IB) = 0
            ENDIF
         ENDIF

 310  CONTINUE
      GOTO 300
 320  CONTINUE

      NHCEL = 0
      DO 330 ISEG = 1,NHHITS
         IF (BKHDT(1,ISEG).GT.0) THEN
            NHCEL = NHCEL + 1 
            DO 331 JSEG = 1,LNGBKH
               BKHDT(JSEG,NHCEL)=BKHDT(JSEG,ISEG)
 331        CONTINUE
         ENDIF
 330  CONTINUE

*-- Create ZBS Bank MCBEHIT
      
*-- First count number of hits
      NCELLS = 0

      DO 410 ISEG = 1,NECEL
         IF (BKEDT(2,ISEG).GT.EEMCUT) THEN
            NCELLS = NCELLS+1
            IF (NCELLS.GT.NPECEL) THEN
               CALL MSGOUT('BCDIGI',
     &              'No. of EM cells overflow (full)',NCELLS)
               NCELLS = NPECEL
            ENDIF
         ENDIF
 410  CONTINUE 
      
      CALL KZBCR2('MCBEHIT',NWMBEH,FMSBEH,
     &     NCELLS,NSMBEH,FSMBEH,LSGBEH,FSGBEH,IERR)

      IDATA(1) = NCELLS
      CALL KZREP2('MCBEHIT',0,0,NWMBEH,IDATA)

*-- Store Data to given Segment of Bank MCBEHIT
      NCELLS = 0

      DO 420 ISEG = 1,NECEL
         IF (BKEDT(2,ISEG).GT.EEMCUT) THEN
            NCELLS = NCELLS+1
            IF (NCELLS.GT.NPECEL) THEN
               CALL MSGOUT('BCDIGI',
     &              'No. of EM cells overflow (full)',NCELLS)
               NCELLS = NPECEL
            ENDIF
            IDATA(1)  = BKEDT(1,ISEG)
            RDATA(2)  = BKEDT(2,ISEG)
            RDATA(3)  = BKEDT(3,ISEG)
            IDATA(4)  = BKEDT(4,ISEG)
            CALL KZREP2('MCBEHIT',NCELLS,0,NSMBEH,IDATA)
            DO 421 IC = 1,5
               IDATA(1)  = BKEDT(3*IC+2,ISEG)
               RDATA(2)  = BKEDT(3*IC+3,ISEG)
               RDATA(3)  = BKEDT(3*IC+4,ISEG)
               CALL KZREP2('MCBEHIT',NCELLS,IC,LSGBEH,IDATA)
 421        CONTINUE
            IECELL(1,NCELLS) = IDATA(1)/1000 
            IECELL(2,NCELLS) = MOD(IDATA(1),1000)
            EECELL(NCELLS) = RDATA(2)
         ENDIF
 420  CONTINUE

*-- for smearing DATA
      IF (ISIMJD(IDETBC).EQ.ISMRJD) THEN

*-- First count number of hits
         NCELLS = 0
         
         DO 430 I=1,NMXBET
            DO 431 J=1,NMXBEP
*-- skip if no deposit            
               IF (EBCSMP(I,J)+EBCSME(I,J).GT.0.) THEN
                  NCELLS = NCELLS+1
                  IF (NCELLS.GT.NPECEL) THEN
                     CALL MSGOUT('BCDIGI',
     &                    'No. of EM cells overflow (smear)',NCELLS)
                     NCELLS = NPECEL
                  ENDIF
               ENDIF
 431        CONTINUE 
 430     CONTINUE

*-- Create ZBS Bank MCBESMH
      
         CALL KZBCR1('MCBESMH',NWMBEH,FMSBEH,
     &        NCELLS,NSMBEH,FSGBEH,IERR)

         IDATA(1) = NCELLS
         CALL KZREP1('MCBESMH',0,NWMBEH,IDATA)

*-- Store Data to given Segment of Bank MCBESMH
         NCELLS = 0
         
         DO 440 I=1,NMXBET
            DO 441 J=1,NMXBEP
*-- skip if no deposit            
               IF (EBCSMP(I,J)+EBCSME(I,J).GT.0.) THEN
*-- fill 
                  NCELLS = NCELLS+1
                  IF (NCELLS.GT.NPECEL) THEN
                     CALL MSGOUT('BCDIGI',
     &                    'No. of EM cells overflow (smear)',NCELLS)
                     NCELLS = NPECEL
                  ENDIF
                  IDATA(1)  = I*1000+J
                  RDATA(2)  = EBCSMP(I,J)+EBCSME(I,J)
                  RDATA(3)  = 0.
                  IDATA(4)  = 0
                  CALL KZREP1('MCBESMH',NCELLS,NSMBEH,IDATA)
                  IECELL(1,NCELLS) = -I
                  IECELL(2,NCELLS) = -J
                  EECELL(NCELLS) = RDATA(2)
               ENDIF
 441        CONTINUE 
 440     CONTINUE 

      ENDIF

*-- Create ZBS Bank MCBHHIT

*-- First count number of hits
      NCELLS = 0

      DO 510 ISEG = 1,NHCEL
         IF (BKHDT(2,ISEG).GT.EHDCUT) THEN
            NCELLS = NCELLS+1
            IF (NCELLS.GT.NPHCEL) THEN
               CALL MSGOUT('BCDIGI',
     &              'No. of HD cells overflow (full)',NCELLS)
               NCELLS = NPHCEL
            ENDIF
         ENDIF
 510  CONTINUE
      
      CALL KZBCR2('MCBHHIT',NWMBHH,FMSBHH,
     &     NCELLS,NSMBHH,FSMBHH,LSGBHH,FSGBHH,IERR)

      IDATA(1) = NCELLS
      CALL KZREP2('MCBHHIT',0,0,NWMBHH,IDATA)

*-- Store Data to given Segment of Bank MCBCHIT
      NCELLS = 0

      DO 520 ISEG = 1,NHCEL
         IF (BKHDT(2,ISEG).GT.EHDCUT) THEN
            NCELLS = NCELLS+1
            IF (NCELLS.GT.NPHCEL) THEN
               CALL MSGOUT('BCDIGI',
     &              'No. of HD cells overflow (full)',NCELLS)
               NCELLS = NPHCEL
            ENDIF
            IDATA(1)  = BKHDT(1,ISEG)
            RDATA(2)  = BKHDT(2,ISEG)
            RDATA(3)  = BKHDT(3,ISEG)
            RDATA(4)  = BKHDT(4,ISEG)
            IDATA(5)  = BKHDT(5,ISEG)
            CALL KZREP2('MCBHHIT',NCELLS,0,NSMBHH,IDATA)
            DO 521 IC = 1,5
               IDATA(1)  = BKHDT(4*IC+2,ISEG)
               RDATA(2)  = BKHDT(4*IC+3,ISEG)
               RDATA(3)  = BKHDT(4*IC+4,ISEG)
               RDATA(4)  = BKHDT(4*IC+5,ISEG)
               CALL KZREP2('MCBHHIT',NCELLS,IC,LSGBHH,IDATA)
 521        CONTINUE
            IHCELL(1,NCELLS) = IDATA(1)/1000 
            IHCELL(2,NCELLS) = MOD(IDATA(1),1000)
            EHCELL(NCELLS) = RDATA(2)+RDATA(3)
         ENDIF
 520  CONTINUE         

*-- for smearing DATA
      IF (ISIMJD(IDETBC).EQ.ISMRJD) THEN

*-- First count number of hits
         NCELLS = 0
         
         DO 530 I=1,NMXBHT
            DO 531 J=1,NMXBHP
*-- skip if no deposit            
               IF (EBCSMH(I,J).GT.0.) THEN
                  NCELLS = NCELLS+1
                  IF (NCELLS.GT.NPHCEL) THEN
                     CALL MSGOUT('BCDIGI',
     &                    'No. of HD cells overflow (smear)',NCELLS)
                     NCELLS = NPHCEL
                  ENDIF
               ENDIF
 531        CONTINUE 
 530     CONTINUE

*-- Create ZBS Bank MCBHSMH
         CALL KZBCR1('MCBHSMH',NWMBHH,FMSBHH,
     &        NCELLS,NSMBHH,FSGBHH,IERR)
         
         IDATA(1) = NCELLS
         CALL KZREP1('MCBHSMH',0,NWMBHH,IDATA)

*-- Store Data to given Segment of Bank MCBHSMH
         NCELLS = 0
         
         DO 540 I=1,NMXBHT
            DO 541 J=1,NMXBHP
*-- skip if no deposit            
               IF (EBCSMH(I,J).GT.0.) THEN
*-- fill 
                  NCELLS = NCELLS+1
                  IF (NCELLS.GT.NPHCEL) THEN
                     CALL MSGOUT('BCDIGI',
     &                    'No. of HD cells overflow (smear)',NCELLS)
                     NCELLS = NPHCEL
                  ENDIF
                  IDATA(1)  = I*1000+J
                  RDATA(2)  = EBCSMH(I,J)
                  RDATA(3)  = 0.
                  IDATA(4)  = 0
                  CALL KZREP1('MCBHSMH',NCELLS,NSMBHH,IDATA)
                  IHCELL(1,NCELLS) = -I
                  IHCELL(2,NCELLS) = -J
                  EHCELL(NCELLS) = RDATA(2)
               ENDIF
 541        CONTINUE 
 540     CONTINUE 
      ENDIF
      
      RETURN
      END
