      SUBROUTINE BCSUB2
*.
*...BCSUB2   Geomtry of the JLC barrel calorimeter.
*.                                 (DZ CONSTANT OPTION)
*.
*. INPUT     : none
*. OUTPUT    : none
*.
*. SEQUENCE  :
*. COMMON    : GCONST GCFLAG UCTMED BCGEO
*. CALLS     : GSVOLU GSROTM GSPOSP GSATT  GSPOS  GSDET  GSDETH    
*.             GPSETS
*. CALLED    : UGEOM
*.
*. BANKS L   :
*. BANKS U   :
*. BANKS M   :
*. BANKS D   :
*.
*. REPORT CONDITIONS
*.
*. AUTHOR    : Nakamura Isamu
*. VERSION   : 1.00
*. CREATED   : 13-Jun-1994
*. LAST MOD  : 21-Apr-1995
*.
*. MODIFICATION LOG
*.  21-Apr-1995 : Nakamura Isamu
*.                Geometry modification. (TRD2 >> GTRA)
*.                HIT Bank (JHIT) configuration is remaked
*.
*.  06-Mar-1996 : K.Ishii
*.                Modified the tilt angle.
*.  26-Feb-1999 : K.Ishii
*.                debug (I <-> I+1)
*.
*.**********************************************************************
*
*. Parameters ( User Define )
*.
*.     RINBC   : BCAL inner radius
*.     ZLBC    : BCAL Z length
*.     ZIBC    : Half Z length you want to cover.
*.     NDTHBC  : # of division in THETA direction
*.     NDPHBC  : # of division in PHI direction
*.     TSCEBC  : Tickness of Scintillator (EM)
*.     TSCHBC  : Tickness of Scintillator (HAD)
*.     BLBC    : BC Tower Block Length
*.     DEMBC   : Depth of EM part
*.
*.**********************************************************************
      IMPLICIT NONE

*-- Include GEANT Common Block
#include "geant321/gconst.inc"
#include "geant321/gcflag.inc"

#include "jmmed.inc"
#include "jmdet.inc"
#include "bcgeo.inc"
#include "bchit.inc"
#include "bcmed.inc"

      REAL DSIZE(11)
      REAL THETA(100),RTHETA(100)

*-- Define Detector Parameters
      CHARACTER*4 NMVE(NDSVBC),NMVH(NDSVBC) ,NMH(NDHTBC)
      CHARACTER*4 VNLG,VNSCNE,VNSCNH,VNABS
      INTEGER NBITV(NDSVBC),NBITH(NDHTBC)
      REAL ORIG(NDHTBC),FACT(NDHTBC)
      DATA NBITV  /     8,     8/
      DATA NMH    /'TOF ','E1  ','E2  ','E3  ','E4  '/
CKI      DATA NBITH  /   24 ,    24,    24,    24,    24/
      DATA NBITH  /   24 ,    32,    32,    32,    32/
      DATA ORIG   /    0.,    0.,    0.,    0.,    0./
CKI      DATA FACT   / 1.E12,  1.E4,  1.E4,  1.E4,  1.E4/
      DATA FACT   / 1.E12,  1.E8,  1.E8,  1.E8,  1.E8/

      REAL THEOFF,ROUTBC,RINBC2,RINBC3,THEMAX,DTHE,
     &     RDTHE,DPHI,RDPHI
      INTEGER NSAMPE,NSAMPH

      REAL DTHETA,DISTA2
      REAL PHI
      REAL R0,X0,Y0,Z0,R1,X1,Y1
      INTEGER IVOL,IROT
      INTEGER ISET,IDET

      INTEGER I,J

*-- Calculate some Parameters
      THEOFF    = 100.
      ROUTBC    = RINBC+BLBC+10.
      RINBC2    = RINBC+10.
      RINBC3    = RINBC+THEOFF
      NDTHBC    = INT(ZIBC*2./BSBC)
      THEMAX    = RADDEG*ATAN(FLOAT(NDTHBC)*BSBC/2/RINBC3)
      DTHE      = THEMAX/NDTHBC
      NDPHBC    = INT((2*PI*RINBC2/BSBC))
      RDTHE     = DEGRAD*DTHE
      DPHI      = 360./NDPHBC
      RDPHI     = DEGRAD*DPHI
      NSAMPE    = INT(DEMBC/(5*TSCEBC))
      NSAMPH    = INT((BLBC-DEMBC)/(5*TSCHBC))

*-- Define BC Master Volume 
      DSIZE(1)  = RINBC
      DSIZE(2)  = ROUTBC
      DSIZE(3)  = ZLBC/2.
      CALL GSVOLU('BCMA','TUBE',IAIRJM,DSIZE,3,IVOL)
      CALL GSATT('BCMA','SEEN',0)

*-- Define BC Sub Volume 
      DSIZE(1)  = RINBC
      DSIZE(2)  = ROUTBC
      DSIZE(3)  = ZLBC/2.
      DSIZE(4)  = 0.-DPHI/2.
      DSIZE(5)  = 0.+DPHI/2.
      CALL GSVOLU('BCP ','TUBS',IAIRJM,DSIZE,5,IVOL)
      CALL GSATT('BCP ','SEEN',0)

*-- Initialize Rotation Matrix Number. 5xxx are for BC.
      IROT = NROTJM*IDETBC

*-- Define Each Tower Polar Angle by DO LOOP.
      DO 10 I=1,NDTHBC+1
         RTHETA(I)  = ATAN(((I-1)*BSBC-FLOAT(NDTHBC)*BSBC/2.)/RINBC3)
         THETA(I)   = RADDEG*RTHETA(I)
 10   CONTINUE

*-- Define Each Tower Sturucture by DO LOOP.
      DO 20 I=1,NDTHBC
         DTHETA    = (THETA(I+1)-THETA(I))/2
         RDTHE     = DEGRAD*DTHETA
C        DISTAN    = RINBC2/COS(RTHETA(I)+RDTHE)
         DISTA2    = RINBC3/COS(RTHETA(I)+RDTHE)

*-- Calcurate Rotation Matrix
         IROT      = IROT+1
         CALL GSROTM(IROT,
     &        90.,90.,0.-THETA(I)-DTHETA,0.,90.-THETA(I)-DTHETA,0.)
*-- Define Detecter Volume Name
         WRITE(VNLG,'(''BL'',I2.2)') I
         WRITE(VNABS,'(''BA'',I2.2)') I
         WRITE(VNSCNE,'(''BE'',I2.2)') I
         WRITE(VNSCNH,'(''BH'',I2.2)') I
         
         NMVE(1) = 'BCP '
         NMVE(2) = VNABS
         
         NMVH(1) = 'BCP '
         NMVH(2) = VNABS

         VNEBC(I)= VNSCNE
         VNHBC(I)= VNSCNH
         VNLBC(I)= VNLG
         VNABC(I)= VNABS

*-- Define EM Scinti. part.
         CALL GSVOLU(VNSCNE,'TRAP',ISCIBC,0,0,IVOL)
         CALL GSATT(VNSCNE,'SEEN',0)

*-- Define HAD Scinti. part.
         CALL GSVOLU(VNSCNH,'TRAP',ISCIBC,0,0,IVOL)
         CALL GSATT(VNSCNH,'SEEN',0)

*-- Define BC Tower Structure Volume 
         DSIZE(1)  = BLBC/2.
         DSIZE(2)  = 0.
         DSIZE(3)  = 0.
         DSIZE(4)  = DISTA2*TAN(RDTHE)
         DSIZE(5)  = (DISTA2/COS(RDTHE)*COS(RTHETA(I))-THEOFF)
     &               *TAN(RDPHI/2.)
         DSIZE(6)  = (DISTA2/COS(RDTHE)*COS(RTHETA(I+1))-THEOFF)
     &               *TAN(RDPHI/2.)
         DSIZE(7)  = 0.
         DSIZE(8)  = (DISTA2+BLBC)*TAN(RDTHE)
         DSIZE(9)  = 
     &        ((DISTA2+BLBC)/COS(RDTHE)*COS(RTHETA(I))
     &        -THEOFF)*TAN(RDPHI/2.)
         DSIZE(10) = 
     &        ((DISTA2+BLBC)/COS(RDTHE)*COS(RTHETA(I+1))
     &        -THEOFF)*TAN(RDPHI/2.)
         DSIZE(11) = 0. 
         CALL GSVOLU(VNLG,'TRAP',ISCIBC,DSIZE,11,IVOL)
         CALL GSATT(VNLG,'SEEN',0)

*-- Define BC Absorber part.
         DSIZE(1)  = BLBC/2.     
         DSIZE(2)  = 0.
         DSIZE(3)  = 0.
         DSIZE(4)  = DISTA2*TAN(RDTHE)
         DSIZE(5)  = (DISTA2/COS(RDTHE)*COS(RTHETA(I))-THEOFF)
     &               *TAN(RDPHI/2.)-TLGBC
         DSIZE(6)  = (DISTA2/COS(RDTHE)*COS(RTHETA(I+1))-THEOFF)
     &               *TAN(RDPHI/2.)-TLGBC
         DSIZE(7)  = 0.
         DSIZE(8)  = (DISTA2+BLBC)*TAN(RDTHE)
         DSIZE(9)  = 
     &        ((DISTA2+BLBC)/COS(RDTHE)*COS(RTHETA(I))
     &        -THEOFF)*TAN(RDPHI/2.)-TLGBC
         DSIZE(10) = 
     &        ((DISTA2+BLBC)/COS(RDTHE)*COS(RTHETA(I+1))
     &        -THEOFF)*TAN(RDPHI/2.)-TLGBC
         DSIZE(11) = 0. 
         CALL GSVOLU(VNABS,'TRAP',ILEDBC,DSIZE,11,IVOL)
         Z0        = 0.
         CALL GSPOS(VNABS,I,VNLG,0.,0.,Z0,0.,'ONLY')
C         CALL GSATT(VNSCNE,'SEEN',0)

*-- Store Block Size into COMMON
         SIZBC(1,I) = DSIZE(4)
         SIZBC(2,I) = DSIZE(5)
         SIZBC(3,I) = DSIZE(6)
         SIZBC(4,I) = DSIZE(8)
         SIZBC(5,I) = DSIZE(9)
         SIZBC(6,I) = DSIZE(10)
         SIZBC(7,I) = DSIZE(1)

*-- Put the COPY of EM Scintillator Plate into BC Tower
         DO 40 J=1,NSAMPE
            DSIZE(1)  = TSCEBC/2.
            DSIZE(2)  = 0.
            DSIZE(3)  = 0.
            DSIZE(4)  = (DISTA2+(5.*J-1)*TSCEBC)*TAN(RDTHE)
            DSIZE(5)  = 
     &           ((DISTA2+(5.*J-1)*TSCEBC)/COS(RDTHE)*COS(RTHETA(I))
     &           -THEOFF)*TAN(RDPHI/2.)-TLGBC
            DSIZE(6)  = 
     &           ((DISTA2+(5.*J-1)*TSCEBC)/COS(RDTHE)*COS(RTHETA(I+1))
     &           -THEOFF)*TAN(RDPHI/2.)-TLGBC
            DSIZE(7)  = 0.
            DSIZE(8)  = (DISTA2+5.*J*TSCEBC)*TAN(RDTHE)
            DSIZE(9)  = 
     &           ((DISTA2+5.*J*TSCEBC)/COS(RDTHE)*COS(RTHETA(I))
     &           -THEOFF)*TAN(RDPHI/2.)-TLGBC
            DSIZE(10) = 
     &           ((DISTA2+5.*J*TSCEBC)/COS(RDTHE)*COS(RTHETA(I+1))
     &           -THEOFF)*TAN(RDPHI/2.)-TLGBC
            DSIZE(11) = 0. 
            Z0        = (-1.)*BLBC/2.+(5.*J-.5)*TSCEBC
            CALL GSPOSP(VNSCNE,J,VNABS,0.,0.,Z0,0.,'ONLY',DSIZE,11)
            IF (J.EQ.1) THEN
               SURFBC(1,1,I) = DSIZE(4)
               SURFBC(2,1,I) = DSIZE(5)
               SURFBC(3,1,I) = DSIZE(6)
               SURFBC(4,1,I) = Z0 - 0.5*TSCEBC
            ENDIF
 40         CONTINUE
C           ZSURF = (-1.)*BLBC/2.+4.*TSCEBC

*-- Put the COPY of HAD Scintillator Plate into BC Tower
         DO 50 J=1,NSAMPH
            DSIZE(1)  = TSCHBC/2.
            DSIZE(2)  = 0.
            DSIZE(3)  = 0.
            DSIZE(4)  = (DISTA2+DEMBC+(5*J-1)*TSCHBC)*TAN(RDTHE)
            DSIZE(5)  = 
     &           ((DISTA2+DEMBC+(5.*J-1)*TSCHBC)/COS(RDTHE)
     &           *COS(RTHETA(I))-THEOFF)*TAN(RDPHI/2.)-TLGBC
            DSIZE(6)  = 
     &           ((DISTA2+DEMBC+(5.*J-1)*TSCHBC)/COS(RDTHE)
     &           *COS(RTHETA(I+1))-THEOFF)*TAN(RDPHI/2.)-TLGBC
            DSIZE(7)  = 0.
            DSIZE(8)  = (DISTA2+DEMBC+5*J*TSCHBC)*TAN(RDTHE)
            DSIZE(9)  = 
     &           ((DISTA2+DEMBC+5.*J*TSCHBC)/COS(RDTHE)
     &           *COS(RTHETA(I))-THEOFF)*TAN(RDPHI/2.)-TLGBC
            DSIZE(10) = 
     &           ((DISTA2+DEMBC+5.*J*TSCHBC)/COS(RDTHE)
     &           *COS(RTHETA(I+1))-THEOFF)*TAN(RDPHI/2.)-TLGBC
            DSIZE(11) = 0. 
            Z0        = (-1)*BLBC/2.+DEMBC+(5*J-.5)*TSCHBC
            CALL GSPOSP
     &           (VNSCNH,J+NSAMPE,VNABS,0.,0.,Z0,0,'ONLY',DSIZE,11)
            IF (J.EQ.1) THEN
               SURFBC(1,2,I) = DSIZE(4)
               SURFBC(2,2,I) = DSIZE(5)
               SURFBC(3,2,I) = DSIZE(6)
               SURFBC(4,2,I) = Z0 - 0.5*TSCHBC
            ENDIF
 50      CONTINUE         

*-- Put the BC Tower Structure into BC sub Volume.
         X0       = RINBC2+BLBC/2.*COS(RTHETA(I)+RDTHE)
         Y0       = 0.
         Z0       = (DISTA2+BLBC/2)*SIN(RTHETA(I)+RDTHE)
         CALL GSPOS(VNLG,I,'BCP ',X0,Y0,Z0,IROT,'ONLY')

*-- Define Detector
         CALL GSDET
     &    ('BC  ',VNSCNE,NDSVBC,NMVE,NBITV,1,NHMXBC,0,ISET,IDET)
         CALL GSDET
     &    ('BC  ',VNSCNH,NDSVBC,NMVH,NBITV,1,NHMXBC,0,ISET,IDET)
         CALL GSDETH('BC  ',VNSCNE,NDHTBC,NMH,NBITH,ORIG,FACT)
         CALL GSDETH('BC  ',VNSCNH,NDHTBC,NMH,NBITH,ORIG,FACT)
         
 20   CONTINUE

*-- Put the COPY of BC sub Volume into BC Master Volume.
      DO 30 J=1,NDPHBC
         PHI      = (J-1)*DPHI+DPHI/2.
C        RPHI     = PHI*DEGRAD
         IROT     = IROT+1
         R0       = (ROUTBC+RINBC2)/2.
C        R2       = R0/COS(DEGRAD*PTLTBC)
         R1       = 2*R0*SIN(DEGRAD*(PTLTBC/2.))
         X1       = R0*(1./COS(DEGRAD*PTLTBC)-1.)*COS(DEGRAD*PHI)
         Y1       = R0*(1./COS(DEGRAD*PTLTBC)-1.)*SIN(DEGRAD*PHI)
C        X2       = R0*(1./COS(DEGRAD*PTLTBC)-1.)
C        Y2       = R0*(1./COS(DEGRAD*PTLTBC)-1.)
         X0       = R1*COS(DEGRAD*(PTLTBC/2.-90+PHI))+X1
         Y0       = R1*SIN(DEGRAD*(PTLTBC/2.-90+PHI))+Y1
C         R0       = (ROUTBC-RINBC)/2.
C         X0       = R0*(COS(DEGRAD*(PHI+PTLTBC))-COS(DEGRAD*PHI))
C         Y0       = R0*(SIN(DEGRAD*(PHI+PTLTBC))-SIN(DEGRAD*PHI))
         CALL GSROTM(IROT,90.,PHI+PTLTBC,90.,90+PHI+PTLTBC,180.,0.)
         CALL GSPOS('BCP ',J,'BCMA',X0,Y0,0.,IROT,'ONLY')
C         WRITE(*,*) X2,R0
 30   CONTINUE
C
C Put BC into Master Volume.
C
      CALL GSPOS('BCMA',1,'MAST',0.,0.,0.,0,'ONLY')

      RETURN
      END
