      SUBROUTINE BCSUB3
*.
*...BCSUB1   Geomtry of the JLC barrel calorimeter.
*.                                    (BLOCK SIZE CONSTANT OPTION)
*.
*. INPUT     : none
*. OUTPUT    : none
*.
*. SEQUENCE  :
*. COMMON    : GCONST
*. CALLS     : GSVOLU GSPOS GSPOSP GSDVN  GSDET  GSDETH GSDETD GPSETS
*. CALLED    : UGEOM
*.
*. BANKS L   :
*. BANKS U   :
*. BANKS M   :
*. BANKS D   :
*.
*. REPORT CONDITIONS
*.
*. AUTHOR    : Nakamura Isamu
*. VERSION   : 1.00
*. CREATED   : 13-Jun-1994
*. LAST MOD  : 
*.
*. MODIFICATION LOG
*.
*.**********************************************************************
*
*. Parameters ( User Define )
*.
*.     RINBC   : BCAL inner radius
*.     ZLBC    :      length
*.     ZIBC    : Half Z length you want to cover.
*.     NDTHBC  : # of division in THETA direction
*.     NDPHBC  :               in PHI direction
*.     TSCEBC  : Tickness of Scintillator (EM)
*.     TSCHBC  : Tickness of Scintillator (HAD)
*.     BLBC    : BC Tower Block Length
*.     DEMBC   : Depth of EM part
*.
*.**********************************************************************
      IMPLICIT NONE

*-- Include GEANT include file
#include "geant321/gconst.inc"
#include "geant321/gcflag.inc"

#include "jmmed.inc"
#include "jmdet.inc"
#include "bcgeo.inc"
#include "bchit.inc"
#include "bcmed.inc"

      REAL DSIZE(11)

*-- Define Detector Parameters
      CHARACTER*4 NMVE(NDSVBC),NMVH(NDSVBC) ,NMH(NDHTBC),CHNMSD
      CHARACTER*4 VNLG,VNSCNE,VNSCNH
      INTEGER NBITV(NDSVBC),NBITH(NDHTBC)
      REAL ORIG(NDHTBC),FACT(NDHTBC)
      INTEGER NBITSD
      DATA NBITV  /    16,    16/
      DATA NMH    /'X   ','Y   ','Z   ','E   ','ELOS'/
      DATA NBITH  /    16,    16,    16,    24,   24/
      DATA ORIG   /10000.,10000.,10000.,    0.,    0./
      DATA FACT   /    1.,    1.,    1., 1000., 1000./
      DATA CHNMSD /'ADC '/
      DATA NBITSD /    16/

      REAL ROUTBC,RINBC2,RINBC3,THEMAX,DPHI,RDPHI
      INTEGER NSAMPE,NSAMPH

      REAL THETA,DTHE,ODTHE,RDTHE,RTHETA,DISTAN,DISTA2
      REAL PHI
      REAL X0,Y0,Z0
      INTEGER IROT,IVOL
      INTEGER ISET,IDET

      INTEGER I,J

*-- Calculate some parameters
      ROUTBC    = RINBC+BLBC+10.
      RINBC2    = RINBC+5.
      RINBC3    = RINBC+100.
      THEMAX    = RADDEG*ATAN(ZIBC/RINBC3)
      NDPHBC    = INT((2*PI*RINBC2/BSBC))
      DPHI      = 360./NDPHBC
      RDPHI     = DEGRAD*DPHI
      NSAMPE    = INT(DEMBC/(5*TSCEBC))
      NSAMPH    = INT((BLBC-DEMBC)/(5*TSCHBC))

*-- Define BC master volume 
      DSIZE(1)  = RINBC
      DSIZE(2)  = ROUTBC
      DSIZE(3)  = ZLBC/2.
      CALL GSVOLU('BCMA','TUBE',IAIRJM,DSIZE,3,IVOL)
      CALL GSATT('BCMA','SEEN',0)

*--  Define BC sub volume 
      DSIZE(1)  = RINBC
      DSIZE(2)  = ROUTBC
      DSIZE(3)  = ZLBC/4.
      DSIZE(4)  = 0.-DPHI/2.
      DSIZE(5)  = 0.+DPHI/2.
      CALL GSVOLU('BCP ','TUBS',IAIRJM,DSIZE,5,IVOL)
      CALL GSATT('BCP ','SEEN',0)

*-- Define EM and HAD Pb part.
      CALL GSVOLU('BCAB','TRD2',ILEDBC,0,0,IVOL)
      CALL GSATT('BCAB','SEEN',0)

*-- Initialize COPY Number and Rotation Matrix Number. 
*-- IROT=5xxx are for BC
      I         = 0
      IROT      = NROTJM*IDETBC
      DTHE      = 0.
      THETA     = 0.
      RTHETA    = 0.
      DISTAN    = RINBC2
      DISTA2    = RINBC3

*-- Define Each Tower Sturucture by LOOP.
 10   CONTINUE

*-- Cheking Current Theta Angle.
      IF (THETA.GE.THEMAX) GO TO 50

*-- Calculate Parameters
      I         = I+1
      NDTHBC    = I
      ODTHE     = DTHE
      DTHE      = 2*RADDEG*ATAN(BSBC/2./DISTA2)
      IF (I.EQ.1) THEN
         THETA     = DTHE/2.
      ELSE
         THETA     = THETA+DTHE/2.+ODTHE/2.
      ENDIF
      RTHETA    = DEGRAD*THETA
      DISTAN    = RINBC2/COS(RTHETA)
      DISTA2    = RINBC3/COS(RTHETA)
      RDTHE     = DEGRAD*DTHE

*-- Calculate Rotation Matrix
      IROT      = IROT+1
      CALL GSROTM(IROT,90.,90.,0.-THETA,0.,90.-THETA,0.)

*-- Define Volume Name
      WRITE(VNLG,'(''BL'',I2.2)') I
      WRITE(VNSCNE,'(''BE'',I2.2)') I
      WRITE(VNSCNH,'(''BH'',I2.2)') I

      NMVE(1) = 'BCP '
      NMVE(2) = VNSCNE
      NMVH(1) = 'BCP '
      NMVH(2) = VNSCNH

      VNEBC(I) = VNSCNE
      VNHBC(I) = VNSCNH
      VNLBC(I) = VNLG

*-- Define BC Tower Structure Volume 
      DSIZE(1) = DISTAN*TAN(RDPHI/2.)
      DSIZE(2) = (DISTAN+BLBC)*TAN(RDPHI/2.)
      DSIZE(3) = DISTA2*TAN(RDTHE/2.)
      DSIZE(4) = (DISTA2+BLBC)*TAN(RDTHE/2.)
      DSIZE(5) = BLBC/2.
      CALL GSVOLU(VNLG,'TRD2',ISCIBC,DSIZE,5,IVOL)
      CALL GSATT(VNLG,'SEEN',0)

*-- Define BC EM Scintillator part.
      DSIZE(1)  = DISTAN*TAN(RDPHI/2.)-TLGBC
      DSIZE(2)  = (DISTAN+DEMBC)*TAN(RDPHI/2.)-TLGBC
      DSIZE(3)  = DISTA2*TAN(RDTHE/2.)
      DSIZE(4)  = (DISTA2+DEMBC)*TAN(RDTHE/2.)
      DSIZE(5)  = DEMBC/2.     
      CALL GSVOLU(VNSCNE,'TRD2',ISCIBC,DSIZE,5,IVOL)
      Z0        = -BLBC/2.+DEMBC/2.
      CALL GSPOS(VNSCNE,I,VNLG,0.,0.,Z0,0.,'ONLY')
C     CALL GSATT(VNSCNE,'SEEN',0)

*-- Define BC HAD Scintillator part.
      DSIZE(1)  = (DISTAN+DEMBC)*TAN(RDPHI/2.)-TLGBC
      DSIZE(2)  = (DISTAN+BLBC)*TAN(RDPHI/2.)-TLGBC
      DSIZE(3)  = (DISTA2+DEMBC)*TAN(RDTHE/2.)
      DSIZE(4)  = (DISTA2+BLBC)*TAN(RDTHE/2.)
      DSIZE(5)  = (BLBC-DEMBC)/2.     
      CALL GSVOLU(VNSCNH,'TRD2',ISCIBC,DSIZE,5,IVOL)
      Z0        = -BLBC/2.+DEMBC+(BLBC-DEMBC)/2.
      CALL GSPOS(VNSCNH,I,VNLG,0.,0.,Z0,0.,'ONLY')
C      CALL GSATT(VNSCNH,'SEEN',0)

*-- Put the Pb Absorber into EM Scinti.
      DO 30 J=1,NSAMPE
         DSIZE(1)  = (DISTAN+(5*(J-1)*TSCEBC))*TAN(RDPHI/2.)-TLGBC
         DSIZE(2)  = (DISTAN+(5*J-1)*TSCEBC)*TAN(RDPHI/2.)-TLGBC
         DSIZE(3)  = (DISTA2+(5*(J-1)*TSCEBC))*TAN(RDTHE/2.)
         DSIZE(4)  = (DISTA2+(5*J-1)*TSCEBC)*TAN(RDTHE/2.)
         DSIZE(5)  = TSCEBC*2.
         Z0        = (-1)*DEMBC/2.+(5*J-2)*TSCEBC
         CALL GSPOSP('BCAB',J,VNSCNE,0.,0.,Z0,0,'ONLY',DSIZE,5)
 30   CONTINUE

*-- Put the Pb Absorber into HAD Scinti.
      DO 40 J=1,NSAMPH
         DSIZE(1)  = 
     &        (DISTAN+DEMBC+(5*(J-1)*TSCHBC))*TAN(RDPHI/2.)-TLGBC
         DSIZE(2)  = 
     &        (DISTAN+DEMBC+(5*J-1)*TSCHBC)*TAN(RDPHI/2.)-TLGBC
         DSIZE(3)  = (DISTAN+DEMBC+(5*(J-1)*TSCHBC))*TAN(RDTHE/2.)
         DSIZE(4)  = (DISTAN+DEMBC+(5*J-1)*TSCHBC)*TAN(RDTHE/2.)
         DSIZE(5)  = TSCHBC*2.
         Z0        = (-1)*(BLBC-DEMBC)/2.+(5*J-2)*TSCHBC
         CALL GSPOSP('BCAB',J,VNSCNH,0.,0.,Z0,0,'ONLY',DSIZE,5)
 40   CONTINUE         

*-- Put the BC Tower Structure into BC sub Volume.
      X0       = RINBC2+BLBC/2.*COS(RTHETA)
      Y0       = 0.
      Z0       = (RINBC3+BLBC/2.)*TAN(RTHETA)-ZLBC/4.
      CALL GSPOS(VNLG,I,'BCP ',X0,Y0,Z0,IROT,'ONLY')

*-- Set GEANT Detector & Hit 
      CALL GSDET
     &     ('BC  ',VNSCNE,NDSVBC,NMVE,NBITV,1,NHMXBC,0,ISET,IDET)
      CALL GSDET
     &     ('BC  ',VNSCNH,NDSVBC,NMVH,NBITV,1,NHMXBC,0,ISET,IDET)
      CALL GSDETH('BC  ',VNSCNE,NDHTBC,NMH,NBITH,ORIG,FACT)
      CALL GSDETH('BC  ',VNSCNH,NDHTBC,NMH,NBITH,ORIG,FACT)

      GO TO 10

 50   CONTINUE

*-- Put the COPY of BC sub Volume into BC Master Volume.
      DO 20 J=1,NDPHBC
         PHI      = (J-1)*DPHI+DPHI/2.
C        RPHI     = PHI*DEGRAD
         IROT     = IROT+2
         Z0       = ZLBC/4.
         CALL GSROTM(IROT-1,90.,PHI+PTLTBC,90.,90+PHI+PTLTBC,0.,0.)
         CALL GSROTM(IROT,90.,PHI+PTLTBC,90.,90+PHI+PTLTBC,180.,0.)
         CALL GSPOS('BCP ',2*J-1,'BCMA',0.,0.,Z0,IROT-1,'ONLY')
         CALL GSPOS('BCP ',2*J,'BCMA',0.,0.,0.-Z0,IROT,'ONLY')
 20   CONTINUE
      CALL GSPOS('BCMA',1,'MAST',0.,0.,0.,0,'ONLY')

      RETURN
      END
