      SUBROUTINE BMDIGI
*.
*...BMDIGI   Compute and store DIGIts for Si vertex detector
*.
*. INPUT     : none
*. OUTPUT    : none
*.
*. SEQUENCE  :
*. COMMON    : BMGEO  GCNUM
*. CALLS     : GFHITS KZBCR1 KZPUT1 JGP2LP
*. CALLED    : GUDIGI
*.
*. BANKS L   : MCBMHIT
*. BANKS U   :
*. BANKS M   :
*. BANKS D   :
*.
*. REPORT CONDITIONS
*.
*. AUTHOR    : T. Tauchi
*. VERSION   : 1.00
*. CREATED   : 29-Aug-1996
*. LAST MOD  : 
*.
*. MODIFICATION LOG
*. 28-Jan-1997 T.Tauchi; add Px,Py,Pz in hits#11-13.
*.
*. Bank Data Structure ;
*. name = MCBMHIT
*.        First Two Words #1  total number of hits
*.                        #2  total number of tracks
*.      
*.         21 Words/Hit   #1  track number responsible to this hit
*.                        #2  detector/layer(1F,2F,1B,2B) number of this hit
*.                        #3  1,2  for +z amd -z position, respectively
*.                        #4  pixel number in theta direction
*.                        #5  pixel number in phai direction
*.                        #6  particle number
*.                        #7  x coordinate of this hit [cm]
*.                        #8  y coordinate of this hit [cm]
*.                        #9  z coordinate of this hit [cm]
*.                        #10 p of particle generated this hit [GeV]
*.                        #11 Px of particle [GeV]
*.                        #12 Py of particle [GeV]
*.                        #13 Pz of particle [GeV]
*.                        #14  particle number of previous track
*.                        #15  vertex position in x
*.                        #16  vertex position in y
*.                        #17  vertex position in z
*.                        #18  momentum-x at vertex position
*.                        #19  momentum-y at vertex position
*.                        #20  momentum-z at vertex position
*.                        #21 energy deposit to this pixel [keV]
*.
*.**********************************************************************
      IMPLICIT NONE

*-- Common
#include "geant321/gcnum.inc"

#include "bmgeo.inc"
#include "bmhit.inc"
      
      INTEGER NDATMX
      PARAMETER (NDATMX=21)
      INTEGER NUMVS(NDSVBM),NUMBV(NDSVBM,NHMXBM),ITRA(NHMXBM)
      REAL HITS(NDHTBM,NHMXBM)

      INTEGER IDATA(NDATMX)
      REAL    RDATA(NDATMX)
      EQUIVALENCE (IDATA(1), RDATA(1))
      CHARACTER*32 CNABMH,CFMBMH,CFSBMH
      INTEGER NWMBMH,NWPBMH
*-- Parameter for bank
      PARAMETER (CNABMH='MCBMHIT')
      PARAMETER (NWMBMH=2)
      PARAMETER (CFMBMH='2I')
      PARAMETER (CFSBMH='5I 7F 1I 7F')
      PARAMETER (NWPBMH=0)

      CHARACTER*4 VNAMBM
      
      INTEGER NUMHIT
      INTEGER NHITS,IHIT,IHT
      INTEGER IDET

      INTEGER ILPC

      INTEGER I,J,IERR


*-- Initialize
      DO 10 I=1,NDSVBM
         NUMVS(I)=0
 10   CONTINUE

      NUMHIT=0
      CALL GFHITS('BM  ','BM1P',NDSVBM,NDHTBM,NHMXBM,0,NUMVS,
     &     ITRA,NUMBV,HITS,NHITS)
      IF (NHITS.GT.NHMXBM) THEN
         CALL MSGOUT('BMDIGI','Nhits overflow for BM1',NHMXBM)
         NHITS = NHITS-1
      ENDIF
      NUMHIT = NUMHIT+NHITS
      CALL GFHITS('BM  ','BM2P',NDSVBM,NDHTBM,NHMXBM,0,NUMVS,
     &     ITRA,NUMBV,HITS,NHITS)
      IF (NHITS.GT.NHMXBM) THEN
         CALL MSGOUT('BMDIGI','Nhits overflow for BM2',NHMXBM)
         NHITS = NHITS-1
      ENDIF
      NUMHIT = NUMHIT+NHITS
      
*-- Create new ZBS bank with the 1-dim. fixed length segment.
      CALL KZBCR1('MCBMHIT',NWMBMH,CFMBMH,NUMHIT,NDATMX,CFSBMH, IERR)

      IHIT = 0
      
      DO 100 IDET=1,2
         IF (IDET.EQ.1) THEN
            VNAMBM = 'BM1P'
         ELSE
            VNAMBM = 'BM2P'
         ENDIF
         CALL GFHITS('BM  ',VNAMBM,NDSVBM,NDHTBM,NHMXBM,0,NUMVS,
     &     ITRA,NUMBV,HITS,NHITS)
      IF (NHITS.GT.NHMXBM) THEN
         NHITS = NHMXBM
      ENDIF

*-- Loop of hit in this detector name
      DO 110 IHT=1,NHITS
         IHIT = IHIT+1
         IDATA(1) = ITRA(IHT)
         IDATA(2) = IDET
         IDATA(3) = NUMBV(1,IHT)
         IDATA(4) = NUMBV(2,IHT)
         IDATA(5) = NUMBV(3,IHT)

*-- Convert geant p-code to LUND74 P-code
         CALL JGP2LP(NINT(HITS(1,IHT)), ILPC,IERR)
         IF (IERR.EQ.0) THEN
            IDATA(6) = ILPC
         ELSE
            IDATA(6) = 9999
         ENDIF
         CALL JGP2LP(NINT(HITS(9,IHT)), ILPC,IERR)
         IF (IERR.EQ.0) THEN
            IDATA(14) = ILPC
         ELSE
            IDATA(14) = 9999
         ENDIF
         DO 120 J=7,NDATMX
            IF (J.NE.14) RDATA(J) = HITS(J-5,IHT)
 120     CONTINUE
CCCC         WRITE(6,*) 'BMDIGI:JGP2LP; IDATA(1-5 )=',(IDATA(II),II=1,5)
CCCC         WRITE(6,*) 'BMDIGI:JGP2LP; IDATA(6-10)=',(RDATA(II),II=6,10)

*-- Put data words into the given segment of the bank
c         print *,'mcbmhit;ihit = ',ihit
c         print *,'       ;idata = ',(idata(j),j=1,5)
c         print *,'       ;idata = ',idata(6),idata(14)
c         print *,'       ;rdata = ',(rdata(j),j=7,13)
c         print *,'       ;rdata = ',(rdata(j),j=15,21)
         CALL KZPUT1('MCBMHIT',IHIT,NWPBMH,NDATMX,IDATA)

 110  CONTINUE 

 100  CONTINUE 

*-- Put data words into the master segment of the bank
      IDATA(1) = NUMHIT
      IDATA(2) = NTRACK
      CALL KZPUT1('MCBMHIT',0,0,NWMBMH,IDATA)

      RETURN
      END
