      SUBROUTINE BMGEOM
*.
*...BMGEOM   Define beam profile monitor
*.           geometry and tracking media.
*.
*. INPUT     : non
*. OUTPUT    : non
*.
*. SEQUENCE  :
*. COMMON    : GCFLAG GCONST BMGEO UCTMED
*. CALLS     : GSVOLU GSDVN  GSPOS GSDET  GSDETH GSDETD GPSETS
*. CALLED    : UGEOM
*.
*. BANKS L   :
*. BANKS U   :
*. BANKS M   :
*. BANKS D   :
*.
*. REPORT CONDITIONS
*.
*. AUTHOR    : T. Tauchi
*. VERSION   : 1.00
*. CREATED   : 29-Aug-1996
*. LAST MOD  : 
*.
*. MODIFICATION LOG
*. 12-Jan-1998 Y.Fujii ; Added jmmat.inc and jmdet.inc
*.                       Added ISIMJD(IDETBM) judgement
*.                       Added GSTMED for silicon.
*.**********************************************************************
      IMPLICIT NONE

*--   Common for GEANT
#include "geant321/gcflag.inc"
#include "geant321/gconst.inc"

*--   Common for jim
#include "jmmed.inc"
#include "jmdet.inc"
#include "jmmat.inc"
#include "jmrot.inc"
#include "bmgeo.inc"
#include "bmhit.inc"
      
      INTEGER ISILI
      
*-- Tracking media parameters for Si
      REAL STEMXS,EPSILS,STMINS
      PARAMETER ( STEMXS=0.01 )
      PARAMETER ( EPSILS=0.0001)
      PARAMETER ( STMINS=0.0001)
      
      REAL BMTK
      
      REAL PAR(12)

      CHARACTER*4 CHNM1(NDSVBM),CHNM2(NDSVBM)
      INTEGER NBITSV(NDSVBM)
      DATA CHNM1/'BM1 ','BM1R','BM1P'/
      DATA CHNM2/'BM2 ','BM2R','BM2P'/
      DATA NBITSV /    8,    32,   32/

      CHARACTER*4 CHNAMH(NDHTBM)
      INTEGER NBITSH(NDHTBM)
      REAL ORIG(NDHTBM),FACT(NDHTBM)
      DATA CHNAMH /  'PCOD','X   ','Y   ','Z   ','P   ',
     &     'Px  ','Py  ','Pz  ',
     &     'PCDM','Xm  ','Ym  ','Zm  ','Px-m','Py-m','Pz-m','ELOS'/
      DATA NBITSH/ 1*16,3*32,32,3*32,1*16,6*32,32 /
      DATA ORIG/ 30000.,3*200.,0.,3*500.,30000.,3*500.,3*500.,0./
      DATA FACT/ 1., 3*1.E6,1.E6, 3*1.E6,1.,3*1.E6,3*1.E6,1.E6/
      
      INTEGER IVOL,IROT
      INTEGER IDET,ISET

      INTEGER NDIV


      IF (ISIMJD(IDETBM).EQ.ISKPJD) RETURN

      ISILI = NMEDJM*IDETBM+1
      CALL GSTMED(ISILI,'SILICONBM',IMSILI,ISENS,IFIELD, FIELDM,
     &     TMAXFD, STEMXS, DEEMAX, EPSILS,STMINS,UBUF, NWBUF)

*--   Define a volume with a given name, shape, tracking medium number 
*--   and shape parameters
*--   BM master
      BMTK  =SITKBM*0.5
      PAR(1)=RINNBM(1)
      PAR(2)=ROUTBM(1)
      PAR(3)=BMTK      
      CALL GSVOLU('BM1 ', 'TUBE', ISILI, PAR, 3, IVOL)
C      CALL GSVOLU('BM3 ', 'TUBE', ISILI, PAR, 3, IVOL)

      PAR(1)=RINNBM(2)
      PAR(2)=ROUTBM(2)
      PAR(3)=BMTK      
      CALL GSVOLU('BM2 ', 'TUBE', ISILI, PAR, 3, IVOL)
C      CALL GSVOLU('BM4 ', 'TUBE', ISILI, PAR, 3, IVOL)

*--   Division of a volume into a given number of cells
*--   Divide cells for r-direction
      NDIV=NINT((ROUTBM(1)-RINNBM(1))/PXCLBM(2))
      NDIVBM(1,1) = NDIV
      CALL GSDVN('BM1R', 'BM1 ', NDIV, 1)
*--   Divide cells for phi-direction
      NDIV=NINT(2.0*PI*RINNBM(1)/PXCLBM(1))
      NDIVBM(2,1) = NDIV
      CALL GSDVN('BM1P', 'BM1R', NDIV, 2)
*--   Divide cells for r-direction
      NDIV=NINT((ROUTBM(2)-RINNBM(2))/PXCLBM(2))
      NDIVBM(1,2) = NDIV
      CALL GSDVN('BM2R', 'BM2 ', NDIV, 1)
*--   Divide cells for phi-direction
      NDIV=NINT(2.0*PI*RINNBM(2)/PXCLBM(1))
      NDIVBM(2,2) = NDIV
      CALL GSDVN('BM2P', 'BM2R', NDIV, 2)
*-- place the detectors
C     IROT = NROTJM*IDETBM+1
C     CALL GSROTM(IROT,90.,0.,90.,90.,180.,0.)
      CALL GSPOS('BM1 ', 1, 'MAST', 0.0, 0.0,  ZBM(1), 0, 'ONLY')
      CALL GSPOS('BM1 ', 2, 'MAST', 0.0, 0.0, -ZBM(1), IRMIRR, 'ONLY')
      CALL GSPOS('BM2 ', 1, 'MAST', 0.0, 0.0,  ZBM(2), 0, 'ONLY')
      CALL GSPOS('BM2 ', 2, 'MAST', 0.0, 0.0, -ZBM(2), IRMIRR, 'ONLY')

*-- Assign detector to the set BM and defines its basic params.
      CALL GSDET ('BM  ','BM1P',NDSVBM,CHNM1,NBITSV,
     &     NDETJM*IDETBM+1,NHMXBM,0,ISET,IDET)
      CALL GSDET ('BM  ','BM2P',NDSVBM,CHNM2,NBITSV,
     &     NDETJM*IDETBM+2,NHMXBM,0,ISET,IDET)

*-- Define hit parameters for detector of set BM.
      CALL GSDETH('BM  ','BM1P',NDHTBM,CHNAMH,NBITSH,ORIG,FACT)
      CALL GSDETH('BM  ','BM2P',NDHTBM,CHNAMH,NBITSH,ORIG,FACT)

*-- Print SET and DETector parameters
      IF (ISWIT(3).GE.1) THEN
        CALL GPSETS('BM  ','*   ')
      ENDIF

*--   Change the attribute of the volume called NAME to the value
*--   all volume set to seen
      CALL GSATT('BM1 ','SEEN',-2)
      CALL GSATT('BM2 ','SEEN',-2)
C      CALL GSATT('BM3 ','SEEN',-2)
C      CALL GSATT('BM4 ','SEEN',-2)

      RETURN
      END
