      SUBROUTINE CTGEOM
*.
*...CTGEOM   Define the central tracker 
*.           geometry and tracking media.
*.
*. INPUT     : none
*. OUTPUT    : none
*.
*. SEQUENCE  :
*. COMMON    : GCFLAG UCTMED CTGEO  GCONST
*. CALLS     : GSVOLU GSROTM GSPOS  GSDET  GSDETH GPSETS GSATT
*. CALLED    : UGEOM
*.
*. BANKS L   :
*. BANKS U   :
*. BANKS M   :
*. BANKS D   :
*.
*. REPORT CONDITIONS
*.
*. AUTHOR    : Naoyasu HOSODA
*. VERSION   : 1.00
*. CREATED   : 11-Mar-1994
*. LAST MOD  :  2-Mar-1999
*.
*. MODIFICATION LOG
*. 21-Apr-1999 J.Kanzaki, detailed version which include Tauchi's
*.             extension of hit bank is available
*. 11-Mar-1999  J.Kanzaki  Change default to use single anode layer cell
*.                         for sensitive detector
*.  2-Mar-1999  Nakamura  Change HIT bank
*. 20-Sep-1996  Y.Fujii ; Reduced cell size to avoid overlapping and
*.                      invading of cell volumes.
*. 27-Jul-1994  N.Hosoda; Remake all
*.
*.**********************************************************************
      IMPLICIT NONE

*--   Common for GEANT
#include "geant321/gcflag.inc"
#include "geant321/gconst.inc"

#include "jmfile.inc"
#include "jmdet.inc"
#include "jmmat.inc"
#include "jmmed.inc"
#include "ctgeo.inc"
#include "ctsim.inc"
#include "cthit.inc"
      
      INTEGER IALUM,ICO2I,ICFRP

      REAL PAR(15)
      CHARACTER*4 VNAMC,VNAME,VNAMD(NPANCT),VNAMG,VNAMT

      CHARACTER*4 CHNMSV(NDSVCT)
      INTEGER NBITSV(NDSVCT)
      DATA NBITSV /    16,    16,    16/

      CHARACTER*4 CHNAMH(NPHTCT)
      INTEGER NBITSH(NPHTCT)
      REAL    ORIG(NPHTCT),FACT(NPHTCT)
      DATA CHNAMH 
     &    /'XI  ','YI  ','ZI  ','PXI ','PYI ','PZI ','TOFI',
     &     'XO  ','YO  ','ZO  ','PXO ','PYO ','PZO ','TOFO',
     &     'PCDM','Xm  ','Ym  ','Zm  ','Px-m','Py-m','Pz-m',
     &     'EDEP'/
      DATA NBITSH 
     &    /32,    32,    32,    32,    32,    32,    32,
     &     32,    32,    32,    32,    32,    32,    32,
     &     32,    32,    32,    32,    32,    32,    32, 
     &     32/
      DATA ORIG   
     &   / 1000., 1000., 1000.,  500.,  500.,  500.,    0.,
     &     1000., 1000., 1000.,  500.,  500.,  500.,    0., 
     &     30000., 400.,  400.,  400.,  500.,  500.,  500.,
     &     0./
      DATA FACT 
     &   / 1.0E6, 1.0E6, 1.0E6, 1.0E6, 1.0E6, 1.0E6, 1.0E3,
     &     1.0E6, 1.0E6, 1.0E6, 1.0E6, 1.0E6, 1.0E6, 1.0E3,
     &     1.,    1.0E6, 1.0E6, 1.0E6, 1.0E6, 1.0E6, 1.0E6,
     &     1.0E4/

      REAL RMRGN
      PARAMETER (RMRGN = 0.05)

      REAL    SCELR(NPSLCT),SCELZ(NPSLCT),BL(NPSLCT),TL(NPSLCT)
      REAL    STEANG(NPSLCT),STETWI(NPSLCT),XYLEN(NPSLCT)
      INTEGER NPHICL(NPSLCT)

      REAL EPC1,EPC2
      REAL EPA,EPB
      REAL EPR,EPZ
      REAL AK,STEAN,STETW
      REAL RCENT

      REAL PHI,XPOS,YPOS,ZPOS
      INTEGER IROT

      INTEGER IDC
      REAL XYLEND,STEAND

      INTEGER IDTYP
      
      INTEGER IVOLU 
      INTEGER NAMID

      INTEGER ISET,IDET

      INTEGER LNBLNK
      CHARACTER*80 CMSG

      REAL SINTW,COSTW

cc      INTEGER NANO
cc      PARAMETER (NANO=1)

      INTEGER I,J


      IF (ISIMJD(IDETCT).EQ.ISKPJD) RETURN
      
      IALUM = NMEDJM*IDETCT+1
      CALL GSTMED(IALUM, 'ALUM', IMALUM, NOSENS, IFIELD, FIELDM,
     &            TMAXFD, STEMAX, DEEMAX, EPSIL, STMIN, UBUF, NWBUF)

      ICO2I = NMEDJM*IDETCT+2
      CALL GSTMED(ICO2I, 'CO2ISO', IMCO2I, ISENS, IFIELD, FIELDM,
     &            TMAXFD, STEMAX, DEEMAX, EPSIL, STMIN, UBUF, NWBUF)

*--   CFRP = CARBON
      ICFRP = NMEDJM*IDETCT+3
      CALL GSTMED(ICFRP, 'CFRP', IMCARB, NOSENS, IFIELD, FIELDM,
     &            TMAXFD, STEMAX, DEEMAX, EPSIL, STMIN, UBUF, NWBUF)

      NAMID = 0

*--   Define a volume with a given name, shape, tracking medium number 
*--   and shape parameters
*--   CT master
      PAR(1) = RIMACT-TCYLCT
      PAR(2) = ROMACT+TCYLCT
      PAR(3) = ZMACT
      CALL GSVOLU('CTMA', 'TUBE',ICO2I, PAR, 3, IVOLU)
      IF (IVOLU.LE.0) THEN
         CALL MSGOUT('CTGEOM','CTMA illegal volume number',IVOLU)
      ENDIF

*--   Locate in jim master
      CALL GSPOS('CTMA', 1, 'MAST', 0., 0., 0., 0, 'ONLY')

*--   Change the attribute of the volume called NAME to the value
      CALL GSATT('CTMA','SEEN', 0)

*--   Calculate each parameters
*--   end-plate Z length is hyperboric cosine
      EPC1 = (EXP(ROMACT/100.)+EXP(-ROMACT/100.))/2.
      EPC2 = (EXP(EPY1CT/100.)+EXP(-EPY1CT/100.))/2.
      EPA  = ((ZMACT-TENPCT)/100.-EPX1CT/100.)/(EPC1-EPC2)
      EPB  =  (ZMACT-TENPCT)/100.-EPC1*EPA
      
      IF (IBNKCT.EQ.ISBKCT) THEN
         
         CHNAMH(NSHTCT) = CHNAMH(NLHTCT)
         NBITSH(NSHTCT) = NBITSH(NLHTCT)
         ORIG  (NSHTCT) = ORIG  (NLHTCT)
         FACT  (NSHTCT) = FACT  (NLHTCT)
         
         NDHTCT = NSHTCT
         
      ELSEIF  (IBNKCT.EQ.ILBKCT) THEN
         
         NDHTCT = NLHTCT
         
      ELSE
         
         CALL MSGOUT('CTGEOM','Wrong bank simulation flag',IBNKCT)
         
      ENDIF

      DO 10 I=1,NSLYCT
         
*--   inner most radius of each layers
         EPR = (RIMACT+(I-1)*(ROMACT-RIMACT)/NSLYCT+
     &        ((ROMACT-RIMACT)/NSLYCT-RLSCCT)*2./3.)/100.
         RILYCT(I) = EPR*100.
         
*--   Half Z length of each layers
         EPZ = EPA*(EXP(EPR)+EXP(-EPR))/2.+EPB
         ZLLYCT(I) = EPZ*100.
         
*--   Number of cell for phi direction
*--   Define # of cell is even
         NPLYCT(I) = 2*NINT(2.*PI*RILYCT(I)/PLSCCT/2.)
         
*--   Half length of shorter side of trapezoid
         BL(I) = 2.*PI*RILYCT(I)/FLOAT(NPLYCT(I))/2.
         
*--   Half length of longer side of trapezoid
         TL(I) = 2.*PI*(RILYCT(I)+RLSCCT)/FLOAT(NPLYCT(I))/2.
         
*--   Calculate stereo angle & twist angle of stereo cell
C         AK=SQRT(RLUNCT*(2.*RILYCT(I)-RLUNCT))/(RILYCT(I)-RLUNCT)
C         XYLYCT(I)=SQRT(4.*RILYCT(I)**2*AK**2/(1.+AK**2))
C         STEAN=ATAN2(XYLYCT(I),ZLLYCT(I)*2.)*RADDEG
C         STETW=ACOS(1.-XYLYCT(I)**2/2./RILYCT(I)**2)*RADDEG/2.
         COSTW = (RILYCT(I)-RLUNCT)/RILYCT(I)
         SINTW = SQRT((1.-COSTW)*(1.+COSTW))
         XYLYCT(I) = 2.*RILYCT(I)*SINTW
         STETW = ACOS(COSTW)*RADDEG
         STEAN = ATAN2(XYLYCT(I),2.*ZLLYCT(I))*RADDEG
         IF (MOD(I,4) .EQ. 0.) THEN
            IAXSCT(I) = +1
         ELSEIF (MOD(I,2) .EQ. 0.) THEN
            IAXSCT(I) = -1
         ELSE
            IAXSCT(I) = 0
         ENDIF
         SANGCT(I) =  FLOAT(IAXSCT(I))*STEAN
         STWICT(I) = -FLOAT(IAXSCT(I))*STETW
         
*--   Set volume name
         WRITE(VNAMC, '(''CC'',I2.2)') I
         WRITE(VNAME, '(''CE'',I2.2)') I
         WRITE(VNAMG, '(''CG'',I2.2)') I
         WRITE(VNAMT, '(''CT'',I2.2)') I
         
*--   Divide CTMA toward radius
         PAR(1) = RIMACT+(I-1)*(ROMACT-RIMACT)/NSLYCT
         PAR(2) = RIMACT+ I   *(ROMACT-RIMACT)/NSLYCT
         PAR(3) = ZMACT
         CALL GSVOLU(VNAMT, 'TUBE',ICO2I, PAR, 3, IVOLU)
         IF (IVOLU.LE.0) THEN
            WRITE(CMSG,'(A4,'' illegal volume number'')') VNAMT
            CALL MSGOUT('CTGEOM',CMSG(:LNBLNK(CMSG)),IVOLU)
         ENDIF
         CALL GSPOS(VNAMT, I, 'CTMA', 0., 0., 0., 0, 'ONLY')
         CALL GSATT(VNAMT,'SEEN', 1)
         
*--   Set cell
         PAR( 1) = ZLLYCT(I)
         PAR( 2) = SANGCT(I)
         PAR( 3) = 0.
         PAR( 4) = STWICT(I)
         PAR( 5) = RLSCCT/2. + RMRGN*2.
         PAR( 6) = BL(I) - RMRGN
         PAR( 7) = TL(I) - RMRGN
         PAR( 8) = 0.
         PAR( 9) = PAR(5)
         PAR(10) = PAR(6)
         PAR(11) = PAR(7)
         PAR(12) = PAR(8)
         CALL GSVOLU(VNAMC, 'GTRA',ICO2I, PAR, 12, IVOLU)
         IF (IVOLU.LE.0) THEN
            WRITE(CMSG,'(A4,'' illegal volume number'')') VNAMC
            CALL MSGOUT('CTGEOM',CMSG(:LNBLNK(CMSG)),IVOLU)
         ENDIF
         CALL GSATT(VNAMC,'SEEN', -2)
         
*--   Radius of module center
         RCENT = (RILYCT(I)+RLSCCT/2.)*COS(STWICT(I)*DEGRAD)
         
*--   Loop of phi direction
         DO 50 J=1,NPLYCT(I)
            
*--   Store the rotation matrix with serial number
            PHI = (J-1)*360./NPLYCT(I)
            IROT = I*1000+J+20000
c            IROT = NROTJM*IDETCT+I*1000+J
            CALL GSROTM(IROT,  90.,PHI-90.,  90.,PHI,  0.,0.)
            
*--   Places a copy of the volume with user number inside its
*--   mother volume 
            XPOS = RCENT*COS(PHI*DEGRAD)
            YPOS = RCENT*SIN(PHI*DEGRAD)
            ZPOS = 0.
            CALL GSPOS(VNAMC, J, VNAMT, XPOS,YPOS,ZPOS, IROT, 'ONLY')
 50      CONTINUE
         
*--   Drift cell definition
C     PAR(5) = RLSCCT/2./NANOCT
C     PAR(5) = RLSCCT/2./NANOCT-RMRGN
C     PAR(5) = RLSCCT/2./NANOCT-RMRGN/NANOCT
*--   Use single cell for sensitive detector
         PAR(5) = RLSCCT/2./NAHTCT-RMRGN/NAHTCT
         PAR(9) = PAR(5)
C         DO 70 IDC=1,NANOCT
         DO 70 IDC=1,NAHTCT
            
*--   Set volume name
            WRITE(VNAMD(IDC), '(''C'',I2.2,I1)') I,IDC
            
*--   Position in the VNAMC module
            XPOS = 0.
C     YPOS=(IDC*2.-1-NANOCT)*RLSCCT/2./NANOCT
C            YPOS = (IDC*2.-1-NANOCT)*PAR(5)
            YPOS = (IDC*2.-1-NAHTCT)*PAR(5)
            ZPOS = 0.
            
*--   Redefine stereo angle for each drift cell
            XYLEND = XYLYCT(I)+2.*YPOS*TAN(STETW*DEGRAD)
            STEAND = ATAN2(XYLEND,ZLLYCT(I)*2.)*RADDEG
            PAR( 2) = FLOAT(IAXSCT(I))*STEAND
            
C     PAR(6)=BL(I)+(TL(I)-BL(I))*(IDC-1)/NANOCT
C     PAR(7)=BL(I)+(TL(I)-BL(I))*(IDC)/NANOCT
C            PAR( 6) = BL(I)+(TL(I)-BL(I))*(IDC-1)/NANOCT-RMRGN*2
C            PAR( 7) = BL(I)+(TL(I)-BL(I))*(IDC  )/NANOCT-RMRGN*2
            PAR( 6) = BL(I)+(TL(I)-BL(I))*(IDC-1)/NAHTCT-RMRGN*2
            PAR( 7) = BL(I)+(TL(I)-BL(I))*(IDC  )/NAHTCT-RMRGN*2
            PAR(10) = PAR(6)
            PAR(11) = PAR(7)
            
            CALL GSVOLU(VNAMD(IDC), 'GTRA',ICO2I, PAR, 12, IVOLU)
            IF (IVOLU.LE.0) THEN
               WRITE(CMSG,'(A4,'' illegal volume number'')') VNAMD(IDC)
               CALL MSGOUT('CTGEOM',CMSG(:LNBLNK(CMSG)),IVOLU)
            ENDIF
            CALL GSPOS(VNAMD(IDC), IDC, VNAMC, 
     &           XPOS,YPOS,ZPOS, 0, 'ONLY')
            
*--   Assign detector to the set CT and defines its basic params.
            CHNMSV(1) = VNAMT
            CHNMSV(2) = VNAMC
            CHNMSV(3) = VNAMD(IDC)
*--  numbering scheme for "CT" is different from other detectors
*--  because the number of phi divisions is rather large
            IDTYP=IDETCT*10000+I*100+IDC
            CALL GSDET ('CT  ',VNAMD(IDC),NDSVCT,CHNMSV,NBITSV,
     &           IDTYP,NHMXCT,0,ISET,IDET)
            NAMID = NAMID+1
            VNAMCT(IDC,I) = VNAMD(IDC)
            
*--   Define hit parameters for detector of set CT.
            
            CALL GSDETH
     &           ('CT  ',VNAMD(IDC),NDHTCT,CHNAMH,NBITSH,ORIG,FACT)
            
 70      CONTINUE
         
*--   Locate end-plate (AL) to +z & -z side
         PAR(1) = RIMACT+(I-1)*(ROMACT-RIMACT)/NSLYCT
         PAR(2) = RIMACT+ I   *(ROMACT-RIMACT)/NSLYCT
         PAR(3) = TENPCT/2.
         CALL GSVOLU(VNAME, 'TUBE',IALUM, PAR, 3, IVOLU)
         IF (IVOLU.LE.0) THEN
            WRITE(CMSG,'(A4,'' illegal volume number'')') VNAME
            CALL MSGOUT('CTGEOM',CMSG(:LNBLNK(CMSG)),IVOLU)
         ENDIF
         ZPOS = ZLLYCT(I)+TENPCT/2.
         CALL GSPOS(VNAME, 1, VNAMT, 0., 0., ZPOS, 0, 'ONLY')
         CALL GSPOS(VNAME, 2, VNAMT, 0., 0.,-ZPOS, 0, 'ONLY')
         
*--   Locate air to outside of end-plate
         PAR(3) = (ZMACT-ZLLYCT(I))/2.
         CALL GSVOLU(VNAMG, 'TUBE',IAIRJM, PAR, 3, IVOLU)
         IF (IVOLU.LE.0) THEN
            WRITE(CMSG,'(A4,'' illegal volume number'')') VNAMG
            CALL MSGOUT('CTGEOM',CMSG(:LNBLNK(CMSG)),IVOLU)
         ENDIF
         ZPOS = ZLLYCT(I)+PAR(3)
         CALL GSPOS(VNAMG, 1, VNAMT, 0., 0., ZPOS, 0, 'ONLY')
         CALL GSPOS(VNAMG, 2, VNAMT, 0., 0.,-ZPOS, 0, 'ONLY')
         CALL GSATT(VNAMG,'SEEN',-1)
 10   CONTINUE
      
*--   Locate CFRP inner & outer cylinder
      PAR(1) = RIMACT-TCYLCT
      PAR(2) = RIMACT
      PAR(3) = ZMACT
      CALL GSVOLU('CTIN', 'TUBE',ICFRP, PAR, 3, IVOLU)
      IF (IVOLU.LE.0) THEN
         CALL MSGOUT('CTGEOM','CTIN illegal volume number',IVOLU)
      ENDIF
      CALL GSPOS('CTIN', 1, 'CTMA', 0., 0., 0., 0, 'ONLY')

      PAR(1) = ROMACT
      PAR(2) = ROMACT+TCYLCT
      PAR(3) = ZMACT
      CALL GSVOLU('CTOU', 'TUBE',ICFRP, PAR, 3, IVOLU)
      IF (IVOLU.LE.0) THEN
         CALL MSGOUT('CTGEOM','CTOU illegal volume number',IVOLU)
      ENDIF
      CALL GSPOS('CTOU', 1, 'CTMA', 0., 0., 0., 0, 'ONLY')
      
*--   Print SET and DETector parameters
      IF (ISWIT(3) .GE. 1) THEN
         CALL GPSETS('CT  ','*   ')
         WRITE(LUSOJM,'(A)') 
     &        'CTGEOM: Layer,# cell,Z len,Ste ang,BL,TL'
         DO 60 I=1,NSLYCT
            WRITE(LUSOJM,'(2I6,4F15.6)') 
     &           I,NPLYCT(I),ZLLYCT(I),SANGCT(I),BL(I),TL(I)
 60      CONTINUE
      ENDIF
      
 999  CONTINUE
      RETURN
      END
