      SUBROUTINE CTHITS
*.
*...CTHITS   Store HITS for the central tracker
*.
*. INPUT     : none
*. OUTPUT    : none
*.
*. SEQUENCE  :
*. COMMON    : GCSETS GCKINE GCTRAK GCONST CTHIT
*. CALLS     : UHTOC  GFPARA GMTOD  GSAHIT
*. CALLED    : GUSTEP
*.
*. BANKS L   :
*. BANKS U   :
*. BANKS M   :
*. BANKS D   :
*.
*. REPORT CONDITIONS
*.
*. AUTHOR    : Naoyasu HOSODA
*. VERSION   : 1.00
*. CREATED   : 27-Apr-1994
*. LAST MOD  :  2-Mar-1999 
*.
*. MODIFICATION LOG
*. 21-Apr-1999 J.Kanzaki, detailed version which include Tauchi's
*.             extension of hit bank is available
*. 15-Apr-1999 T.Tauchi; add parameters of origin;IPAOLD,VERT(3),
*.             PVERT(3).
*.             Check on particle charge is removed.
*.  2-Mar-1999 Nakamura change HIT stracture. Now it's very simple.
*. 07-Nov-1998 J.Kanzaki, Track# for GSAHIT and GSCHIT is changed to
*.                      include Stack Track#
*. 05-Nov-1998 J.Kanzaki, Add SAVE statment for /CTHIT/ and ITROLD
*. 19-Sep-1996 Y.Fujii,   Generate hits also when ISTOP=2 for photons. 
*. 30-Aug-1996 Y.Fujii,   Added dE and Drift time(actually null now)
*. ??-Mar-1996 J.Kanzaki, Added x and y positions in the BANK.
*. 24-Jun-1994 N.Hosoda,  Change HITS contents
*. 18-May-1994 replace geant common by INCLUDE
*.
*.**********************************************************************
*
*--   HITS(1) : X position            (when the track enters)
*--   HITS(2) : Y position            (when the track enters)
*--   HITS(3) : Z position            (when the track enters)
*--   HITS(4) : Px                    (when the track enters)
*--   HITS(5) : Py                    (when the track enters)
*--   HITS(6) : Pz                    (when the track enters)
*--   HITS(7) : Time of fright (nsec) (when the track enters)
*--   HITS(8) : X position            (when the track leaves)
*--   HITS(9) : Y position            (when the track leaves)
*--   HITS(10): Z position            (when the track leaves)
*--   HITS(11): Px                    (when the track leaves)
*--   HITS(12): Py                    (when the track leaves)
*--   HITS(13): Pz                    (when the track leaves)
*--   HITS(14): Time of fright (nsec) (when the track leaves)
*--
*--   If "BSIZ 1" for short MCCTHITINF bank,
*--   HITS(15): Energy Deposit
*--
*--   If "BSIZ 2" for long MCCTHITINF bank,
*--   HITS(15): Particle code of previous track
*--   HITS(16): VERT(1) of current track
*--   HITS(17): VERT(2) of current track
*--   HITS(18): VERT(3) of current track
*--   HITS(19): PVERT(1) of current track
*--   HITS(20): PVERT(2) of current track
*--   HITS(21): PVERT(3) of current track
*--   HITS(22): Energy Deposit
*-- 
*--   GSCHIT requires that "Energy Deposit" must be the last element. 
*--
      IMPLICIT NONE

*--   Common for GEANT
#include "geant321/gcsets.inc"
#include "geant321/gckine.inc"
#include "geant321/gctrak.inc"
#include "geant321/gconst.inc"
#include "geant321/gckmax.inc"
#include "geant321/gcking.inc"

#include "jmdet.inc"
#include "ctgeo.inc"      
#include "ctsim.inc"
#include "cthit.inc"      
*
*-- HIT buffer
      REAL HITS(NPHTCT)
      SAVE HITS
*-- track ID 
      INTEGER ITROLD
      DATA ITROLD / -1 /
      SAVE ITROLD

      INTEGER IHIT

      IF (ISIMJD(IDETCT).EQ.ISKPJD) RETURN
      
*-- Only charged particle
CCCCC IF (CHARGE .EQ. 0.) GOTO 999

*-- Calculate positions etc. only when particle enters or exits the cell,
*-- or new particle is generated, or the particle stoped.
      IF (INWVOL.NE.1 .AND. INWVOL.NE.2 
     &  .AND.INWVOL.NE.0 .AND. ISTOP.NE.2 ) GOTO 999
      
*-- Track has entered a new volume or it is a new track
*      CALL GPCXYZ
      IF (INWVOL.EQ.1) THEN
         
*-- Fill drift length & left-right ambiguity
         HITS(1)  = VECT(1)
         HITS(2)  = VECT(2)
         HITS(3)  = VECT(3)
         HITS(4)  = VECT(4)*VECT(7)
         HITS(5)  = VECT(5)*VECT(7)
         HITS(6)  = VECT(6)*VECT(7)
         HITS(7) = TOFG*10**9

*-- Track is exiting from the current volume or stops.
      ELSEIF (INWVOL.EQ.0 .OR. INWVOL.EQ.2 .OR. ISTOP.EQ.2) THEN
C      ELSEIF (INWVOL.EQ.2 .OR. 
C     &        INWVOL.EQ.0 .AND. ISTOP.EQ.2) THEN

         HITS(8) = VECT(1)
         HITS(9) = VECT(2)
         HITS(10) = VECT(3)
         HITS(11) = VECT(4)*VECT(7)
         HITS(12) = VECT(5)*VECT(7)
         HITS(13) = VECT(6)*VECT(7)
         HITS(14) = TOFG*10**9

         IF (ISTOP.EQ.2) THEN

            HITS(NDHTCT) = (DESTEP+GEKIN)*1.0E6

         ELSE

            HITS(NDHTCT) = DESTEP*1.0E6

         ENDIF

         IF (IBNKCT.EQ.ILBKCT) THEN
C-- add informations of origin of current track

            HITS(15)=IPAOLD
            HITS(16)=VERT(1)
            HITS(17)=VERT(2)
            HITS(18)=VERT(3)
            HITS(19)=PVERT(1)
            HITS(20)=PVERT(2)
            HITS(21)=PVERT(3)

         ENDIF
         
         IF (DESTEP.LE.0.) GOTO 999
         
*--     Stores element values for current hit into data structure JHITS.
         IF (ITRA.NE.ITROLD) THEN
            CALL GSAHIT(ISET,IDET,ITSTAK*ISTAK+ITRA,NUMBV,HITS,IHIT)
            ITROLD = ITRA
         ELSE 
            CALL GSCHIT(ISET,IDET,ITSTAK*ISTAK+ITRA,NUMBV,HITS,1,IHIT)
         ENDIF
      ENDIF
      
 999  CONTINUE
      

      RETURN
      END

