      SUBROUTINE ECDIGI
*.
*...ECDIGI   Compute and store DIGIts for end-cap calorimeter
*.
*. INPUT     : none
*. OUTPUT    : none
*.
*. SEQUENCE  :
*. COMMON    : 
*. CALLS     : 
*. CALLED    : GUDIGI
*.
*. BANKS L   :
*. BANKS U   :
*. BANKS M   :
*. BANKS D   :
*.
*. REPORT CONDITIONS
*.
*. AUTHOR    : Nakamura Isamu
*. VERSION   : 1.00
*. CREATED   : 17-May-1994
*. LAST MOD  : 18-Feb-1999
*.
*. MODIFICATION LOG
*.    18-Feb-1999  K.Ishii  Added new bank for smearing mode
*.    18-Feb-1999  K.Ishii  EEMCUT=0.0, EHDCUT=0.0 (temporary!)
*.    08-Nov-1998  J.Kanzaki Apply cuts on cell energies
*.    21-Aug-1996, Y.Fujii   Hadron Bank IDATA(5) was ToF => now #ofTrack.
*.       //        //      Cell-ID was 1000*Ipfai+Ir => now 100*Ir+Iphai.
*.       //        //      ForB for HAD used NUMBVE. Now using NUMBVH.  
*.       //        //      BKEDT/BKHDT was omitted if Icell<0. Now Icell.NE.0.
*.    23-May-1995, Nakamura.I modify Bank Structure.
*.    20-Jul-1994 : Nakamura Isamu
*.                  Add MCECHITS ZBS Bank.
*.    6-July-1994 : Nakamura Isamu
*.                  Add GFHITS to retrive HITS info.
*.
*.**********************************************************************
*.
*.*********************************************************************
*.     +------------------------+                                     *
*.     | Bank Name  :  MCEESMH  |                                     *
*.     +------------------------+                                     *
*.     Same as MCEEHIT (Endcap ECAL), but for smearing mode           *
*.*********************************************************************
*.     +------------------------+                                     *
*.     | Bank Name  :  MCEHSMH  |                                     *
*.     +------------------------+                                     *
*.     Same as MCEHHIT (Endcap HCAL), but for smearing mode           *
*.*********************************************************************
*.     +------------------------+                                     *
*.     | Bank Name  :  MCEEHIT  |                                     *
*.     +------------------------+                                     *
*.                                                                    *
*.     Contents    : List of Endcap Calorimeter hits on tracks which  *
*.                   the JIM generated.                               *
*.     Record Type : 1-dim. variable length segment                   *
*.     Version     : Release 1.0  1995-Apr-21                         *
*.                                                                    *
*.*********************************************************************
*.
*.(1) Contents in the Master Segment
*.----------------------------------
*.
*.----------------------------------------------------------------------
*.  DATA                            CONTENTS
*.----------------------------------------------------------------------
*. IDATA(1)   Total number of hit blocks ( EM part )
*.----------------------------------------------------------------------
*.
*.(Note)
*.   Master segment = segment # 0
*.
*.
*.(2) Contents in the Data Segment
*.--------------------------------
*.
*.----------------------------------------------------------------------
*.  DATA                            CONTENTS
*.----------------------------------------------------------------------
*. IDATA(1)      Cell Id # (1000*IR+IPHI)
*.  DATA(2)      Total Energy of the Cell ( GeV )
*.  DATA(3)      Time of Flight ( psec )
*. IDATA(4)      # of Contributed Track
*.
*. IDATA(3*I+2)  Contributed Track #  |
*.  DATA(3*I+3)  Deposit Energy       + I = 1, 5
*.  DATA(3*I+4)  Time of Flight       |       Variable Length
*.----------------------------------------------------------------------
*.
*.(Note)
*.   Segment # = hits #
*.
*.---------------------------------------------------------------------
*.*********************************************************************
*.     +------------------------+                                     *
*.     | Bank Name  :  MCEHHIT  |                                     *
*.     +------------------------+                                     *
*.                                                                    *
*.     Contents    : List of Endcap Calorimeter hits on tracks which  *
*.                   the JIM generated.                               *
*.     Record Type : 1-dim. variable length segment                   *
*.     Version     : Release 1.0  1995-May-5                          *
*.                                                                    *
*.*********************************************************************
*.
*.(1) Contents in the Master Segment
*.----------------------------------
*.
*.----------------------------------------------------------------------
*.  DATA                            CONTENTS
*.----------------------------------------------------------------------
*. IDATA(1)   Total number of hit blocks 
*.----------------------------------------------------------------------
*.
*.(Note)
*.   Master segment = segment # 0
*.
*.
*.(2) Contents in the Data Segment
*.--------------------------------
*.
*.----------------------------------------------------------------------
*.  DATA                            CONTENTS
*.----------------------------------------------------------------------
*. IDATA(1)        Cell # (1000*IR+IPHI)
*.  DATA(2)        Deposit Energy in PMT1 ( GeV )
*.  DATA(3)        Deposit Energy in PMT2 ( GeV )
*.  DATA(4)        Time of Flight ( nsec )
*. IDATA(5)        # of Contributed Track
*.
*. IDATA(4*I+2)    Contributed Track #     |
*.  DATA(4*I+3)    Deposit Energy (PMT1)   + I = 1,5 
*.  DATA(4*I+4)    Deposit Energy (PMT2)   |    Variable Length
*.  DATA(4*I+5)    Time of Flight          |
*.----------------------------------------------------------------------
*.
*.(Note)
*.   Segment # = hits #
*.
*.---------------------------------------------------------------------
      IMPLICIT NONE

#include "geant321/gcvolu.inc"
#include "jmdet.inc"
#include "ecgeo.inc"
#include "echit.inc"
#include "ecsmdt.inc"

      INTEGER LNGBKE,LNGBKH
      PARAMETER(LNGBKE = 19)
      PARAMETER(LNGBKH = 25)
      INTEGER ITRAE(NHMXEC),NUMBVE(NDSVEC,NHMXEC)
      INTEGER ITRAH(NHMXEC),NUMBVH(NDSVEC,NHMXEC)
      INTEGER NUMVS(NDSVEC)
      REAL HITSE(NDHTEC,NHMXEC)
      REAL HITSH(NDHTEC,NHMXEC)
      INTEGER FORB
      INTEGER NDATA
      INTEGER IDATA(25)
      REAL    RDATA(25)
      EQUIVALENCE (RDATA(1),IDATA(1))

      REAL BKEDT(LNGBKE,NHMXEC),BKHDT(LNGBKH,NHMXEC)
      
      DATA NUMVS/NDSVEC*0/

      REAL EEMCUT,EHDCUT
      PARAMETER (EEMCUT=0.0, EHDCUT=0.0)

CKI 100 MeV CUT case
CKI      PARAMETER (EEMCUT=0.1*0.028, EHDCUT=0.1*0.024)

      INTEGER NHHITS,NEHITS
      INTEGER NHITSH,NHITSE
      INTEGER NCELLS
      INTEGER IX,IY
      INTEGER NHCEL,NECEL
      INTEGER IC,IB,IOFFS
      INTEGER IERR
      INTEGER ISEG,JSEG
      INTEGER I,J,K,IP,IR

*-- Parameters for bank
*-    for EM hits
      INTEGER NWMEEH,NSMEEH,LSGEEH
      PARAMETER (NWMEEH=1, NSMEEH=4, LSGEEH=3)
      CHARACTER*32 FMSEEH,FSMEEH,FSGEEH
      PARAMETER (FMSEEH='1I', FSMEEH='1I 2F 1I', FSGEEH='1I 2F')
*-    for HD hits
      INTEGER NWMEHH,NSMEHH,LSGEHH
      PARAMETER (NWMEHH=1, NSMEHH=5, LSGEHH=4)
      CHARACTER*32 FMSEHH,FSMEHH,FSGEHH
      PARAMETER (FMSEHH='1I', FSMEHH='1I 3F 1I', FSGEHH='1I 3F')

      CHARACTER*80 CMSG
      INTEGER LNBLNK

      INTEGER ISTART

*-- Initialize HIT ID#
      NEHITS = 0
      NHHITS = 0
      DO 10 I=1,NHMXEC
         DO 11 J = 1,LNGBKE
            BKEDT(J,I)=0
 11      CONTINUE
         DO 12 J = 1,LNGBKH
            BKHDT(J,I)=0
 12      CONTINUE
 10   CONTINUE

      DO 100 I=1,NDREC

*-- Retrieve ECAL Hit Informations from JHITS
         CALL GFHITS ('EC  ',VNEEC(I),NDSVEC,NDHTEC,
     &        NHMXEC,0,NUMVS,ITRAE,NUMBVE,HITSE,NHITSE)

        IF (NHITSE.GT.NHMXEC) THEN
           WRITE(CMSG,'(''NHITSE('',I8,'') > '',I8)') NHITSE,NHMXEC
           CALL MSGOUT('ECDIGI',CMSG(1:LNBLNK(CMSG)),1)
           NHITSE = NHMXEC
        ENDIF      

*-- EM Part Calculation
         DO 110 J=1,NHITSE
            IF (NUMBVE(1,J).EQ.1) THEN
               FORB = 1
            ELSE
               FORB = -1
            ENDIF
            DO 111 IX = 1,2
               DO 112 IY = 1,2
                  IF (HITSE(2*(IX-1)+IY+1,J).GT.EEMCUT) THEN
                     NEHITS = NEHITS+1

*-- Calculate Data to be Stored to BANK
                     BKEDT(1,NEHITS) = (1000*(2*(NUMBVE(2,J)-1)+IX)
     &                    +(2*(NUMBVE(3,J)-1)+IY))*FORB
                     BKEDT(2,NEHITS) = HITSE(2*(IX-1)+IY+1,J)
                     BKEDT(3,NEHITS) = HITSE(1,J)*1.E9
                     BKEDT(4,NEHITS) = 1
                     BKEDT(5,NEHITS) = ITRAE(J)
                     BKEDT(6,NEHITS) = HITSE(2*(IX-1)+IY+1,J)
                     BKEDT(7,NEHITS) = HITSE(1,J)*1.E9
                  ENDIF
 112           CONTINUE
 111        CONTINUE
 110     CONTINUE

*-- Retrieve HCAL Hit Informations from JHITS
         CALL GFHITS('EC  ',VNHEC(I),NDSVEC,NDHTEC,
     &        NHMXEC,0,NUMVS,ITRAH,NUMBVH,HITSH,NHITSH)

        IF (NHITSH.GT.NHMXEC) THEN
           WRITE(CMSG,'(''NHITSH('',I8,'') > '',I8)') NHITSH,NHMXEC
           CALL MSGOUT('ECDIGI',CMSG(1:LNBLNK(CMSG)),2)
           NHITSH = NHMXEC
        ENDIF      

*-- Hadron Part Calculation
         DO 120 J=1,NHITSH
            IF (NUMBVH(1,J).EQ.1) THEN
               FORB = 1
            ELSE
               FORB = -1
            ENDIF
            IF ((HITSH(2,J)+HITSH(3,J)).GT.EHDCUT) THEN
               NHHITS = NHHITS+1
*-- Calculate Data to be Stored to BANK
               BKHDT(1,NHHITS) = (NUMBVH(3,J)+1000*NUMBVH(2,J))*FORB
               BKHDT(2,NHHITS) = HITSH(2,J)
               BKHDT(3,NHHITS) = HITSH(3,J)
               BKHDT(4,NHHITS) = HITSH(1,J)*1.E9
               BKHDT(5,NHHITS) = 1
               BKHDT(6,NHHITS) = ITRAH(J)
               BKHDT(7,NHHITS) = HITSH(2,J)
               BKHDT(8,NHHITS) = HITSH(3,J)
               BKHDT(9,NHHITS) = HITSH(1,J)*1.E9
            ENDIF
 120     CONTINUE

 100   CONTINUE

*-- Marge EM Hit DATA
      ISTART = 0
 200  CONTINUE
      ISTART = ISTART + 1
      IF (ISTART.GT.NEHITS) GOTO 220
      IF (BKEDT(1,ISTART).EQ.0) GOTO 200

      DO 210 IB = ISTART+1,NEHITS
         IF (BKEDT(1,IB).EQ.BKEDT(1,ISTART)) THEN
            IF (BKEDT(4,ISTART) .LT.5) THEN
               IOFFS = INT(3*BKEDT(4,ISTART))
               BKEDT(2,ISTART) = BKEDT(2,ISTART)+BKEDT(2,IB)
               BKEDT(4,ISTART) = BKEDT(4,ISTART) + 1
               BKEDT(IOFFS+5,ISTART) = BKEDT(5,IB)
               BKEDT(IOFFS+6,ISTART) = BKEDT(6,IB)
               BKEDT(IOFFS+7,ISTART) = BKEDT(7,IB)
               BKEDT(1,IB) = 0
            ENDIF
         ENDIF

 210  CONTINUE
      GOTO 200
 220  CONTINUE

      NECEL = 0
      DO 230 ISEG = 1,NEHITS
         IF (BKEDT(1,ISEG).NE.0) THEN
            NECEL = NECEL + 1 
            DO 231 JSEG = 1,LNGBKE
               BKEDT(JSEG,NECEL)=BKEDT(JSEG,ISEG)
 231        CONTINUE
         ENDIF
 230  CONTINUE

*-- Marge HAD Hit DATA
      ISTART = 0
 300  CONTINUE
      ISTART = ISTART + 1
      IF (ISTART.GT.NHHITS) GOTO 320
      IF (BKHDT(1,ISTART).EQ.0) GOTO 300
      DO 310 IB = ISTART+1,NHHITS
         IF (BKHDT(1,IB).EQ.BKHDT(1,ISTART)) THEN
            IF (BKHDT(5,ISTART) .LT.5) THEN
               IOFFS = INT(4*BKHDT(5,ISTART))
               BKHDT(2,ISTART) = BKHDT(2,ISTART)+BKHDT(2,IB)
               BKHDT(3,ISTART) = BKHDT(3,ISTART)+BKHDT(3,IB)
               BKHDT(5,ISTART) = BKHDT(5,ISTART) + 1
               BKHDT(IOFFS+6,ISTART) = BKHDT(6,IB)
               BKHDT(IOFFS+7,ISTART) = BKHDT(7,IB)
               BKHDT(IOFFS+8,ISTART) = BKHDT(8,IB)
               BKHDT(IOFFS+9,ISTART) = BKHDT(9,IB)
               BKHDT(1,IB) = 0
            ENDIF
         ENDIF

 310  CONTINUE
      GOTO 300
 320  CONTINUE

      NHCEL = 0
      DO 330 ISEG = 1,NHHITS
         IF (BKHDT(1,ISEG).NE.0) THEN
            NHCEL = NHCEL + 1 
            DO 331 JSEG = 1,LNGBKH
               BKHDT(JSEG,NHCEL)=BKHDT(JSEG,ISEG)
 331        CONTINUE
         ENDIF
 330  CONTINUE

*-- Create ZBS Bank MCEEHIT
      
*-- First count number of hits
      NCELLS = 0
      DO 410 ISEG = 1,NECEL
         IF (BKEDT(2,ISEG).GT.EEMCUT) THEN
            NCELLS = NCELLS+1
         ENDIF
 410  CONTINUE
 
      CALL KZBCR2('MCEEHIT',NWMEEH,FMSEEH,
     &     NCELLS,NSMEEH,FSMEEH,LSGEEH,FSGEEH,IERR)

      IDATA(1) = NCELLS
      CALL KZREP2('MCEEHIT',0,0,NWMEEH,IDATA)
      
*-- Store Data to given Segment of Bank MCEEHIT
      NCELLS = 0
      DO 420 ISEG = 1,NECEL
         IF (BKEDT(2,ISEG).GT.EEMCUT) THEN
            NCELLS = NCELLS+1
            IDATA(1) = BKEDT(1,ISEG)
            RDATA(2) = BKEDT(2,ISEG)
            RDATA(3) = BKEDT(3,ISEG)
            IDATA(4) = BKEDT(4,ISEG)
            CALL KZREP2('MCEEHIT',NCELLS,0,NSMEEH,IDATA)
            DO 421 IC = 1,5
               IDATA(1) = BKEDT(3*IC+2,ISEG)
               RDATA(2) = BKEDT(3*IC+3,ISEG)
               RDATA(3) = BKEDT(3*IC+4,ISEG)
               CALL KZREP2('MCEEHIT',NCELLS,IC,LSGEEH,IDATA)
 421        CONTINUE
         ENDIF
 420  CONTINUE

*-- for smearing DATA
      IF (ISIMJD(IDETEC).EQ.ISMRJD) THEN

*--  First count number of hits
         DO 430 I=1,NMXEEP
            DO 431 J=1,NMXEER
               DO 432 K=1,2
*-- skip if no deposit            
                  IF (EECSMP(I,J,K)+EECSME(I,J,K).GT.0.) THEN
                     NCELLS = NCELLS+1
                  ENDIF
 432           CONTINUE 
 431        CONTINUE
 430     CONTINUE

*-- Create ZBS Bank MCEESMH
         CALL KZBCR1('MCEESMH',NWMEEH,FMSEEH,
     &        NCELLS,NSMEEH,FSGEEH,IERR)

         IDATA(1) = NCELLS
         CALL KZREP1('MCEESMH',0,NWMEEH,IDATA)

*-- Store Data to given Segment of Bank MCEESMH
         NCELLS = 0
         
         DO 440 I=1,NMXEEP
            DO 441 J=1,NMXEER
               DO 442 K=1,2
*-- skip if no deposit            
                  IF (EECSMP(I,J,K)+EECSME(I,J,K).GT.0.) THEN
*-- forward or backward
                     IF(K.EQ.1) THEN
                        FORB = 1
                     ELSE
                        FORB = -1
                     ENDIF
*-- fill 
                     NCELLS = NCELLS+1
                     IDATA(1)  = (J*1000+I)*FORB
                     RDATA(2)  = EECSMP(I,J,K)+EECSME(I,J,K)
                     RDATA(3)  = 0.
                     IDATA(4)  = 0
                     CALL KZREP1('MCEESMH',NCELLS,NSMEEH,IDATA)
                  ENDIF
 442           CONTINUE 
 441        CONTINUE 
 440     CONTINUE 
      ENDIF

*-- Create ZBS Bank MCEHHIT

*-- First count number of hits
      NCELLS = 0
      DO 510 ISEG = 1,NHCEL
         IF (BKHDT(2,ISEG).GT.EHDCUT) THEN
            NCELLS = NCELLS+1
         ENDIF
 510  CONTINUE 

      CALL KZBCR2('MCEHHIT',NWMEHH,FMSEHH,
     &     NCELLS,NSMEHH,FSMEHH,LSGEHH,FSGEHH,IERR)

      IDATA(1) = NCELLS
      CALL KZREP2('MCEHHIT',0,0,NWMEHH,IDATA)

*-- Store Data to given Segment of Bank MCEHHIT
      NCELLS = 0
      DO 520 ISEG = 1,NHCEL
         IF (BKHDT(2,ISEG).GT.EHDCUT) THEN
            NCELLS = NCELLS+1
            IDATA(1)  = BKHDT(1,ISEG)
            RDATA(2)  = BKHDT(2,ISEG)
            RDATA(3)  = BKHDT(3,ISEG)
            RDATA(4)  = BKHDT(4,ISEG)
            IDATA(5)  = BKHDT(5,ISEG)
            CALL KZREP2('MCEHHIT',NCELLS,0,NSMEHH,IDATA)
            DO 521 IC = 1,5
               IDATA(1)  = BKHDT(4*IC+2,ISEG)
               RDATA(2)  = BKHDT(4*IC+3,ISEG)
               RDATA(3)  = BKHDT(4*IC+4,ISEG)
               RDATA(4)  = BKHDT(4*IC+5,ISEG)
               CALL KZREP2('MCEHHIT',NCELLS,IC,LSGEHH,IDATA)
 521           CONTINUE
         ENDIF
 520  CONTINUE

*-- for smearing DATA
      IF (ISIMJD(IDETEC).EQ.ISMRJD) THEN

*-- First count number of hits
         NCELLS = 0
         
         DO 530 I=1,NMXEHP
            DO 531 J=1,NMXEHR
               DO 532 K=1,2
*-- skip if no deposit            
                  IF (EECSMH(I,J,K).GT.0.) THEN
                     NCELLS = NCELLS+1
                  ENDIF
 532           CONTINUE
 531        CONTINUE
 530     CONTINUE

*-- Create ZBS Bank MCEHSMH
         CALL KZBCR1('MCEHSMH',NWMEHH,FMSEHH,
     &        NCELLS,NSMEHH,FSGEHH,IERR)

         IDATA(1) = NCELLS
         CALL KZREP1('MCEHSMH',0,NWMEHH,IDATA)

*-- Store Data to given Segment of Bank MCEHSMH
         NCELLS = 0
         
         DO 541 I=1,NMXEHP
            DO 542 J=1,NMXEHR
               DO 543 K=1,2
*-- skip if no deposit            
                  IF (EECSMH(I,J,K).GT.0.) THEN
*-- forward or backward
                     IF(K.EQ.1) THEN
                        FORB = 1
                     ELSE
                        FORB = -1
                     ENDIF
*-- fill 
                     NCELLS = NCELLS+1
                     IDATA(1)  = (J*1000+I)*FORB
                     RDATA(2)  = EECSMH(I,J,K)
                     RDATA(3)  = 0.
                     IDATA(4)  = 0
                     CALL KZREP1('MCEHSMH',NCELLS,NSMEHH,IDATA)
                  ENDIF
 543           CONTINUE 
 542        CONTINUE 
 541     CONTINUE 
      ENDIF

      RETURN
      END
