      SUBROUTINE ECSUB2
*.
*...ECSUB1   Define the electromagnetic endcap calorimeter
*.           geometry and tracking media.
*.
*. INPUT     : none
*. OUTPUT    : none
*.
*. SEQUENCE  :
*. COMMON    : GCFLAG GCONST UCTMED ECGEO
*. CALLS     : GSVOLU GSPOS  GSDVN  GSDET  GSDETH GSDETD GPSETS
*. CALLED    : UGEOM
*.
*. BANKS L   :
*. BANKS U   :
*. BANKS M   :
*. BANKS D   :
*.
*. REPORT CONDITIONS
*.
*. AUTHOR    : Nakamura Isamu
*. VERSION   : 1.00
*. CREATED   : 05-May-1995
*. LAST MOD  : 23-May-1995
*.
*. MODIFICATION LOG
*. 23-May-1995 : Namaura.I, Modify JHITS structure.
*. 06-Mar-1996 : K.Ishii
*.               Modified the cone structure
*.
*.**********************************************************************
      IMPLICIT NONE

*-- Include GEANT common block
#include "geant321/gconst.inc"
#include "geant321/gcflag.inc"
      
#include "jmmed.inc"
#include "jmdet.inc"
#include "jmrot.inc"
#include "ecgeo.inc"
#include "echit.inc"
#include "ecmed.inc"

      REAL DSIZE(7)

*-- Detecter Prameters
      CHARACTER*4 NMVE(NDSVEC),NMVH(NDSVEC),NMH(NDHTEC)
      INTEGER NBITV(NDSVEC),NBITH(NDHTEC)
      REAL ORIG(NDHTEC),FACT(NDHTEC)
      CHARACTER*4 VNER,VNET,VNSCNE,VNSCNH
      DATA NBITV  /     8,     8,     8/
      DATA NMH    /'TOF ','E1  ','E2  ','E3  ','E4  '/
CKI      DATA NBITH  /   24 ,    24,    24,    24,    24/
      DATA NBITH  /   24 ,    32,    32,    32,    32/
      DATA ORIG   /    0.,    0.,    0.,    0.,    0./
CKI      DATA FACT   / 1.E12,  1.E4,  1.E4,  1.E4,  1.E4/
      DATA FACT   / 1.E12,  1.E8,  1.E8,  1.E8,  1.E8/

      REAL ZOEC,DRI,DRO
      INTEGER NSAMPE,NSAMPH

      REAL RI1,RI2
      REAL RO1,RO2

      INTEGER NDPECM,NDPECN

      REAL Z0

      INTEGER IROT,IVOL
      INTEGER ISET,IDET
      INTEGER I,J

*-- Fundamental Parameters 
C      RI1EC    = 25. 
C      RO1EC    = 230.
C      RI2EC    = 25. 
C      RO2EC    = 300.
C      ZIEC     = 300.
C      BLEC     = 140.
C      DEMEC    = 20. 
C      TSCEEC   = .1  
C      TSCHEC   = .2  
C      BSIZEC   = 10. 

*-- Calculate some Parameters
      ZOEC     = ZIEC+BLEC
      NDREC    = INT((RO1EC-RI1EC)/BSIZEC)
      DRI      = (RO1EC-RI1EC)/NDREC
      DRO      = (RO2EC-RI2EC)/NDREC
      NSAMPE   = INT(DEMEC/(5*TSCEEC))
      NSAMPH   = INT((BLEC-DEMEC)/(5*TSCHEC))

*-- Define EC Master Volume
      DSIZE(1) = BLEC/2.
      DSIZE(2) = RI1EC
      DSIZE(3) = RO1EC
      DSIZE(4) = RI2EC
      DSIZE(5) = RO2EC
      CALL GSVOLU('ECMA','CONE',IAIRJM,DSIZE,5,IVOL)

*-- Do Loop Define Each TUBE Structure.
      DO 10 I=1,NDREC

*-- Define Volume Name
         WRITE(VNER,'(''ER'',I2.2)') I
         WRITE(VNET,'(''ET'',I2.2)') I
         WRITE(VNSCNE,'(''EE'',I2.2)') I
         WRITE(VNSCNH,'(''EH'',I2.2)') I

         NMVE(1) = 'ECMA'
         NMVE(2) = VNER
         NMVE(3) = VNET
         NMVH(1) = 'ECMA'
         NMVH(2) = VNER
         NMVH(3) = VNET
         VNREC(I)= VNER
         VNTEC(I)= VNET
         VNEEC(I)= VNSCNE
         VNHEC(I)= VNSCNH

*-- Calcurate Detector Size Paramters
         RI1      = RI1EC+(I-1)*DRI
         RO1      = RI1EC+I*DRI
         RI2      = RI2EC+(I-1)*DRO
         RO2      = RI2EC+I*DRO

*-- Define EC Cone Structure
         DSIZE(1) = BLEC/2.
         DSIZE(2) = RI1
         DSIZE(3) = RO1
         DSIZE(4) = RI2
         DSIZE(5) = RO2
         CALL GSVOLU(VNER,'CONE',ILEDEC,DSIZE,5,IVOL)

*-- Store Block Size into COMMON
         SIZEC(1,I) = DSIZE(1)
         SIZEC(2,I) = DSIZE(2)
         SIZEC(3,I) = DSIZE(3)
         SIZEC(4,I) = DSIZE(4)
         SIZEC(5,I) = DSIZE(5)

*-- Divide Cone Structure in Phi derection
         NDPECM   = 16
         NDPECN   = INT(2.*RO1EC*PI/BSIZEC/NDPECM)
         IF (NDPECN.LT.1.) NDPECN=1
         NDPEC(I)    = NDPECM*NDPECN
C        WRITE(*,*) NDPEC(I),RO1EC,BSIZEC
C        NDPEC(1)    = 9
         CALL GSDVN(VNET,VNER,NDPEC(1),2)
C        SIZEC(6,I) = 360./NDPEC(1)/2.
         SIZEC(6,I) = 360./NDPEC(1)

*-- Define EM and HAD Scintillator Plate.
         CALL GSVOLU(VNSCNE,'CONS',ISCIEC,0,0,IVOL)
         CALL GSVOLU(VNSCNH,'CONS',ISCIEC,0,0,IVOL)

*-- Put the COPY of EM Scintillator Plate into EC Tower by DO LOOP
         SFPHEC(I) = PI/FLOAT(NDPEC(1))

         DO 30 J=1,NSAMPE
            DSIZE(1) = TSCEEC/2.
            DSIZE(2) = RI1+(5*J-1)*TSCEEC/BLEC*(RI2-RI1)
            DSIZE(3) = RO1+(5*J-1)*TSCEEC/BLEC*(RO2-RO1)
            DSIZE(4) = RI1+5*J*TSCEEC/BLEC*(RI2-RI1)
            DSIZE(5) = RO1+5*J*TSCEEC/BLEC*(RO2-RO1)
            DSIZE(6) = 0.-360./NDPEC(1)/2.
            DSIZE(7) = 0.+360./NDPEC(1)/2.
            Z0       = (-1.)*BLEC/2.+(5.*J-.5)*TSCEEC
            CALL GSPOSP(VNSCNE,J,VNET,0.,0.,Z0,0,'ONLY',DSIZE,7)
            CALL GSATT(VNSCNE,'SEEN',0)
            SURFEC(1,1,J,I) = DSIZE(2)
            SURFEC(2,1,J,I) = DSIZE(3)
 30      CONTINUE

*-- Put the COPY of HAD Scintillator Plate into EC Tower by DO LOOP
         DO 40 J=1,NSAMPH
            DSIZE(1) = TSCHEC/2.
            DSIZE(2) = RI1+(DEMEC+(5*J-1)*TSCHEC)/BLEC*(RI2-RI1)
            DSIZE(3) = RO1+(DEMEC+(5*J-1)*TSCHEC)/BLEC*(RO2-RO1)
            DSIZE(4) = RI1+(DEMEC+5*J*TSCHEC)/BLEC*(RI2-RI1)
            DSIZE(5) = RO1+(DEMEC+5*J*TSCHEC)/BLEC*(RO2-RO1)
            DSIZE(6) = 0.-360./NDPEC(1)/2.
            DSIZE(7) = 0.+360./NDPEC(1)/2.
            Z0       = (-1.)*BLEC/2.+DEMEC+(5.*J-.5)*TSCHEC
            CALL GSPOSP(VNSCNH,J,VNET,0.,0.,Z0,0,'ONLY',DSIZE,7)
            CALL GSATT(VNSCNH,'SEEN',0)
            SURFEC(1,2,J,I) = DSIZE(2)
            SURFEC(2,2,J,I) = DSIZE(3)
 40      CONTINUE         

         CALL GSPOS(VNER,I,'ECMA',0.,0.,0.,0,'ONLY')

*-- Define Detector
         CALL GSDET
     &        ('EC  ',VNSCNE,NDSVEC,NMVE,NBITV,1,NHMXEC,0,ISET,IDET)
         CALL GSDET
     &        ('EC  ',VNSCNH,NDSVEC,NMVH,NBITV,1,NHMXEC,0,ISET,IDET)
         CALL GSDETH('EC  ',VNSCNE,NDHTEC,NMH,NBITH,ORIG,FACT)
         CALL GSDETH('EC  ',VNSCNH,NDHTEC,NMH,NBITH,ORIG,FACT)

 10   CONTINUE

*-- Put the EC into Master Volume
      IROT    = NROTJM*IDETEC
      Z0      = (ZIEC+ZOEC)/2.
      CALL GSPOS('ECMA',1,'MAST',0.,0.,Z0,0,'ONLY')
      CALL GSPOS('ECMA',2,'MAST',0.,0.,0.-Z0,IRMIRR,'ONLY')

*-- Print SET and DETector parameters
      IF (ISWIT(3) .GE. 1) CALL GPSETS('EC  ','*   ')

      RETURN
      END


