CDECK  ID>, EVWRIT.
      SUBROUTINE EVWRIT (NTRAK,ITKDM,MXTRK,PTRAK,IDTRAK,IHSTRY,
     +                   VTX,IRUN,IEVT,IZRN,LUN,COPTN,IERR)
*.
*...EVWRIT
*.
*. Routine to write an event record to a file in a relatively compact
*. ascii form, or to read it back.  This simple routine has the
*. peculiarity for read mode (COPTN='R') that if the number of records
*. in any event between the first event of the file and the event to be
*. read is larger than MXTRK, the maximum number of records reserved for
*. PTRAK, the reading routine gets "lost"  (an error message will be
*. written in this case). The user must ensure that MXTRK is larger
*. than the number of event record entries for all events in the file.
*. MXTRK=400 is certainly sufficient for a multi-hadronic event
*. generated at a center-of-mass energy of 92 GeV.
*.
*. SEQUENCE  : RCREP GOXTRA
*. IN/OUTPUT : NTRAK   The number of entries in the event record
*. IN/OUTPUT : ITKDM   The first dimension of PTRAK
*. INPUT     : MXTRK   The maximum number of entries in PTRAK
*. IN/OUTPUT : PTRAK   The 5-momenta array of the event record (Px,Py,Pz,E,M)
*. IN/OUTPUT : IDTRAK  The IDs of the particles in the event record
*. IN/OUTPUT : IHSTRY  The history of the particles in the event record
*. IN/OUTPUT : VTX     The vertex position for the event (for all particles)
*. INPUT     : IRUN    For COPTN='W' : the run number, to label the record
*.                     For COPTN='R' : the run number to read
*. INPUT     : IEVT    For COPTN='W' : the event number, to label the record
*.                     For COPTN='R' : the event to read
*. INPUT     : IZRN    The random number seed for the event
*. INPUT     : LUN     The logical unit for read or for write
*. INPUT     : COPTN   = 'W' to write,
*.                     = 'F' to read the event labeled IEVT
*.                     = 'R' to read the next event regardless of IEVT value
*.                     = 'WB' 'FB' 'RB' for binary file option.
*. OUTPUT    : IERR    = -1 if there is a problem
*.                     =  0 if all is OK
*.                     =  1 if end of file
*. CALLS     : REPORT
*. CALLED    : By User
*.
*. AUTHOR    :  J.W.Gary
*. CREATED   :  17-Mar-89
*. LAST MOD  :  14-May-93
*.
*. Modification Log.
*. 14-May-93  Fix bug in previous modification          D.R.Ward
*.  5-Mar-93  Disable checking of run/event if no RUNG card D.R.Ward
*.  8-Jan-92  Improve (?) error trapping                D.R.Ward
*.  3-Jan-92  Add binary option                         D.R.Ward
*. 26-Apr-91  write or read the contents of /GOXTRA/    S.L.Lloyd
*. 15-Oct-90  increase maximum possible event number    J.W.Gary
*. 15-Apr-90  add protection against bad event records  J.W.Gary
*. 11-May-89  various improvements for flexibility      J.W.Gary
*.
**********************************************************************
      IMPLICIT NONE
      INTEGER MXXTRA
      PARAMETER (MXXTRA=100)
      INTEGER NWXTRA
      REAL RXTRA
      COMMON /GOXTRA/ NWXTRA,RXTRA(MXXTRA)
      CHARACTER*132 CHREP
      COMMON/RCREP/CHREP
      INTEGER  MXEVT,MXRUN
      PARAMETER  (MXEVT=999999,MXRUN=100000)
      INTEGER  IDTRAK (*),IHSTRY (*)
      INTEGER  NTRAK,ITKDM,LUN,IP,IERR,IX,IRUN,IEVT,IZRN,
     +         IRUNL,IEVTL,MXTRK,IH1,IH2,IPSAV,IHSAV
      REAL  PTRAK (ITKDM,*),VTX (*)
      CHARACTER*(*) COPTN
      CHARACTER*5  LABEL,CHEADR
      CHARACTER*2  TAG,CDUM
      DATA  LABEL / 'EVENT' /,TAG / 'ZZ' /

      IERR = 0
*
*-- Reading.
*
      IF (COPTN(1:1).EQ.'F'.OR.COPTN(1:1).EQ.'R') THEN
          IF (IEVT.GT.MXEVT.OR.IRUN.GT.MXRUN) THEN
              WRITE (CHREP,FMT='('' Event, Run number must'',
     +            '' be less than'',2I8)') MXRUN+1,MXEVT+1
              CALL REPORT ('EVWRIT',1,'C')
              IERR = -1
              GO TO 990
          END IF
  110     IF(LEN(COPTN).GE.2 .AND. COPTN(2:2).EQ.'B') THEN
             READ (LUN,ERR=1000,END=1020) CHEADR,IRUNL,IEVTL,
     +         CDUM,NTRAK,IZRN,(VTX (IX),IX=1,3)
          ELSE
             READ (LUN,1080,ERR=1000,END=1020) CHEADR,IRUNL,IEVTL,
     +         CDUM,NTRAK,IZRN,(VTX (IX),IX=1,3)
          ENDIF
Comment          WRITE (6,FMT='(''EVWRIT '',A6,3I8,I12)')
Comment     +           CHEADR,IRUNL,IEVTL,NTRAK,IZRN
          IF (COPTN(1:1).EQ.'F'.AND.IRUNL.NE.IRUN) THEN
              WRITE (CHREP,FMT='('' Error, Run on file='',
     +          I7,'', requested='',I7)') IRUNL,IRUN
              CALL REPORT ('EVWRIT',2,'C')
              IERR = -1
              GO TO 990
          END IF
          IF (COPTN(1:1).EQ.'F'.AND.(IEVTL.GT.IEVT)) THEN
              WRITE (CHREP,FMT='('' Error, Requested event number ='',
     +          I7,'', smaller than that on file ='',I7)') IEVT,IEVTL
              CALL REPORT ('EVWRIT',6,'C')
              IERR = -1
              GO TO 990
          END IF
*
          IF (CDUM.EQ.'XX') THEN
            IF (NTRAK.GT.MXXTRA) THEN
              WRITE (CHREP,FMT='('' Error, Max.Nr.of extra words ='',
     +          I10,'' exceeded'')') MXXTRA
              CALL REPORT ('EVWRIT',8,'W')
              NWXTRA = MXXTRA
            ELSE
              NWXTRA = NTRAK
            ENDIF
            IF(LEN(COPTN).GE.2 .AND. COPTN(2:2).EQ.'B') THEN
               READ (LUN,ERR=1000) (RXTRA(IX),IX=1,NWXTRA)
            ELSE
               READ (LUN,1070,ERR=1000) (RXTRA(IX),IX=1,NWXTRA)
            ENDIF
            GO TO 110
*
          ELSE
            IF (NTRAK.GT.MXTRK) THEN
              WRITE (CHREP,FMT='('' Error, Max.Nr.of tracks ='',
     +          I10,'' exceeded'')') MXTRK
              CALL REPORT ('EVWRIT',3,'C')
              IERR = -1
              GO TO 990
            END IF
*
            DO 140 IP = 1,NTRAK
              IF(LEN(COPTN).GE.2 .AND. COPTN(2:2).EQ.'B') THEN
                 READ (LUN,ERR=1001) IDTRAK (IP),IHSTRY (IP),
     +             (PTRAK (IX,IP),IX=1,5)
              ELSE
                 READ (LUN,1090,ERR=1001) IDTRAK (IP),IHSTRY (IP),
     +             (PTRAK (IX,IP),IX=1,5)
              ENDIF
              IF (IDTRAK (IP).EQ.91.OR.IDTRAK (IP).EQ.92) THEN
                 IH1 = MOD (IHSTRY (IP),1000)
                 IH2 = NINT (FLOAT (IHSTRY (IP)) / 1000.)
                 IF (IH1.GE.IP.OR.IH2.GE.IP.OR.
     +               IH1.LT.0.OR.IH2.LT.0) THEN
                    IPSAV = IP
                    IHSAV = IHSTRY (IP)
                    IERR = -1
                 END IF
              ELSE IF (IHSTRY (IP).GE.IP.OR.IHSTRY(IP).LT.0) THEN
                 IPSAV = IP
                 IHSAV = IHSTRY (IP)
                 IERR = -1
              END IF
              GO TO 140
*
 1001         IF(COPTN(1:1).EQ.'F'.AND.IEVTL.EQ.IEVT) GO TO 1000
*
  140       CONTINUE
*
            IF (COPTN(1:1).EQ.'F'.AND.IEVTL.NE.IEVT) THEN
               IERR=0
               GO TO 110
            ENDIF
*
            IF (IERR.NE.0) THEN
              WRITE (CHREP,FMT='('' Error, Bad History record'',
     +          '' IPSAV,IHSAV='',2I6)') IPSAV,IHSAV
              CALL REPORT ('EVWRIT',3,'F')
              GO TO 990
            END IF
          ENDIF
*
          IF (COPTN(1:1).EQ.'F'.AND.IEVTL.NE.IEVT) GO TO 110
          IRUN=IRUNL
          IEVT=IEVTL
*
*-- Writing
*
      ELSE IF (COPTN(1:1).EQ.'W') THEN
          IF (NWXTRA.GT.0) THEN
            IF(LEN(COPTN).GE.2 .AND. COPTN(2:2).EQ.'B') THEN
               WRITE (LUN) LABEL,IRUN,IEVT,'XX',NWXTRA,
     +             IZRN,(VTX (IX),IX=1,3)
               WRITE (LUN) (RXTRA(IX),IX=1,NWXTRA)
            ELSE
               WRITE (LUN,1080) LABEL,IRUN,IEVT,'XX',NWXTRA,
     +             IZRN,(VTX (IX),IX=1,3)
               WRITE (LUN,1070) (RXTRA(IX),IX=1,NWXTRA)
            ENDIF
          ENDIF
*
          IF(LEN(COPTN).GE.2 .AND. COPTN(2:2).EQ.'B') THEN
             WRITE (LUN) LABEL,IRUN,IEVT,TAG,NTRAK,
     +           IZRN,(VTX (IX),IX=1,3)
             DO 161 IP = 1,NTRAK
                 WRITE (LUN) IDTRAK (IP),IHSTRY (IP),
     +                 (PTRAK (IX,IP),IX=1,5)
  161        CONTINUE
          ELSE
             WRITE (LUN,1080) LABEL,IRUN,IEVT,TAG,NTRAK,
     +           IZRN,(VTX (IX),IX=1,3)
             DO 160 IP = 1,NTRAK
                 WRITE (LUN,1090) IDTRAK (IP),IHSTRY (IP),
     +                 (PTRAK (IX,IP),IX=1,5)
  160        CONTINUE
          ENDIF
*
*-- Something else ?
*
      ELSE
          WRITE (CHREP,FMT='('' COPTN ='',A6,
     +          '' option unknown'')')  COPTN
          CALL REPORT ('EVWRIT',4,'F')
          IERR = -1
          GO TO 990
      END IF

  990 RETURN
 1000 WRITE (CHREP,FMT='('' Error reading LUN '',I6)') LUN
      CALL REPORT ('EVWRIT',5,'C')
      IERR = -1
      RETURN
 1020 WRITE (CHREP,FMT='('' End-of-file, LUN '',I6)') LUN
      CALL REPORT ('EVWRIT',7,'C')
      IERR =  1
      RETURN
 1070 FORMAT (1X,6E13.7)
 1080 FORMAT (1X,A5,2I6,A2,I6,I12,1X,3E13.7)
 1090 FORMAT (1X,I7,I6,1X,5E13.7)
      END
