      SUBROUTINE GUFLDQ(XV, BF)
*.
*...GUFLDQ   Calculate Magnetic field BV=(Bx, By, Bz) at position 
*.           XV(x,y,z). 
*.           In this version, B field map of 2D calculation is used. 
*.
*. INPUT     :
*.        XV : (x,y,z) in global coordinate system (cm)
*. OUTPUT    :
*.        BF : (Bx, By, Bz) in global coordinate system (Kgauss)
*.           : BF=0, when XV is outside QCS.
*.      
*. AUTHOR    : A. Miyamoto
*. VERSION   : 1.00
*. CREATED   : 07-Jul-1999
*. LAST MOD  : 07-Jul-1999
*.
*.
*. MODIFICATION LOG
*. 22-Jul-1999  J.Kanzaki Path for the map file is fixed in the
*.                        installation directory
*. 22-Jul-1999  J.Kanzaki Logical unit number for the map file is taken
*.                        from "jmfile.inc"   
*.
*.**********************************************************************
      IMPLICIT NONE
    
#include "jmfile.inc"

      REAL  XV(3), BF(3)

*-- Field map data.
*--   BDAT(2,i) : Bx and By 
*--   IPB(ix,iy,im) : pointer to BDAT, im is an index to 
*--           matterial code, 1=air or coil, 2=iron.
*--           = 0, when no field data is available.
      INTEGER MAXX,MAXY,MAXM
      PARAMETER (MAXX=150, MAXY=150, MAXM=2)
      INTEGER  IPB(0:MAXX, 0:MAXY, MAXM)
      INTEGER MAXBUF
      PARAMETER (MAXBUF=15000)
      REAL     BDAT(2,0:MAXBUF)
      DATA     BDAT(1,0)/0.0/,  BDAT(2,0)/0.0/
      
      REAL     RGNMAX(2)
      DATA     RGNMAX/15.0, 15.0/
      REAL     XBNWID, YBNWID

*-- About variables
*--     ROTANG : Half of the crossing angle of beam (radian)
*--     QCS1(1:2) : Z position of QCS1, min. and max.
*--     QCS2(1:2) : Z position of QCS2, min. and max.
*--     QCSRMX    : Outer radius of QCS.
*--        BF=0, when XV is outside the QCS region defined
*--     RGNBND(2,i) : xw and yw coordinates of pole tipe ot QCS.
*--        (xw,yw) is a local coordinate system attached to QCS
*--     RGNLIN(2,i) : line parameter connecting RGNBND(,i) and RGNBND(,i+1)
*--                Valid upto MAXBND-1
*--     QCSLIN(2) : Line parameter of QCS outside edge

      REAL ROTANG/-0.0040/
      REAL COSTH, SINTH

      REAL QCS1(2)/200.0, 420.0/
      REAL QCS2(2)/440.0, 840.0/
      REAL QCSRMX /15.0/

      INTEGER MAXBND
      PARAMETER  ( MAXBND=7 )
      REAL RGNBND(2,MAXBND)
      DATA RGNBND/0.474, 0.474, 0.85, 0.264, 1.0, 0.264, 2.0, 1.0,
     &   4.0, 1.4, 5.3, 2.6, 9.5, 2.6/
      REAL RGNLIN(2,MAXBND)
      REAL QCSLIN(2)/-1.0, 17.0/

      INTEGER PRJM(2,2,0:7)
      DATA PRJM/ 1.,0.,0.,1.,  0.,1.,1.,0., 
     &     0.,1.,-1.,0., -1.,0.,0.,1.,
     &     -1.,0.,0.,-1.,  0.,-1.,-1.,0.,  
     &     0.,-1.,1.,0.,   1.,0.,0.,-1.  /
      
      INTEGER IFIRST
      DATA IFIRST/0/
      SAVE IFIRST
      SAVE COSTH, SINTH, PI, PIQUAD
      SAVE XBNWID, YBNWID
      SAVE RGNLIN

      CHARACTER*20 BMFILE/'bmap-qcxy.dat'/
      CHARACTER*256 FILE

      REAL PI,PIQUAD

      INTEGER IX,IY
      INTEGER IM,IP
      REAL BX,BY
      REAL SGNROT
      REAL XP,YP,ZP
      REAL XRAD
      INTEGER NRAD
      REAL XW,YW,RW
      INTEGER NWX,NWY
      INTEGER MAT
      REAL XI,ETA
      REAL BXW,BYW
      REAL BXP,BYP

      INTEGER LNBLNK
      
      INTEGER I
      

*-- (0) Initialize at first call
      IF (IFIRST.EQ.0) THEN
         IFIRST=1
         COSTH=COS(ROTANG)
         SINTH=SIN(ROTANG)
         PI = ACOS(-1.0)
         PIQUAD = PI/4.0
         XBNWID = RGNMAX(1)/FLOAT(MAXX)
         YBNWID = RGNMAX(2)/FLOAT(MAXY)

         DO 100 I=1,MAXBND-1
            RGNLIN(1,I)=(RGNBND(2,I+1)-RGNBND(2,I))/
     &           (RGNBND(1,I+1)-RGNBND(1,I))
            RGNLIN(2,I)=RGNBND(2,I)
 100     CONTINUE
         RGNLIN(1,MAXBND)=0.0
         RGNLIN(2,MAXBND)=0.0

         DO 120 IX=0,MAXY
            DO 130 IY=0,MAXX
               IPB(IX,IY,1)=0
               IPB(IX,IY,2)=0
 130        CONTINUE
 120     CONTINUE
#ifdef BMAPDataDir
         FILE = BMAPDataDir
         FILE = FILE(:LNBLNK(FILE))//BMFILE
#else
         FILE = BMFILE
#endif         
         OPEN(LUFMPQ,FILE=FILE,FORM='FORMATTED',STATUS='OLD')
 140     CONTINUE
         READ(LUFMPQ,*) IM,IX,IY,IP
         IF (IM.EQ.-1) GOTO 150
         IPB(IX,IY,IM)=IP
         GO TO 140
 150     CONTINUE
         READ(LUFMPQ,*,END=160) IP,BX,BY
         BDAT(1,IP)=BX
         BDAT(2,IP)=BY
         GO TO 150
 160     CONTINUE
         CLOSE(LUFMPQ)

         WRITE(LUSOJM,*) 'GUFLDQ:',
     &        ' field data obtained from ', BMFILE
         
      ENDIF
      
*-- (1) Transfer XV to map coordinate system
      BF(1)=0.0
      BF(2)=0.0
      BF(3)=0.0

      SGNROT = SIGN(1.0, XV(3))
      XP=XV(1)*SGNROT*COSTH - XV(3)*SINTH
      YP=XV(2)
      ZP=XV(1)*SINTH + XV(3)*SGNROT*COSTH

c      print *,' xv=',xv,' SGNROT=',SGNROT
c      PRINT *,' XP, YP, ZP',XP,YP,ZP
c      print *,' sinth,costh=',sinth,costh
      
      IF (ABS(ZP).LT.QCS1(1).OR.
     &   (ABS(ZP).GT.QCS1(2).AND.ABS(ZP).LT.QCS2(1)).OR.
     &    ABS(ZP).GT.QCS2(2) ) RETURN

      XRAD = ATAN2(YP,XP)
      IF (XRAD.LT.0.0) XRAD=XRAD+2*PI
      NRAD = XRAD / PIQUAD
      NRAD = MIN(7, NRAD)
      XW = PRJM(1,1,NRAD)*XP + PRJM(2,1,NRAD)*YP
      YW = PRJM(1,2,NRAD)*XP + PRJM(2,2,NRAD)*YP
      RW = SQRT(XW*XW + YW*YW)

c      print *,' nradm rw=',nrad,rw
c      print *,' xw,yw=',xw,yw, ' atan2=',atan2(yp,xp),
c     >  ' xrad=',xrad
c      print *,' piquad=',piquad
      
      IF (RW.GT.QCSRMX) RETURN

*-- (2) Get Pointer to field map
      NWX = MAX(MIN(MAXX-1,INT(XW/XBNWID)),0)
      NWY = MAX(MIN(MAXY-1,INT(YW/YBNWID)),0)

      MAT=1
      IF (YW.GE.RGNBND(2,MAXBND)) THEN
         IF (YW .LE. QCSLIN(1)*XW + QCSLIN(2)) MAT=2
      ELSEIF (XW.GE.RGNBND(1,MAXBND)) THEN
         MAT=2
      ELSE
         DO 200 I=MAXBND-2,1,-1
            IF (XW.GE.RGNBND(1,I).AND.XW.LE.RGNBND(1,I+1)) THEN
               IF (YW.GE.RGNLIN(1,I)*(XW-RGNBND(1,I))+RGNLIN(2,I)) MAT=2
c              print *,' xw,yw=',xw,yw,' i=',i,
c     &        ' rgnlin=',rgnlin(1,i),rgnlin(2,i)
              GO TO 220
           ENDIF
 200     CONTINUE
 220     CONTINUE
      ENDIF
 
*-- (3) Calculate field from field map.
      XI =2*((XW-XBNWID*NWX)/XBNWID-0.5)
      ETA=2*((YW-YBNWID*NWY)/YBNWID-0.5)

c      print *,' nwx, nwy,mat=',nwx,nwy,mat,' xi, eta=',xi,eta
c      print *,' ipb=',ipb(nwx,nwy,mat),
c     >' bx=',bdat(1,IPB(NWX,NWY,MAT))
c      print *,' ipb=',ipb(nwx+1,nwy,mat),
c     >' bx=',bdat(1,IPB(NWX+1,NWY,MAT))
c      print *,' ipb=',ipb(nwx,nwy+1,mat),
c     > ' bx=',bdat(1,IPB(NWX,NWY+1,MAT))
c      print *,' ipb=',ipb(nwx+1,nwy+1,mat),
c     > ' bx=',bdat(1,IPB(NWX+1,NWY+2,MAT))

      BXW=(BDAT(1,IPB(NWX,NWY,MAT))*(1-XI)*(1-ETA) +
     &     BDAT(1,IPB(NWX+1,NWY,MAT))*(1+XI)*(1-ETA) +
     &     BDAT(1,IPB(NWX,NWY+1,MAT))*(1-XI)*(1+ETA) +
     &     BDAT(1,IPB(NWX+1,NWY+1,MAT))*(XI+1)*(ETA+1))/4
      BYW=(BDAT(2,IPB(NWX,NWY,MAT))*(1-XI)*(1-ETA) +
     &     BDAT(2,IPB(NWX+1,NWY,MAT))*(1+XI)*(1-ETA) +
     &     BDAT(2,IPB(NWX,NWY+1,MAT))*(1-XI)*(1+ETA) +
     &     BDAT(2,IPB(NWX+1,NWY+1,MAT))*(1+XI)*(1+ETA))/4

*-- (4) Rotate axis back and return BF
      BXP = PRJM(1,1,NRAD)*BXW + PRJM(2,1,NRAD)*BYW
      BYP = PRJM(1,2,NRAD)*BXW + PRJM(2,2,NRAD)*BYW
      IF (NRAD.EQ.3 .OR. NRAD.EQ.6) THEN
         BXP=-1*BXP
         BYP=-1*BYP
      ENDIF

      BF(1)=BXP*SGNROT*COSTH
      BF(2)=BYP
      BF(3)=-BXP*SINTH

      RETURN
      END
