      SUBROUTINE GUFLDS(VECT,F)
*.
*...GUFLDS   Calculate the Magnetic field by Main solenoid and Shield coil
*.           Field map data are obtained from a file, scfldmap.dat.
*.           The file is read at the first call to this routine.
*.           Field is calculated by ANSYS. 
*,           In the field calculation, Coil of the compensation magnet is at
*            15 cm < r < 16 cm.  And Z from +- 180cm to the pole tipe.
*,           Call to this routine from |R| > 16cm is invalid.
*.           For Z > 200.0 cm and Z < -200 cm, this routine returns just a 
*            null field, since it should be calculated by field map data
*.           of QCS.           
*.           
*. INPUT     : VECT= array of 3 with the position in space where the
*.             field has to be returned
*.             VECT(1)= x (cm)
*.             VECT(2)= y (cm)
*.             VECT(3)= z (cm)
*. OUTPUT    : F= array of 3 with the magnetic field in VECT, in
*.             kilo-Gauss units
*.             F(1)= Bx (KG)
*.             F(2)= By (KG)
*.             F(3)= Bz (KG)
*.
*. SEQUENCE  :
*. COMMON    : 
*. CALLS     : 
*. CALLED    : GHELIX GRKUTA
*.
*. BANKS L   :
*. BANKS U   :
*. BANKS M   :
*. BANKS D   :
*.
*. REPORT CONDITIONS
*.
*. AUTHOR    : Akiya Miyamoto
*. VERSION   : 1.00
*. CREATED   : 15-Jun-1999
*. LAST MOD  : 15-Jun-1999
*.
*. MODIFICATION LOG
*. 22-Oct-1999  J.Kanzaki convert unit of field strength from
*.                        Tesla to kGauss
*. 22-Jul-1999  J.Kanzaki Path for the map file is fixed in the
*.                        installation directory
*. 22-Jul-1999  J.Kanzaki Print statements are converted to MSGOUT
*. 22-Jul-1999  J.Kanzaki Logical unit number for the map file is taken
*.                        from "jmfile.inc"   
*.
*.**********************************************************************
      IMPLICIT NONE
      
#include "jmfile.inc"
      
*--   Common
      REAL VECT(3),F(3)

      REAL R

      INTEGER MAXDAT
      PARAMETER (MAXDAT=4000)
      REAL FLD(4,MAXDAT)
C     FLD(1,I)=R, FLD(2,I)=Z, FLD(3,I)=BR, FLD(4,I)=BZ
      SAVE FLD

      INTEGER MAXIND
      PARAMETER (MAXIND=60)
      REAL RTBL(3,MAXIND)
C     RTBL(1,I)=X, RTBL(2,I)=ZMIN, RTBL(3,I)=ZMAX
      INTEGER NRPNT(2,MAXIND), NIND
C     NRPNT(1,I) = 1ST, NRPNT(2,I)=LAST
      SAVE RTBL, NRPNT, NIND

C     FLD(1,)=R, FLD(2,)=Z, FLD(3,)=BR, FLD(4,)=BZ
      INTEGER NUMFLD/0/
      SAVE NUMFLD

      REAL RMAX,ZMAX
      PARAMETER (RMAX=40.0, ZMAX=200.0)

      CHARACTER*20 BMFILE/'bmap-solenoid.dat'/
      CHARACTER*256 FILE
      
      INTEGER NODE
      
      INTEGER MINR,IR
      INTEGER IZN,IZ
      INTEGER IP1,IP2,IP3,IP4

      REAL XI,ETA,FR

      REAL TSL2KG
      PARAMETER (TSL2KG = 10.)

      INTEGER I,K

      INTEGER LNBLNK


*-- (1) At first call read data.
      IF (NUMFLD.EQ.0) THEN
         NUMFLD=0
#ifdef BMAPDataDir
         FILE = BMAPDataDir
         FILE = FILE(:LNBLNK(FILE))//BMFILE
#else
         FILE = BMFILE
#endif
         OPEN(LUFMPS,FILE=FILE,STATUS='OLD',ERR=100)
         DO 110 I=1, MAXDAT
 130        READ(LUFMPS,*,END=120) NODE, (FLD(K,I),K=1,4)
            IF (FLD(1,I).GT.RMAX) GOTO 130
            IF (ABS(FLD(2,I)).GT.ZMAX) GOTO 130
            NUMFLD=NUMFLD+1
 110     CONTINUE
         CALL MSGOUT('GUFLDS','Bfield buffer overflow',MAXDAT)
C         PRINT *,'Error in GUFLDS .. Bfiled buffer overflow'
C         PRINT *,'Please check input data, or increase buffer.'
         STOP
 100     CONTINUE
         CALL MSGOUT('GUFLDS','Cannot open map file',LUFMPS)
C         PRINT *,'Error in GUFLDS .. Can not open file.'
         STOP
         
 120     CONTINUE
         CLOSE(LUFMPS)

         WRITE(LUSOJM,*) 'GUFLDS: There are ',NUMFLD,
     &        ' field data obtained from ', BMFILE
C         PRINT *,' There are ',numfld,
C     &     ' field data obtained from', bmfile
         
*-- create index
         NIND=1
         RTBL(1,NIND)=FLD(1,1)
         RTBL(2,NIND)=FLD(2,1)
         RTBL(3,NIND)=FLD(2,1)
         NRPNT(1,NIND)=1
         NRPNT(2,NIND)=1
         DO 200 I=2, NUMFLD
            IF (FLD(1,I).EQ.RTBL(1,NIND)) GO TO 200
            RTBL(3,NIND)=FLD(2,I-1)
            NRPNT(2,NIND)=I-1
            IF (NRPNT(2,NIND)-NRPNT(1,NIND)+1.GT.10) NIND=NIND+1
            RTBL(1,NIND)=FLD(1,I)
            RTBL(2,NIND)=FLD(2,I)
            RTBL(3,NIND)=FLD(2,I)
            NRPNT(1,NIND)=I
            NRPNT(2,NIND)=I
 200     CONTINUE
C         PRINT *,'IN GUFLDS .. THERE ARE ',NIND,' R DATA'
C         DO 230 I=1, NIND
C            PRINT *,' I=',I,'R=',RTBL(1,I),' Z ',RTBL(2,I),
C     >       ' TO ',RTBL(3,I),' NRPNT=',NRPNT(1,I),NRPNT(2,I),
C     >       ' #NODES=',NRPNT(2,I)-NRPNT(1,I)+1
C 230     CONTINUE

       ENDIF

*-- Calculate bfield in current position.
      F(1)=0.0
      F(2)=0.0
      F(3)=0.0
      R=SQRT(VECT(1)**2+VECT(2)**2)
      IF (R.GT.16.0) THEN
         CALL MSGOUT('GUFLDS','R position is outside the map',INT(R))
c         PRINT *,'Error in GUFLDS... R position',R,
c     &        ' is outside the map.'
      ENDIF   
      
      IF (VECT(3).LT.-200.0 .OR. VECT(3).GT.200.0) RETURN

      
      IF (R.LE.12.5) THEN
         MINR = MIN(8, INT(R/RTBL(1,2)))
         IR = MAX(MINR,1)
      ELSEIF (R.LE.13.75) THEN
         IR = 8
      ELSEIF (R.LE.15.0) THEN
         IR = 9
      ELSE
         IR=10
      ENDIF

      IF (VECT(3).LE.-160.0) THEN
         IZN=INT( VECT(3) +200.0 )/2.0
         IZ = MAX(IZN, 1 )
      ELSEIF (VECT(3).GE.160.0) THEN
         IZ = (VECT(3) - 160.0)/2.0 + 84
      ELSE
         IZ = (VECT(3) +160.0 )/5.0 + 20
      ENDIF

C      PRINT *,' R,Z=',R,VECT(3),' IR=',IR,' IZ=',IZ

      IP1=MIN(NRPNT(1,IR)+IZ,NRPNT(2,IR)-1)
      IP2=IP1+1
      IP3=MIN(NRPNT(1,IR+1)+IZ,NRPNT(2,IR+1)-1)
      IP4=IP3+1
             
C      PRINT *,' IP1=',IP1,' FLD=',(FLD(K,IP1),K=1,4)
C      PRINT *,' IP2=',IP2,' FLD=',(FLD(K,IP2),K=1,4)
C      PRINT *,' IP3=',IP3,' FLD=',(FLD(K,IP3),K=1,4)
C      PRINT *,' IP4=',IP4,' FLD=',(FLD(K,IP4),K=1,4)

      XI = (R-FLD(1,IP1))/(FLD(1,IP3)-FLD(1,IP1))
      ETA = (VECT(3)-FLD(2,IP1))/(FLD(2,IP2)-FLD(2,IP1))
C      PRINT *,' XI, ETA=',XI,ETA
      FR = ( FLD(3,IP1)*(1-XI)*(1-ETA) + FLD(3,IP2)*(1-XI)*ETA
     &      +FLD(3,IP3)*XI*(1-ETA) + FLD(3,IP4)*XI*ETA )
      IF (R.GT.1.E-6) THEN
        F(1) = VECT(1)/R*FR
        F(2) = VECT(2)/R*FR
      ENDIF
      F(3) =( FLD(4,IP1)*(1-XI)*(1-ETA) + FLD(4,IP2)*(1-XI)*ETA
     &      +FLD(4,IP3)*XI*(1-ETA) + FLD(4,IP4)*XI*ETA )

C      PRINT *,' FR=',FR,' F=',F(1),F(2),F(3)
      
      F(1) = TSL2KG * F(1)
      F(2) = TSL2KG * F(2)
      F(3) = TSL2KG * F(3)

      RETURN
      END
