      SUBROUTINE LMDIGI
*.
*...LMDIGI   Compute and store DIGIts for Active Mask
*.
*. INPUT     : non
*. OUTPUT    : non
*.
*. SEQUENCE  :
*. COMMON    : AMGEO  GCNUM
*. CALLS     : GFHITS KZBCR1 KZPUT1 JGP2LP
*. CALLED    : GUDIGI
*.
*. BANKS L   : MCAMHIT
*. BANKS U   :
*. BANKS M   :
*. BANKS D   :
*.
*. REPORT CONDITIONS
*.
*. AUTHOR    : Nakamura Isamu
*. VERSION   : 1.00
*. CREATED   : 25-Feb-1999
*. LAST MOD  : 
*.
*. MODIFICATION LOG
*. 26-Apr-1999 : T.Tauchi
*.               Particle code is inserted in MCLMHIT
*.
*. Bank Data Structure ;
*. name = MCLMHIT
*.        Master segment 
*.                        1  total number of hits
*.        Data Segment
*.                        1  track number responsible to this hit
*.                        2  F/B
*.                        3  Phi#                                   
*.                        4  Radial#                                
*.                        5  PCOD
*.                        6  x coordinate of this hit [cm]
*.                        7  y coordinate of this hit [cm]
*.                        8  z coordinate of this hit [cm]
*.                        9  p of particle generated this hit [GeV]
*.                       10  Px of particle [GeV]
*.                       11  Py of particle [GeV]
*.                       12  Pz of particle [GeV]
*.                       13  energy deposit to this pad [GeV]
*.
*.**********************************************************************
      IMPLICIT NONE

*-- Common
#include "geant321/gcnum.inc" 

#include "lmgeo.inc"
#include "lmhit.inc"
#include "lmsmdt.inc"
#include "jmdet.inc"
     
      INTEGER NDATMX
      PARAMETER (NDATMX=13)
      INTEGER NUMVS(NDSVLM),NUMBV(NDSVLM,NHMXLM),ITRA(NHMXLM)
      REAL    HITS(NDHTLM,NHMXLM)

      INTEGER   IDATA(NDATMX)
      REAL      RDATA(NDATMX)
      EQUIVALENCE (IDATA(1), RDATA(1))
      CHARACTER*32 CFMLMH,CFSLMH
      INTEGER NWMLMH,NWPLMH
*-- Parlmeter for bank
      PARAMETER (NWMLMH=1)
      PARAMETER (CFMLMH='1I')
      PARAMETER (CFSLMH='5I 8F')
      PARAMETER (NWPLMH=0)
      
      INTEGER NUMHIT
      INTEGER NHITS,NHITSH,IHIT,IHT

      INTEGER ILPC

      CHARACTER*80 CMSG
      INTEGER LNBLNK

      INTEGER I,J,IERR


*-- Initialize
      CALL VZERO(NUMVS,NDSVLM)
*
*-- check skip frag
      IF (ISIMJD(IDETLM).EQ.ISKPJD) THEN 
         GOTO 999
      ELSE IF (ISIMJD(IDETLM).EQ.ISMRJD) THEN 
         GOTO 998
      ENDIF
      
      NUMHIT=0
      CALL GFHITS('LM  ','LMC ',NDSVLM,NDHTLM,NHMXLM,0,NUMVS,
     &     ITRA,NUMBV,HITS,NHITS)
      IF (NHITS.GT.NHMXLM) THEN
         WRITE(CMSG,'(''Nhits overflow for layer '',I1)') I
         CALL MSGOUT('LMDIGI',CMSG(1:LNBLNK(CMSG)),NHMXLM)
         NHITS = NHITS-1
      ENDIF    
      NUMHIT = NUMHIT+NHITS

*-- Create new ZBS bank with the 1-dim. fixed length segment.
      CALL KZBCR1('MCLMHIT',NWMLMH,CFMLMH,NUMHIT,NDATMX,CFSLMH, IERR)
      
*-- Retrieve HITS produced by tracks
      
      IHIT = 0
      
      CALL GFHITS('LM  ','LMC ',NDSVLM,NDHTLM,NHMXLM,0,NUMVS,
     &     ITRA,NUMBV,HITS,NHITS)
      IF (NHITS.GT.NHMXLM) THEN
         NHITS = NHMXLM
      ENDIF    

*-- Loop of hit in this detector name
      DO 110 IHT=1,NHITS
         IHIT = IHIT+1
         IDATA(1) = ITRA(IHT)
         IDATA(2) = NUMBV(1,IHT)
         IDATA(3) = NUMBV(2,IHT)
         IDATA(4) = NUMBV(3,IHT)
*-- Convert geant p-code to LUND74 P-code
         CALL JGP2LP(NINT(HITS(1,IHT)), ILPC,IERR)
         IF (IERR .EQ. 0) THEN
            IDATA(5) =ILPC
         ELSE
            IDATA(5) =9999
         ENDIF
         DO 120 J=6,NDATMX
            RDATA(J) = HITS(J-4,IHT)
 120     CONTINUE
         
*-- Put data words into the given segment of the bank
         CALL KZPUT1('MCLMHIT',IHIT,NWPLMH,NDATMX,IDATA)

 110  CONTINUE

*-- Put data words into the master segment of the bank
      IDATA(1) = NUMHIT
      CALL KZPUT1('MCLMHIT',0,0,NWMLMH,IDATA)
*
      GOTO 999
*
*-- in the case smearing
 998  IF (ISIMJD(IDETLM).EQ.ISMRJD) THEN 
         
*-- Create new ZBS bank with the 1-dim. fixed length segment.
         CALL KZBCR1('MCLMSMHT',1,'1I',NLMSMH,9,'2I 7F',IERR)
*-- KZPUT master segment
         IDATA(1) = NLMSMH
         CALL KZPUT1('MCLMSMHT',0,0,1,IDATA)
*-- loop over contributed tracks
         DO 200 I=1,NLMSMH
*-- copy buffer data to array           
            CALL UCOPY(ILMSMH(1,I),IDATA(1),2)
            CALL UCOPY(RLMSMH(1,I),IDATA(3),7)
*-- KZPUT data segment            
            CALL KZPUT1('MCLMSMHT',I,0,9,IDATA)
*            WRITE (*,'(3I5,7F10.3)') I,(IDATA(J),J=1,2),(RDATA(J),J=3,9)
*            
 200     CONTINUE 
      ENDIF
      
 999  CONTINUE 
      RETURN
      END
