      SUBROUTINE QCGEOM
*.
*...BCGEOM   Geomtry of the Q-Magnet at IR.
*.
*.
*. INPUT     : none
*. OUTPUT    : none
*.
*. SEQUENCE  :
*. COMMON    : UCTMED
*. CALLS     : 
*. CALLED    : IRGEOM
*.
*. BANKS L   :
*. BANKS U   :
*. BANKS M   :
*. BANKS D   :
*.
*. REPORT CONDITIONS
*. AUTHOR    : Yoshiaki Fujii
*. VERSION   : 1.00
*. CREATED   : 28-Aug-1996
*. LAST MOD  : 
*.
*. MODIFICATION LOG
*. 12-Jan-1999, Y.Fujii;  Modified accroding to the Kanzaki-san's coding rule.
*. Feb.25 1997, T.Tauchi; added QC0 as a carbon mask in front of QC1.
*.
*.**********************************************************************
      IMPLICIT NONE
      
#include "geant321/gconst.inc"

#include "jmmat.inc"
#include "jmmed.inc"
#include "jmdet.inc"
#include "irgeo.inc"

      INTEGER IQC12,ICOIL,ICFRP
      CHARACTER*4  VNQC
      REAL         QXYNR(2,8), QXYNL(2,8), DSIZE(11),
     &             TH, COSTH, SINTH, RIN, ROUT

      REAL X,Y,Z
      REAL PHIDX,PHIDY
      REAL FAIX,FAIY,FAIZ
      REAL THX,THY,THZ
      REAL D,CRSDEG
      INTEGER IROT,IROT11,IROT12,IROT13,IROT14,IVOL
      CHARACTER*80 CMSG
      INTEGER LNBLNK
      INTEGER I

      IQC12 = NMEDJM*IDETIR+IDQCIR+1
      CALL GSTMED(IQC12,'QC12', IMIRON, NOSENS, NOFILD, 0.,
     &     TMAXFD,STEMAX,DEEMAX,EPSIL,STMIN,UBUF,NWBUF )

      ICOIL = NMEDJM*IDETIR+IDQCIR+2
      CALL GSTMED(ICOIL,'COIL', IMCOIL, NOSENS, NOFILD, 0.,
     &     TMAXFD, STEMAX, DEEMAX, EPSIL, STMIN, UBUF, NWBUF)

      ICFRP = NMEDJM*IDETIR+IDQCIR+3
      CALL GSTMED(ICFRP, 'CFRP', IMCARB, NOSENS, IFIELD, FIELDM,
     &            TMAXFD, STEMAX, DEEMAX, EPSIL, STMIN, UBUF, NWBUF)

*-- Inner and Outer Radii of QC
      RIN = SQRT( QXY1QC(1)**2 + QXY1QC(2)**2 )
      ROUT= SQRT( QXY8QC(1)**2 + QXY8QC(2)**2 )

*-- Name of trapezoids which froms one quadrant 
      VNQC = 'QC10'

*-- Define master volume for one Quadrant of QC pole-tip = 1/4 TUBE
      DSIZE(1) = RIN
      DSIZE(2) = ROUT
      DSIZE(3) = (QC1ZQC(2)-QC1ZQC(1))/2.0
      DSIZE(4) = 225.0
      DSIZE(5) = 315.0
      CALL GSVOLU('QC1Q','TUBS',IAIRJM,DSIZE,5,IVOL)
      IF (IVOL.LE.0) THEN
         CALL MSGOUT('QCGEOM','QC1Q illegal volume number',IVOL)
      ENDIF
      CALL GSATT('QC1Q','SEEN',1)

*-- Rotate each point describing pole-shape by -135degree so that trapezoid 
*-- bottom line is parallel to X-axis ; QXYNQC => QXYNL (Left-side shape)
      TH = -135.0*PI/180.0  
      COSTH = COS(TH)
      SINTH = SIN(TH)
      DO 101   I=1,8
         QXYNL(1,I) = QXYNQC(1,I)*COSTH - QXYNQC(2,I)*SINTH
         QXYNL(2,I) = QXYNQC(1,I)*SINTH + QXYNQC(2,I)*COSTH
  101 CONTINUE

*-- Copy Left-side shape to Right-side shape by mirror reflection.
      DO 102 I=1,8
         QXYNR(1,I) =-QXYNL(1,I)
         QXYNR(2,I) = QXYNL(2,I)
  102 CONTINUE

*-- Define 1st-7th Trapezoids
      DO 111 I=1,7
         DSIZE(1)  =(QC1ZQC(2)-QC1ZQC(1))/2.0
         DSIZE(2)  = 0.0
         DSIZE(3)  = 0.0
         DSIZE(4)  =(QXYNL(2,I)  -QXYNL(2,I+1))/2.0
         DSIZE(5)  =(QXYNR(1,I+1)-QXYNL(1,I+1))/2.0
         DSIZE(6)  =(QXYNR(1,I)  -QXYNL(1,I)  )/2.0
         DSIZE(7)  = 0.0
         DSIZE(8)  =(QXYNL(2,I)  -QXYNL(2,I+1))/2.0
         DSIZE(9)  =(QXYNR(1,I+1)-QXYNL(1,I+1))/2.0
         DSIZE(10) =(QXYNR(1,I)  -QXYNL(1,I)  )/2.0
         DSIZE(11) = 0.0
         WRITE(VNQC(4:4),'(I1)') I
         CALL GSVOLU(VNQC,'TRAP',IQC12,DSIZE,11,IVOL)
         IF (IVOL.LE.0) THEN
            WRITE(CMSG,'(A4,'' illegal volume number'')') VNQC 
            CALL MSGOUT('QCGEOM',CMSG(:LNBLNK(CMSG)),IVOL)
c            WRITE(6,*)
c     &           '!!! In QCGEOM ; failed GSVOLU for QC1',I
c     &           ,'-th trapezoid !!!'  
         ENDIF
         Y = (QXYNL(2,I)+QXYNL(2,I+1))/2.0         
         CALL GSPOS(VNQC,1,'QC1Q',0,Y,0,0,'ONLY')
         CALL GSATT(VNQC,'SEEN',1)
  111 CONTINUE

*-- Define QC1 volume
      DSIZE(1) = RIN
      DSIZE(2) = ROUT
      DSIZE(3) = (QC1ZQC(2)-QC1ZQC(1))/2.0
      CALL GSVOLU('QC1 ','TUBE',IAIRJM,DSIZE,3,IVOL)
      IF (IVOL.LE.0) THEN
         CALL MSGOUT('QCGEOM','QC1 illegal volume number',IVOL)
c         WRITE(6,*)
c     &  '!!! In QCGEOM ; failed GSVOLU for QC1 Volume !!!'  
      ENDIF
      CALL GSATT('QC1 ','SEEN',1)

*-- Locate four Quadrants in QC1 Volume with rotation around Z-axis 
      DO 121 I=1,4
         PHIDX = 45.0 + 90.0*FLOAT(I-1)
         PHIDY = PHIDX + 90.0 
         IROT = NROTJM*IDETIR+10*IDQCIR+I
         CALL GSROTM(IROT,90.0,PHIDX,90.0,PHIDY,0.0,0.0)
         CALL GSPOS('QC1Q',I,'QC1 ',0,0,0,IROT,'ONLY')
  121 CONTINUE

*-- Define 8th trapezoid. Note x-y exchange for 90deg rotation. 
      DSIZE(1)  =(QC1ZQC(2)-QC1ZQC(1))/2.0
      DSIZE(2)  = 0.0
      DSIZE(3)  = 0.0
      DSIZE(4)  =(QXY8QC(1)-QXY7QC(1))/2.0
      DSIZE(5)  = QXY8QC(2)
      DSIZE(6)  = QXY7QC(2)
      DSIZE(7)  = 0.0
      DSIZE(8)  =(QXY8QC(1)-QXY7QC(1))/2.0
      DSIZE(9)  = QXY8QC(2)
      DSIZE(10) = QXY7QC(2)
      DSIZE(11) = 0.0
      CALL GSVOLU('QC18','TRAP',IQC12,DSIZE,11,IVOL)
      IF (IVOL.LE.0) THEN
         CALL MSGOUT('QCGEOM','QC18 illegal volume number',IVOL)
c         WRITE(6,*)
c     &  '!!! In QCGEOM ; failed GSVOLU for QC1 8-th trapezoid !!!'  
      ENDIF
      CALL GSATT('QC18','SEEN',1)

*-- Locate 8-th trapezoid in QC1 volume
      D = (QXY7QC(1)+QXY8QC(1))/2.0         
      IROT11 = NROTJM*IDETIR+10*IDQCIR+11
      CALL GSROTM(IROT11,90.0,  0.0,90.0, 90.0, 0.0, 0.0)
      CALL GSPOS('QC18',1,'QC1 ',0,-D,0,IROT11,'ONLY')
      IROT12 = NROTJM*IDETIR+10*IDQCIR+12
      CALL GSROTM(IROT12,90.0, 90.0,90.0,180.0, 0.0, 0.0)
      CALL GSPOS('QC18',2,'QC1 ', D,0,0,IROT12,'ONLY')
      IROT13 = NROTJM*IDETIR+10*IDQCIR+13
      CALL GSROTM(IROT13,90.0,180.0,90.0,270.0, 0.0, 0.0)
      CALL GSPOS('QC18',3,'QC1 ',0, D,0,IROT13,'ONLY')
      IROT14 = NROTJM*IDETIR+10*IDQCIR+14
      CALL GSROTM(IROT14,90.0,270.0,90.0,  0.0, 0.0, 0.0)
      CALL GSPOS('QC18',4,'QC1 ',-D,0,0,IROT14,'ONLY')

*-- Define Coil in QC1 volume 
      DSIZE(1) = (COIXQC(2)-COIXQC(1))/2.0
      DSIZE(2) = (COIYQC(2)-COIYQC(1))/2.0
      DSIZE(3) = (QC1ZQC(2)-QC1ZQC(1))/2.0
      CALL GSVOLU('QCCH','BOX ',ICOIL,DSIZE,3,IVOL)
      DSIZE(1) = (COIYQC(2)-COIYQC(1))/2.0
      DSIZE(2) = (COIXQC(2)-COIXQC(1))/2.0
      DSIZE(3) = (QC1ZQC(2)-QC1ZQC(1))/2.0
      CALL GSVOLU('QCCV','BOX ',ICOIL,DSIZE,3,IVOL)

      X = (COIXQC(2)+COIXQC(1))/2.0
      Y = (COIYQC(2)+COIYQC(1))/2.0
      CALL GSPOS('QCCH',1,'QC1 ', X, Y,0,0,'ONLY')
      CALL GSPOS('QCCH',2,'QC1 ',-X, Y,0,0,'ONLY')
      CALL GSPOS('QCCH',3,'QC1 ',-X,-Y,0,0,'ONLY')
      CALL GSPOS('QCCH',4,'QC1 ', X,-Y,0,0,'ONLY')
      CALL GSPOS('QCCV',1,'QC1 ', Y, X,0,0,'ONLY')
      CALL GSPOS('QCCV',2,'QC1 ',-Y, X,0,0,'ONLY')
      CALL GSPOS('QCCV',3,'QC1 ',-Y,-X,0,0,'ONLY')
      CALL GSPOS('QCCV',4,'QC1 ', Y,-X,0,0,'ONLY')

***************** QC2 *****************

*-- Name of trapezoids which froms one quadrant 
      VNQC = 'QC20'

*-- Define master volume for one Quadrant of QC pole-tip = 1/4 TUBE
      DSIZE(1) = RIN
      DSIZE(2) = ROUT
      DSIZE(3) = (QC2ZQC(2)-QC2ZQC(1))/2.0
      DSIZE(4) = 225.0
      DSIZE(5) = 315.0
      CALL GSVOLU('QC2Q','TUBS',IAIRJM,DSIZE,5,IVOL)
      IF (IVOL.LE.0) THEN
         CALL MSGOUT('QCGEOM','QC2Q illegal volume number',IVOL)
c      WRITE(6,*)
c     &  '!!! In QCGEOM ; failed GSVOLU for QC one quadrant !!!'  
      ENDIF
      CALL GSATT('QC2Q','SEEN',1)
C
C     Rotate each point describing pole-shape by -135degree so that trapezoid 
C     bottom line is parallel to X-axis ; QXYNQC => QXYNL (Left-side shape)
C
C      TH = -135.0*PI/180.0  
C      COSTH = COS(TH)
C      SINTH = SIN(TH)
C      DO 201   I=1,8
C         QXYNL(1,I) = QXYNQC(1,I)*COSTH - QXYNQC(2,I)*SINTH
C         QXYNL(2,I) = QXYNQC(1,I)*SINTH + QXYNQC(2,I)*COSTH
C  201 CONTINUE
C
C     Copy Left-side shape to Right-side shape by mirror reflection.
C      
C      DO 202 I=1,8
C         QXYNR(1,I) =-QXYNL(1,I)
C         QXYNR(2,I) = QXYNL(2,I)
C  202 CONTINUE

*-- Define 1st-7th Trapezoids
      DO 211 I=1,7
         DSIZE(1)  =(QC2ZQC(2)-QC2ZQC(1))/2.0
         DSIZE(2)  = 0.0
         DSIZE(3)  = 0.0
         DSIZE(4)  =(QXYNL(2,I)  -QXYNL(2,I+1))/2.0
         DSIZE(5)  =(QXYNR(1,I+1)-QXYNL(1,I+1))/2.0
         DSIZE(6)  =(QXYNR(1,I)  -QXYNL(1,I)  )/2.0
         DSIZE(7)  = 0.0
         DSIZE(8)  =(QXYNL(2,I)  -QXYNL(2,I+1))/2.0
         DSIZE(9)  =(QXYNR(1,I+1)-QXYNL(1,I+1))/2.0
         DSIZE(10) =(QXYNR(1,I)  -QXYNL(1,I)  )/2.0
         DSIZE(11) = 0.0
         WRITE(VNQC(4:4),'(I1)') I
         CALL GSVOLU(VNQC,'TRAP',IQC12,DSIZE,11,IVOL)
         IF (IVOL.LE.0) THEN
            WRITE(CMSG,'(A4,'' illegal volume number'')') VNQC 
            CALL MSGOUT('QCGEOM',CMSG(:LNBLNK(CMSG)),IVOL)
c         WRITE(6,*)
c     &  '!!! In QCGEOM ; failed GSVOLU for QC2',I,'-th trapezoid !!!'  
         ENDIF
         Y = (QXYNL(2,I)+QXYNL(2,I+1))/2.0         
         CALL GSPOS(VNQC,1,'QC2Q',0,Y,0,0,'ONLY')
         CALL GSATT(VNQC,'SEEN',1)
  211 CONTINUE

*-- Define QC2 volume
      DSIZE(1) = RIN
      DSIZE(2) = ROUT
      DSIZE(3) = (QC2ZQC(2)-QC2ZQC(1))/2.0
      CALL GSVOLU('QC2 ','TUBE',IAIRJM,DSIZE,3,IVOL)
      IF (IVOL.LE.0) THEN
         CALL MSGOUT('QCGEOM','QC2 illegal volume number',IVOL)
c         WRITE(6,*) 
c     &        '!!! In QCGEOM ; failed GSVOLU for QC2 Volume !!!'  
      ENDIF
      CALL GSATT('QC2 ','SEEN',1)

*-- Locate four Quadrants in QC2 Volume with rotation around Z-axis 
      DO 221 I=1,4
C         PHIDX = 45.0 + 90.0*FLOAT(I-1)
C         PHIDY = PHIDX + 90.0 
         IROT = NROTJM*IDETIR+10*IDQCIR+I
C         CALL GSROTM(IROT,90.0,PHIDX,90.0,PHIDY,0.0,0.0)
         CALL GSPOS('QC2Q',I,'QC2 ',0,0,0,IROT,'ONLY')
  221 CONTINUE

*-- Define 8th trapezoid. Note x-y exchange for 90deg rotation. 
      DSIZE(1)  =(QC2ZQC(2)-QC2ZQC(1))/2.0
      DSIZE(2)  = 0.0
      DSIZE(3)  = 0.0
      DSIZE(4)  =(QXY8QC(1)-QXY7QC(1))/2.0
      DSIZE(5)  = QXY8QC(2)
      DSIZE(6)  = QXY7QC(2)
      DSIZE(7)  = 0.0
      DSIZE(8)  =(QXY8QC(1)-QXY7QC(1))/2.0
      DSIZE(9)  = QXY8QC(2)
      DSIZE(10) = QXY7QC(2)
      DSIZE(11) = 0.0
      CALL GSVOLU('QC28','TRAP',IQC12,DSIZE,11,IVOL)
      IF (IVOL.LE.0) THEN
         CALL MSGOUT('QCGEOM','QC28 illegal volume number',IVOL)
c      WRITE(6,*)
c     &  '!!! In QCGEOM ; failed GSVOLU for QC2 8-th trapezoid !!!'  
      ENDIF
      CALL GSATT('QC28','SEEN',1)

*-- Locate 8-th trapezoid in QC2 volume
      D = (QXY7QC(1)+QXY8QC(1))/2.0         
      CALL GSPOS('QC28',1,'QC2 ',0,-D,0,IROT11,'ONLY')
      CALL GSPOS('QC28',2,'QC2 ', D,0,0,IROT12,'ONLY')
      CALL GSPOS('QC28',3,'QC2 ',0, D,0,IROT13,'ONLY')
      CALL GSPOS('QC28',4,'QC2 ',-D,0,0,IROT14,'ONLY')

*-- Define Coil in QC2 volume 
C      DSIZE(1) = (COIXQC(2)-COIXQC(1))/2.0
C      DSIZE(2) = (COIYQC(2)-COIYQC(1))/2.0
C      DSIZE(3) = (QC2ZQC(2)-QC2ZQC(1))/2.0
C      CALL GSVOLU('QCCH','BOX ',TMCOIL,DSIZE,3,IVOL)
C      DSIZE(1) = (COIYQC(2)-COIYQC(1))/2.0
C      DSIZE(2) = (COIXQC(2)-COIXQC(1))/2.0
C      DSIZE(3) = (QC2ZQC(2)-QC2ZQC(1))/2.0
C      CALL GSVOLU('QCCV','BOX ',TMCOIL,DSIZE,3,IVOL)

      X = (COIXQC(2)+COIXQC(1))/2.0
      Y = (COIYQC(2)+COIYQC(1))/2.0
      CALL GSPOS('QCCH',1,'QC2 ', X, Y,0,0,'ONLY')
      CALL GSPOS('QCCH',2,'QC2 ',-X, Y,0,0,'ONLY')
      CALL GSPOS('QCCH',3,'QC2 ',-X,-Y,0,0,'ONLY')
      CALL GSPOS('QCCH',4,'QC2 ', X,-Y,0,0,'ONLY')
      CALL GSPOS('QCCV',1,'QC2 ', Y, X,0,0,'ONLY')
      CALL GSPOS('QCCV',2,'QC2 ',-Y, X,0,0,'ONLY')
      CALL GSPOS('QCCV',3,'QC2 ',-Y,-X,0,0,'ONLY')
      CALL GSPOS('QCCV',4,'QC2 ', Y,-X,0,0,'ONLY')
***************** QC2 *****************

***************** QC0 *****************
      IF ( QC0ZQC(1).NE.QC0ZQC(2) ) THEN
*****************
*-- Name of trapezoids which froms one quadrant 
      VNQC = 'QC00'

*-- Define master volume for one Quadrant of QC pole-tip = 1/4 TUBE
      DSIZE(1) = RIN
      DSIZE(2) = ROUT
      DSIZE(3) = (QC0ZQC(2)-QC0ZQC(1))/2.0
      DSIZE(4) = 225.0
      DSIZE(5) = 315.0
      CALL GSVOLU('QC0Q','TUBS',IAIRJM,DSIZE,5,IVOL)
      IF (IVOL.LE.0) THEN
         CALL MSGOUT('QCGEOM','QC0Q illegal volume number',IVOL)
c     WRITE(6,*)
c     &  '!!! In QCGEOM ; failed GSVOLU for QC0 one quadrant !!!'  
      ENDIF
      CALL GSATT('QC0Q','SEEN',1)

*-- Define 1st-7th Trapezoids
      DO 311 I=1,7
         DSIZE(1)  =(QC0ZQC(2)-QC0ZQC(1))/2.0
         DSIZE(2)  = 0.0
         DSIZE(3)  = 0.0
         DSIZE(4)  =(QXYNL(2,I)  -QXYNL(2,I+1))/2.0
         DSIZE(5)  =(QXYNR(1,I+1)-QXYNL(1,I+1))/2.0
         DSIZE(6)  =(QXYNR(1,I)  -QXYNL(1,I)  )/2.0
         DSIZE(7)  = 0.0
         DSIZE(8)  =(QXYNL(2,I)  -QXYNL(2,I+1))/2.0
         DSIZE(9)  =(QXYNR(1,I+1)-QXYNL(1,I+1))/2.0
         DSIZE(10) =(QXYNR(1,I)  -QXYNL(1,I)  )/2.0
         DSIZE(11) = 0.0
         WRITE(VNQC(4:4),'(I1)') I
         CALL GSVOLU(VNQC,'TRAP',IQC12,DSIZE,11,IVOL)
         IF (IVOL.LE.0) THEN
            WRITE(CMSG,'(A4,'' illegal volume number'')') VNQC 
            CALL MSGOUT('QCGEOM',CMSG(:LNBLNK(CMSG)),IVOL)
c         WRITE(6,*)
c     &  '!!! In QCGEOM ; failed GSVOLU for QC0',I,'-th trapezoid !!!'  
         ENDIF

         Y = (QXYNL(2,I)+QXYNL(2,I+1))/2.0         
         CALL GSPOS(VNQC,1,'QC0Q',0,Y,0,0,'ONLY')
         CALL GSATT(VNQC,'SEEN',1)
  311 CONTINUE

*-- Define QC0 volume
      DSIZE(1) = RIN
      DSIZE(2) = ROUT
      DSIZE(3) = (QC0ZQC(2)-QC0ZQC(1))/2.0
      CALL GSVOLU('QC0 ','TUBE',IAIRJM,DSIZE,3,IVOL)
      IF (IVOL.LE.0) THEN
         CALL MSGOUT('QCGEOM','QC0 illegal volume number',IVOL)
c      WRITE(6,*)
c     &  '!!! In QCGEOM ; failed GSVOLU for QC0 Volume !!!'  
      ENDIF
      CALL GSATT('QC0 ','SEEN',1)

*-- Locate four Quadrants in QC0 Volume with rotation around Z-axis 
      DO 321 I=1,4
         IROT = NROTJM*IDETIR+10*IDQCIR+I
         CALL GSPOS('QC0Q',I,'QC0 ',0,0,0,IROT,'ONLY')
  321 CONTINUE

*-- Define 8th trapezoid. Note x-y exchange for 90deg rotation. 
      DSIZE(1)  =(QC0ZQC(2)-QC0ZQC(1))/2.0
      DSIZE(2)  = 0.0
      DSIZE(3)  = 0.0
      DSIZE(4)  =(QXY8QC(1)-QXY7QC(1))/2.0
      DSIZE(5)  = QXY8QC(2)
      DSIZE(6)  = QXY7QC(2)
      DSIZE(7)  = 0.0
      DSIZE(8)  =(QXY8QC(1)-QXY7QC(1))/2.0
      DSIZE(9)  = QXY8QC(2)
      DSIZE(10) = QXY7QC(2)
      DSIZE(11) = 0.0
      CALL GSVOLU('QC08','TRAP',ICFRP,DSIZE,11,IVOL)
      IF (IVOL.LE.0) THEN
         CALL MSGOUT('QCGEOM','QC08 illegal volume number',IVOL)
c      WRITE(6,*)
c     &  '!!! In QCGEOM ; failed GSVOLU for QC0 8-th trapezoid !!!'  
      ENDIF
      CALL GSATT('QC08','SEEN',1)

*-- Locate 8-th trapezoid in QC0 volume
      D = (QXY7QC(1)+QXY8QC(1))/2.0         
      CALL GSPOS('QC08',1,'QC0 ',0,-D,0,IROT11,'ONLY')
      CALL GSPOS('QC08',2,'QC0 ', D,0,0,IROT12,'ONLY')
      CALL GSPOS('QC08',3,'QC0 ',0, D,0,IROT13,'ONLY')
      CALL GSPOS('QC08',4,'QC0 ',-D,0,0,IROT14,'ONLY')
*************
      ENDIF
***************** QC0 *****************
*-- Define +z side QC Master volume
      DSIZE(1) = RIN
      DSIZE(2) = ROUT
      DSIZE(3) = QC2ZQC(2)
      CALL GSVOLU('QCPZ','TUBE',IAIRJM,DSIZE,3,IVOL)
      IF (IVOL.LE.0) THEN
         CALL MSGOUT('QCGEOM','QCPZ illegal volume number',IVOL)
C      WRITE(6,*)
C     &  '!!! In QCGEOM ; failed GSVOLU for +Z QC Master Volume !!!'  
      ENDIF
      CALL GSATT('QCPZ','SEEN',0)

*-- Locate two QC's in +z side QC master volume
      IF (QC0ZQC(1).NE.QC0ZQC(2)) THEN
         Z = (QC0ZQC(2)+QC0ZQC(1))/2.0
         CALL GSPOS('QC0 ',1,'QCPZ',0,0,Z,0,'ONLY')
      ENDIF
      Z = (QC1ZQC(2)+QC1ZQC(1))/2.0
      CALL GSPOS('QC1 ',1,'QCPZ',0,0,Z,0,'ONLY')
      Z = (QC2ZQC(2)+QC2ZQC(1))/2.0
      CALL GSPOS('QC2 ',1,'QCPZ',0,0,Z,0,'ONLY')

*-- Define -z side QC Master volume
      DSIZE(1) = RIN
      DSIZE(2) = ROUT
      DSIZE(3) = QC2ZQC(2)
      CALL GSVOLU('QCMZ','TUBE',IAIRJM,DSIZE,3,IVOL)
      IF (IVOL.LE.0) THEN
         CALL MSGOUT('QCGEOM','QCMZ illegal volume number',IVOL)
c      WRITE(6,*)
c     &  '!!! In QCGEOM ; failed GSVOLU for -Z QC Master Volume !!!'  
      ENDIF
      CALL GSATT('QCMZ','SEEN',0)

*-- Locate two QC's in -z side QC master volume
      IF (QC0ZQC(1).NE.QC0ZQC(2)) THEN
         Z =-(QC0ZQC(2)+QC0ZQC(1))/2.0
         CALL GSPOS('QC0 ',1,'QCMZ',0,0,Z,0,'ONLY')
      ENDIF
      Z =-(QC1ZQC(2)+QC1ZQC(1))/2.0
      CALL GSPOS('QC1 ',1,'QCMZ',0,0,Z,0,'ONLY')
      Z =-(QC2ZQC(2)+QC2ZQC(1))/2.0
      CALL GSPOS('QC2 ',1,'QCMZ',0,0,Z,0,'ONLY')

*-- Locate QC +Z/-Z master volume in Detector master volume 
*-- with crossing angle CRSANG
      CRSDEG = CRSAIR/PI*180.0
      THX = 90.0 + CRSDEG
      FAIX= 0.0
      THY = 90.0
      FAIY= 90.0
      THZ = CRSDEG
      FAIZ= 0.0
      IROT = NROTJM*IDETIR+10*IDQCIR+21
      Z = 0.
      CALL GSROTM(IROT,THX,FAIX,THY,FAIY,THZ,FAIZ)
      CALL GSPOS('QCPZ',1,'MAST',0.,0.,Z,IROT,'MANY')
      THX = 90.0 - CRSDEG
      FAIX= 0.0
      THY = 90.0
      FAIY= 90.0
      THZ = CRSDEG
      FAIZ= 180.0
      IROT = NROTJM*IDETIR+10*IDQCIR+22
      Z = 0.
      CALL GSROTM(IROT,THX,FAIX,THY,FAIY,THZ,FAIZ)
      CALL GSPOS('QCMZ',1,'MAST',0,0,Z,IROT,'MANY')

      RETURN
      END
      
