      SUBROUTINE SMFHD1(P,X0,IPART,EK0,AMASS,MAXLX,MAXLY,MAXV,
     &           DELOTO,VELOSS,DESHTO,ESHWER,VENE,ISTOP,INUCLI,RM)
* This routine is intended to be used for the secondary hadron
* The first, interaction position is decided
* For charged particle energy loss
* For hadron interaction, the energy deposit is just in the block where
* the interaction occurs.
*
* IPART =
* 8,9 Pion like   : charged and mass is also deposited
* 14  Proton like : charged and only kinetic energy deposited
* 13  Neutron like: neutral and only kinetic energy deposited
* 10  K0L like    : neutral and mass is also deposited (anti baryon as well)
* 5,6 Muon like   : 
* 4   neutrino    :
*************************************************************************
      IMPLICIT NONE
*   IPART=13,14,8,9 available now  (neutron,proton,pi+-)
      INTEGER IPART
      REAL    EK,AMASS,EK0,ESHWER
      REAL P(*)
      REAL X0(*)
      REAL PTOT,RM(3,3)
      REAL ZI,POSI(3),POSV(3)
      INTEGER IDVCUR,IDXCUR,IDYCUR,I,IDV,IDX,IDY
      REAL    ZS1,ZS2,ZZERO

      REAL ZIPOS,RANGE,DE,DERANGE
      INTEGER ISTOP
      REAL DZELOS,ZLOMAX,PASSEL
      PARAMETER (ZLOMAX = 200.0)

      REAL DELOTO,DESHTO,RDUM,DE2,DE1
      INTEGER INUCLI,ICHRGE 

      INTEGER    IFILL
      INTEGER    IHADRN,IELECT

      INTEGER    IUMASS
*-- for nuclear interaction, mass fraction
      REAL       RINTPI
      PARAMETER (RINTPI = 0.5)
*-- almost always particle stop in Pb
*-- So just 10 % of Mass energy deposit
*-- if particle is not proton nor neutron
*-- no special treatment for nuclear absorption for pi-
      REAL       STOPPI
      PARAMETER (STOPPI = 0.1)

      INTEGER MAXV
      INTEGER MAXLX,MAXLY
      REAL  VENE(MAXLX,MAXLY,MAXV)
      REAL  VELOSS(MAXLX,MAXLY,MAXV)
      
      INTEGER IX,IY,IV

      DELOTO = 0.
      DESHTO = 0.
      ESHWER   = 0.
      ISTOP = 0
      INUCLI= 0
*-- initialyse 0 energy deposit
C      CALL VZERO(VENE,MAXV*MAXLX*MAXLY)
C      CALL VZERO(VELOSS,MAXV*MAXLX*MAXLY)
      DO 100 IV=1,MAXV
         DO 110 IY=1,MAXLY
            DO 120 IX=1,MAXLX
               VENE(IX,IY,IV) = 0.
               VELOSS(IX,IY,IV) = 0.
 120        CONTINUE
 110     CONTINUE
 100  CONTINUE

*-- first check
      PTOT = SQRT(P(1)**2+P(2)**2+P(3)**2)
      IF (PTOT.LE.0.) THEN
         CALL MSGOUT('SMFHD0','ZERO MOMENTUM detected',1)
C         write(*,*)' ZERO MOMENTUM detected '
         RETURN
      ENDIF

* 8,9 Pion like   : charged and mass is also deposited
* 14  Proton like : charged and only kinetic energy deposited
* 13  Neutron like: neutral and only kinetic energy deposited
* 10  K0L like    : neutral and mass is also deposited (anti baryon as well)
* 5,6 Muon like   : 
* 4   neutrino    :

      IF (IPART.EQ.8.OR.IPART.EQ.9) THEN
         ICHRGE = 1
         IUMASS = 1
         IHADRN = 1
         IELECT = 0
      ELSEIF (IPART.EQ.14) THEN
         ICHRGE = 1
         IUMASS = 0
         IHADRN = 1
         IELECT = 0
      ELSEIF (IPART.EQ.13) THEN
         ICHRGE = 0
         IUMASS = 0
         IHADRN = 1
         IELECT = 0
      ELSEIF (IPART.EQ.10) THEN
         ICHRGE = 0
         IUMASS = 1
         IHADRN = 1
         IELECT = 0
      ELSEIF (IPART.EQ.5.OR.IPART.EQ.6) THEN
         ICHRGE = 1
*-- since muon decay is too late, no e response in time
         IUMASS = 0
         IHADRN = 0
         IELECT = 0
      ELSEIF (IPART.EQ.4) THEN
         ICHRGE = 0
         IUMASS = 0
         IHADRN = 0
         IELECT = 0

         RETURN

      ELSEIF (IPART.LE.3) THEN
         ICHRGE = 1
         IUMASS = 0
         IHADRN = 0
         IELECT = 1
         CALL MSGOUT('SMFHD1',
     &        'EM particle should be taken care of by EM routine',2)
C         WRITE(*,*)'EM particle should be taken care of by EM routine'
         RETURN
      ELSE
         CALL MSGOUT('SMFHD0','IPART not supported for this routine',
     &        IPART)
C         write(*,*)' IPART not supported for this routine',IPART
         RETURN
      ENDIF
         

*-- initialization
*-- kinetic energy
      EK = EK0
      
*--  interaction position determination (random generator)
      IF (IHADRN.EQ.1) THEN
         CALL SMPGEN(EK,AMASS,IPART,ZIPOS)
      ELSE
         ZIPOS = 1.E+09
      ENDIF

*-- for charged particle
      IF (ICHRGE.EQ.1) THEN 
         CALL SMIELO(EK,AMASS,ZIPOS, RANGE,DERANGE,ISTOP)
C*-- total energy loss 
C               DE    = DERANGE
         IF (ISTOP.EQ.1) THEN
*-- just energy loss, hence we have to set ZIPOS to range.
            ZIPOS = RANGE
            EK    = 0.0
*-- no more hadron interaction
            INUCLI= 0
         ELSE
            IF (IHADRN.EQ.1) INUCLI= 1
         ENDIF
      ELSE
*-- neutron
         ISTOP = 0
*-- first always intended to be interact
         IF (IHADRN.EQ.1) INUCLI= 1
C*-- no range (neutron)
C         RANGE = 1.E+10
         DE    = 0.0
      ENDIF

*-- check the hadron interaction (including stop) happen inside
*-- detector?
      DO 210 I=1,3
*-- just for 3rd axis matrix calc(fast)
         POSV(I)=X0(I)+RM(3,I)*ZIPOS 
 210  CONTINUE
*-- determine the internal ID for detector coordinate
CKI      CALL SMPCAL(POSV,IDV,IDX,IDY)
      IF(MAXLX.EQ.128) THEN
         CALL SMPEND(POSV,IDV,IDX,IDY)
      ELSE
         CALL SMPBAR(POSV,IDV,IDX,IDY)
      ENDIF
      IF (IDV.LE.0 .OR. IDX.LE.0 .OR. IDY.LE.0) THEN 
*-- reset stop/nucl-int (just energy loss or pass through)
*-- not stop : out of boundary
         ISTOP = 0
*-- no hadron interaction
         INUCLI = 0
      ENDIF

*== STEP by STEP filling for energyloss part 
      IF (ICHRGE.NE.1) GOTO 777

*-- set 0 for total pass length
      PASSEL = 0.

*-- initialyiation for loop
*-- start position
CKI      CALL SMPCAL(X0,IDV,IDX,IDY)
      IF(MAXLX.EQ.128) THEN
         CALL SMPEND(X0,IDV,IDX,IDY)
      ELSE
         CALL SMPBAR(X0,IDV,IDX,IDY)
      ENDIF
      IDVCUR = IDV
      IDXCUR = IDX
      IDYCUR = IDY
      
*-- Energy Loss part

      ZZERO  = 0.0
      DZELOS = 1.0
      ZI     = 0.0

*== LOOP START
 200  CONTINUE
*-- Get sub-detector coordinate position

*-- where is next step ?
      ZI = ZI + DZELOS
      DO 220 I=1,3
         POSV(I)=X0(I)+RM(3,I)*ZI 
 220  CONTINUE
*-- determine the internal ID for detector coordinate
*-- next detector
CKI      CALL SMPCAL(POSV,IDV,IDX,IDY)
      IF(MAXLX.EQ.128) THEN
         CALL SMPEND(POSV,IDV,IDX,IDY)
      ELSE
         CALL SMPBAR(POSV,IDV,IDX,IDY)
      ENDIF

      IFILL = 0
      
*-- next volume is out of boundary
      IF (IDV.EQ.0 .OR. IDX.EQ.0 .OR. IDY.EQ.0) THEN
         IFILL = 1
*-- current pos to next-step pos is the last step
      ELSEIF (ZI.GT.MIN(ZIPOS,ZLOMAX)) THEN
         IFILL = 2
*-- next pos is inside different detector
      ELSEIF (IDV.NE.IDVCUR .OR. IDX.NE.IDXCUR .OR.
     &     IDY.NE.IDYCUR) THEN
         IFILL = 3
      ENDIF

*-- compare current volume and next volume
      IF (IFILL.GT.0) THEN
         ZS1=ZZERO
         IF (IFILL.EQ.1.OR.IFILL.EQ.3) THEN
            ZS2 = ZI-0.5*DZELOS
            CALL SMIELO(EK0,AMASS,ZS2,RDUM,DE2,ISTOP)
            CALL SMIELO(EK0,AMASS,ZS1,RDUM,DE1,ISTOP)
            DE=ABS(DE2-DE1)
            PASSEL = PASSEL + ZS2 - ZS1
         ELSEIF (IFILL.EQ.2) THEN
            ZS2 = MIN(ZIPOS,ZLOMAX)
            IF (ISTOP.EQ.1) THEN
               DE = MAX(EK0-DELOTO,0.)
               IF (IUMASS.EQ.1) THEN
                  DE = DE + STOPPI * AMASS
                  PASSEL = ZS2
               ENDIF
            ELSE
               ZS2 = MIN(ZIPOS,ZLOMAX)
               CALL SMIELO(EK0,AMASS,ZS2,RDUM,DE2,ISTOP)
               CALL SMIELO(EK0,AMASS,ZS1,RDUM,DE1,ISTOP)
               DE=ABS(DE2-DE1)
               PASSEL = MIN(ZIPOS,ZLOMAX)
            ENDIF
         ENDIF
*-- calculate integrated Eloss from ZS1 to ZS2
         IF (IDXCUR.GT.0.AND.IDYCUR.GT.0.AND.IDVCUR.GT.0) 
     &        VELOSS(IDXCUR,IDYCUR,IDVCUR)=
     &        VELOSS(IDXCUR,IDYCUR,IDVCUR)+DE

         DELOTO = DELOTO + DE

*-- exit from loop
         IF (IFILL.EQ.1.OR.IFILL.EQ.2) GOTO 201
            
         IF (IDV.NE.IDVCUR)   IDVCUR=IDV
         IF (IDX.NE.IDXCUR) IDXCUR=IDX
         IF (IDY.NE.IDYCUR) IDYCUR=IDY
         ZZERO=ZS2
      ENDIF

*-- setup next step size 
      IF (POSV(3).GT.40.0) THEN
         DZELOS=2.0
      ELSE
         DZELOS=1.0
      ENDIF

      GOTO 200

 201  CONTINUE

      IF (ISTOP.EQ.1) THEN
         EK = 0.
      ELSE
         EK = EK0 - DELOTO
      ENDIF
*-- new energy

 777  CONTINUE

      IF (IHADRN.NE.1 .OR. INUCLI.NE.1)  GOTO 888

*== For hadron part JUST DEPOSIT AT THE INTERACTION POINT !!
*-- Move from shwer to particle coordinate
      POSI(3) = ZIPOS
*-- Get sub-detector coordinate position
      DO 230 I=1,3
         POSV(I)=X0(I)+RM(3,I)*POSI(3) 
 230  CONTINUE
*-- determine the internal ID for detector coordinate
CKI      CALL SMPCAL(POSV,IDV,IDX,IDY)
      IF(MAXLX.EQ.128) THEN
         CALL SMPEND(POSV,IDV,IDX,IDY)
      ELSE
         CALL SMPBAR(POSV,IDV,IDX,IDY)
      ENDIF

      DE = EK
      IF (IUMASS.EQ.1) THEN
         DE = DE + RINTPI * AMASS
      ENDIF
      IF (IDX.GT.0.AND.IDY.GT.0.AND.IDV.GT.0) THEN
         VENE(IDX,IDY,IDV)=
     &        VENE(IDX,IDY,IDV)+ DE
      ENDIF

      DESHTO = DE
      ESHWER = DE

 888  CONTINUE
      
      RETURN
      END
