      SUBROUTINE VXGEOM
*.
*...VXGEOM   Define the micro vertex detector
*.           geometry and tracking media.
*.
*. INPUT     : none
*. OUTPUT    : none
*.
*. SEQUENCE  :
*. COMMON    : GCFLAG GCONST VXGEO UCTMED
*. CALLS     : GSVOLU GSDVN  GSPOS GSDET  GSDETH GSDETD GPSETS
*. CALLED    : UGEOM
*.
*. BANKS L   :
*. BANKS U   :
*. BANKS M   :
*. BANKS D   :
*.
*. REPORT CONDITIONS
*.
*. AUTHOR    : Naoyasu HOSODA
*. VERSION   : 1.00
*. CREATED   : 12-Mar-1994
*. LAST MOD  : 25-Apr-1995
*.
*. MODIFICATION LOG
*. 15-Apr-1994  N.Hosoda  Add middle part
*. 19-Apr-1994  N.Hosoda  Add detector response package
*. 18-May-1994 replace geant common by INCLUDE
*. 18-May-1994  N.Hosoda Add vxgeo.h
*.    SITKVX = Si thickness
*.    PXCLVX = pixcel size
*.    NLAYVX = Number of layers
*.    RVX(1) = 1st layer radius
*.    ZVX(1) = 1st layer half Z length
*.    RVX(2) = 2nd layer radius
*.    ZVX(2) = 2st layer half Z length
*.    RVX(3) = 3rd layer radius
*.    ZVX(3) = 3st layer half Z length
*. 27-Jul-1994  N.Hosoda 
*. 25-Apr-1995  Y.Fujii,  Divide Silicon to sensitive part and backplane part.
*             DLTKVX=Depletion Layer Thickness (Sensitive)
*.            SITKVX=total silicon thickness.
*.            Correct z-division mistake ( zvx/pxclvx => zvx*2/pxclvx )
*. 11-May-1995  Y.Fujii,  Added precision tracking air layers inside
*.                      of silicon for precision pixel tracking.
*.
*. 15-Dec-1998  A.Miyamoto  Rewritten for more than 3 VTX layer.
*.    In this routine, following geometry is assumed.
*.    VTX consists of "NLAYVX" layers. Each layer consists of three region,
*.
*. 27-Oct-1999  J.Kanzaki
*.    Scale factor for energy loss is changed from 1.E6 to 1.E4
*.
*.**********************************************************************
      IMPLICIT NONE

*--   Common for GEANT
#include "geant321/gcflag.inc"
#include "geant321/gconst.inc"

*--   Common for jim
#include "jmmed.inc"
#include "jmdet.inc"
#include "jmmat.inc"
#include "vxgeo.inc"
#include "vxsim.inc"
#include "vxhit.inc"
      
      INTEGER ISILI,IAIRP,ISIBK
      
*-- Tracking media parameters for Si
      REAL STEMXS,EPSILS,STMINS
      PARAMETER ( STEMXS=0.01 )
      PARAMETER ( EPSILS=0.0001)
      PARAMETER ( STMINS=0.0001)

*-- Tracking media parameters for Si backplane.
      REAL STEMXB,EPSILB,STMINB
      PARAMETER ( STEMXB=0.5 )
      PARAMETER ( EPSILB=0.001 )
      PARAMETER ( STMINB=0.001 )

      REAL PAR(12)
c ... mlayvx is max number of VTX layer.
      CHARACTER*4 NAMDVX(MLAYVX), NAMSVX(MLAYVX)
      CHARACTER*4 NAMAVX(MLAYVX), NAMZVX(MLAYVX)
      CHARACTER*4 NAMPVX(MLAYVX)
      
      CHARACTER*4 CHNMSV(NDSVVX)
      INTEGER NBITSV(NDSVVX)
      DATA NBITSV /     8,    16,    16/

      CHARACTER*4 CHNAMH(NPHTVX)
      INTEGER NBITSH(NPHTVX)
      REAL ORIG(NPHTVX),FACT(NPHTVX)
C      DATA CHNAMH /'PCOD','X   ','Y   ','Z   ','P   ','ELOS'/
C      DATA NBITSH /    16,    32,    32,    32,    32,    32/
C      DATA ORIG   /30000., 5000., 5000., 5000.,    0.,    0./
C      DATA FACT   /    1.,10000.,10000.,10000.,10000.,1000./
      DATA CHNAMH / 
     &     'PCOD','X   ','Y   ','Z   ','P   ',
     &     'Px  ','Py  ','Pz  ','PCDM',
     &     'Xm  ','Ym  ','Zm  ','Px-m','Py-m','Pz-m','ELOS'/
C      DATA NBITSH/ 1*16,3*32,32,3*32,1*16,6*32,32 /
C      DATA ORIG/ 30000.,3*100.,0.,3*500.,30000.,3*500.,3*500.,0./
C      DATA FACT/ 1., 3*1.E6,1.E6, 3*1.E6,1.,3*1.E6,3*1.E6,1.E6/
      DATA NBITSH/
     &     16    ,32    ,32    ,32    ,32    ,
     &     32    ,32    ,32    ,16    ,
     &     32    ,32    ,32    ,32    ,32    ,32    ,32/
      DATA ORIG/
     &     30000.,100.  ,100.  ,100.  ,0.    ,
     &     500.  ,500.  ,500.  ,30000.,
     &     500.  ,500.  ,500.  ,500.  ,500.  ,500.  ,0./
      DATA FACT/
     &     1.    ,1.E6  ,1.E6  ,1.E6  ,1.E6  ,
     &     1.E6  ,1.E6  ,1.E6  ,1.    ,
     &     1.E6  ,1.E6  ,1.E6  ,1.E6  ,1.E6  ,1.E6  ,1.E4/
c note   digi.data = NINT( ( ORIG + hit )*FACT  )  < 32,000 for 16 bits
c                                                  < 2,000,000,000 for 32 bits.
      
      REAL PRCAIR
      DATA PRCAIR / 0.01 /

      INTEGER IVOL
      INTEGER IDET,ISET,IDTYPE

      INTEGER NDIV

      INTEGER I

      IF (ISIMJD(IDETVX).EQ.ISKPJD) RETURN
      
      ISILI = NMEDJM*IDETVX+1
      CALL GSTMED(ISILI, 'SILICON', IMSILI, ISENS, IFIELD, FIELDM,
     &     TMAXFD, STEMXS, DEEMAX, EPSILS,STMINS,UBUF, NWBUF)
      
      IAIRP = NMEDJM*IDETVX+2
      CALL GSTMED(IAIRP, 'PRECISE AIR', IMAIR, NOSENS, IFIELD, FIELDM,
     &     TMAXFD, STEMXS, DEEMAX, EPSILS,STMINS,UBUF, NWBUF)

      ISIBK = NMEDJM*IDETVX+3
      CALL GSTMED(ISIBK, 'SILICON2', IMSILI, NOSENS, IFIELD, FIELDM,
     &     TMAXFD, STEMXB, DEEMAX, EPSILB,STMINB,UBUF, NWBUF)

*--   Define a volume with a given name, shape, tracking medium number 
*--   and shape parameters
*--   VX master
      PAR(1)=RVX(1)-PRCAIR
      PAR(2)=RVX(NLAYVX)+SITKVX 
      PAR(3)=ZVX(NLAYVX)      
      CALL GSVOLU('VXMA', 'TUBE', IAIRJM, PAR, 3, IVOL)
      CALL GSPOS('VXMA', 1, 'MAST', 0., 0., 0., 0, 'MANY')
      CALL GSATT('VXMA','SEEN', 0)

      IF (IBNKVX.EQ.ISBKVX) THEN
         
         CHNAMH(NSHTVX) = CHNAMH(NLHTVX)
         NBITSH(NSHTVX) = NBITSH(NLHTVX)
         ORIG  (NSHTVX) = ORIG  (NLHTVX)
         FACT  (NSHTVX) = FACT  (NLHTVX)

         NDHTVX = NSHTVX

      ELSEIF (IBNKVX.EQ.ILBKVX) THEN

         NDHTVX = NLHTVX

      ENDIF

*-- VX Geometry
      
      DO 100 I = 1, NLAYVX
*--   Set region name according to the NLAYVX
         WRITE(NAMDVX(I),'(''VX'',I1,''D'')') I
         WRITE(NAMSVX(I),'(''VX'',I1,''S'')') I
         WRITE(NAMAVX(I),'(''VX'',I1,''A'')') I
         WRITE(NAMZVX(I),'(''VX'',I1,''Z'')') I
         WRITE(NAMPVX(I),'(''VX'',I1,''P'')') I
         PAR(1)=RVX(I)
         PAR(2)=RVX(I)+DLTKVX
         PAR(3)=ZVX(I)
         CALL GSVOLU(NAMDVX(I),'TUBE',ISILI, PAR, 3, IVOL)
         PAR(1)=RVX(I)+DLTKVX
         PAR(2)=RVX(I)+SITKVX
         PAR(3)=ZVX(1)
         CALL GSVOLU(NAMSVX(I),'TUBE',ISIBK, PAR, 3, IVOL)
         PAR(1)=RVX(I)-PRCAIR
         PAR(2)=RVX(I)
         PAR(3)=ZVX(I)
         CALL GSVOLU(NAMAVX(I),'TUBE',IAIRP, PAR, 3, IVOL)

*--   Division of a volume into a given number of cells
*--   Divide cells for z-direction
         NDIV=NINT(ZVX(I)*2./PXCLVX)
         CALL GSDVN(NAMZVX(I), NAMDVX(I), NDIV, 3)

*--   Divide cells for phi-direction
         NDIV=NINT(2.*PI*RVX(I)/PXCLVX)
         CALL GSDVN(NAMPVX(I), NAMZVX(I), NDIV, 2)

*--   Places a copy of the volume with user number inside its mother 
*--   volume
         CALL GSPOS(NAMDVX(I), I, 'VXMA', 0., 0., 0., 0, 'ONLY')
         CALL GSPOS(NAMSVX(I), I, 'VXMA', 0., 0., 0., 0, 'ONLY')
         CALL GSPOS(NAMAVX(I), I, 'VXMA', 0., 0., 0., 0, 'ONLY')

*--   Assign detector to the set VX and defines its basic params.
         CHNMSV(1) = NAMDVX(I)
         CHNMSV(2) = NAMZVX(I)
         CHNMSV(3) = NAMPVX(I)
         IDTYPE = NDETJM*IDETVX+I
         CALL GSDET('VX  ',NAMPVX(I),NDSVVX,CHNMSV,NBITSV,IDTYPE,
     &        NHMXVX,0,ISET,IDET)

*--   Define hit parameters for detector of set VX.
         CALL GSDETH('VX  ',NAMPVX(I),NDHTVX,CHNAMH,NBITSH,ORIG,FACT)
         VNAMVX(I)=NAMPVX(I)

*--   Change the attribute of the volume called NAME to the value
*--   all volume set to seen
        CALL GSATT(NAMDVX(I),'SEEN',-2)

 100  CONTINUE

*--   Print SET and DETector parameters
      IF (ISWIT(3) .GE. 1) THEN
        CALL GPSETS('VX  ','*   ')
      ENDIF

      RETURN
      END
