C   15/11/92 211150502  MEMBER NAME  SMRHLX   (S)        M    FORTRAN
C***********************************************************************
C*
C*===============================----------===
C* Subroutine SMRHLX(PLIST,HLXCT,HLSMR,IRET)
C*===============================----------===
C*
C* (Purpose)
C*     Smear helix parameters, taking track-parameter-dependence
C*     into account.
C* (Inputs)
C*        PLIST(1-20) : (R*4) :  particle list (20 words)
C*        HLXCT(1- 8) : (R*4) :  exact helix parameters.
C*                            :  (d_r,phi_0,kappa,d_z,tan(lambda))
C* (Output)
C*        HLSMR(1- 8) : (R*4) :  smeared track parameters.
C*             (9-38) : (R*8) :  error matrix ( lower triangle ).
C*             (  39) : (R*4) :  not used.
C*             (  40) : (R*4) :  not used.
C*             (  41) : (I*2) :  # degrees of freedom   / not used.
C*             (  42) : (I*2) :  # track parameters = 5 / not used.
C*             (  43) : (I*2) :  not used               / not used.
C*             (  44) : (R*4) :  R1.
C*             (  45) : (R*4) :  R2.
C*        IRET        : (I*4) :  return code.
C*                            :  = 0 ; OK.
C*                            :  < 0 ; un-trackable.
C* (Relation)
C*     Called by SMRCHT.
C*     Calls the following routines;
C* (Updaterecord)
C*   06-Jul-95 Nakamura Isamu     Replace RNDM with RANNOR
C*     4/18/90  K.Fujii           Original version.
C*     5/25/90  K.Fujii           Heavily modified to take parameter
C*                                dependence and error correlation
C*                                into account.
C*     6/07/90  K.Fujii           Modified to store error matrix and
C*                                related information.
C*     7/16/92  K.Fujii           Fixed a bug concerning shallow
C*                                tracks.
C*
C***********************************************************************
      SUBROUTINE SMRHLX(KS,DL,NHIT,HLXCT,HLSMR,IRET)

      IMPLICIT NONE

#include "jlfeld.inc"
#include "smrres.inc"
#include "smrtgo.inc"

      INTEGER KS
      REAL    DL,HLXCT(8),HLSMR(8)
      INTEGER IRET

      REAL EPSL, PHI, XV(3), PV(3), RCY(2), RNDM1,RNDM2
      DOUBLE PRECISION SGB(5),DADB1(3,3),DADB2(2,2),SGBS(5),DADB1S(3,3),
     &  DADB2S(2,2), ERROR(15)
      PARAMETER (EPSL=0.1)

      REAL RDATA(10)
      INTEGER IDATA(10)
      EQUIVALENCE ( RDATA, IDATA )

      DOUBLE PRECISION RCY1,RCY2,ZCY1,ZCY2
      DOUBLE PRECISION ARTEST,TNTEST,TNTST2
      INTEGER NHIT,IRT

      DOUBLE PRECISION DR,FI0,CPA,TNL
      DOUBLE PRECISION X0,Y0,Z0,R,AR,RDR,CSF0,SNF0
      DOUBLE PRECISION XC,YC,R1

      DOUBLE PRECISION RDK
      DOUBLE PRECISION R2
      DOUBLE PRECISION DEV

      INTEGER LVEC
      PARAMETER (LVEC=5)
      REAL RVEC(LVEC)
      
      INTEGER I,J,N,M
      
      INTEGER NCALL
      DATA NCALL / 0 /
      SAVE NCALL
      
C========< Entry Point >================================================
      
      
C  Reset return flag.
      
      IRET = -1
      
C  Initialization.
      
      IF(NCALL.EQ.0 ) THEN
         NCALL  = 1
         RCY1   = RCYTK(1)
         RCY2   = RCYTK(2)
         ZCY1   = ZCYTK(1)
         ZCY2   = ZCYTK(2)
         ARTEST = RCY1 + RCY2
         TNTEST = 0.8D0*ZCY2/RCY2
         TNTST2 = ZCY2/RCY1
         CALL SMRHLE(NSMPTK,DSMPTK,SIGRF,SIGZE,SGBS,DADB1S,DADB2S,IRT)
         IF ( IRT.LT.0 ) THEN
            CALL MSGOUT('SMRHLX',
     &           'Error return from SMRHLE -> STOP',IRT)
            PRINT *, '       NSMPTK = ', NSMPTK
            PRINT *, '       DSMPTK = ', DSMPTK
            PRINT *, '       SIGRF  = ', SIGRF
            PRINT *, '       SIGZE  = ', SIGZE
            PRINT *, '       PTOR   = ', PTOR
            STOP
         ENDIF
      ENDIF
      
C  Decide radii of 1-st and last sampling points.
      
      DR   = HLXCT(1)
      FI0  = HLXCT(2)
      CPA  = HLXCT(3)
      TNL  = HLXCT(5)
      X0   = HLXCT(6)
      Y0   = HLXCT(7)
      Z0   = HLXCT(8)
      R    = PTOR/CPA
      AR   = ABS(R)
      RDR  = R + DR
      CSF0 = COS(FI0)
      SNF0 = SIN(FI0)
      XC   = X0 + RDR*CSF0
      YC   = Y0 + RDR*SNF0

C  Check starting point.
      
      R1   = SQRT(X0**2+Y0**2)
      IF ( R1.GT.RCY2 .OR. Z0.LT.ZCY1 .OR.
     &     Z0.GT.ZCY2 ) RETURN
      R1   = MAX(R1,RCY1)
      
C  Check end point.
      
      IF(KS.EQ.2) THEN
         PHI = -DL/R
         CALL UPHELX(HLXCT,PHI,XV,PV)
         RDK = SQRT(XV(1)**2+XV(2)**2)
      ELSE
         RDK = 99999.
      ENDIF
      
      IF ( R1.EQ.RCY1 .AND. KS.NE.2 .AND. AR.GT.ARTEST .AND.
     &     ABS(TNL).LT.TNTEST ) THEN
         IRT = 1
      ELSE
         CALL UHLXCY(RCYTK,ZCYTK,EPSL,HLXCT,PHI,XV,IRT)
      ENDIF
      
      IF ( IRT.LE.0 ) THEN
         IF ( ABS(TNL).LT.TNTST2 ) THEN
            R2  = SQRT(XC**2+YC**2) + AR
         ELSE
            RETURN
         ENDIF
      ELSE IF ( IRT.EQ.1 ) THEN
         R2  = RCY2
      ELSE
         R2  = SQRT(XV(1)**2+XV(2)**2)
      ENDIF
      R2 = MIN(R2,RDK)

C  Reject tracks with insufficient # hits.
      
cccc  NHIT = (R2-R1-0.1)/DSMPTK
cccc  IF ( NHIT.LT.NSMPCT ) RETURN
      
C  Move the pivot from the production point to the 1-st
C  measured point.
      
      CALL UCOPY(HLXCT,HLSMR,8)
      IF ( R1.EQ.RCY1 ) THEN
         RCY(1) = 1.E-2
         RCY(2) = RCYTK(1)
         CALL UHLXCY(RCY,ZCYTK,EPSL,HLXCT,PHI,XV,IRT)
cccc     HLSMR(2) = FI0 + PHI
         CALL UCOPY(XV(1),HLSMR(6),3)
      ENDIF
      
C  Calculate inverse error matrix and smear helix parameters.
      
      IF ( NHIT.GE.NSMPTK ) THEN
         CALL UCOPY(SGBS,SGB,10)
         CALL UCOPY(DADB1S,DADB1,18)
         CALL UCOPY(DADB2S,DADB2, 8)
      ELSE
         CALL SMRHLE(NHIT,DSMPTK,SIGRF,SIGZE,SGB,DADB1,DADB2,IRT)
         IF ( IRT.LT.0 ) THEN
            IRET = - 2
            RETURN
         ENDIF
      ENDIF
      
      CALL RNORML(RVEC,LVEC)
      DO 10 I = 1, 5
ccccc    CALL RANNOR(RNDM1,RNDM2)
         SGB(I) = SGB(I)*RVEC(I)
 10   CONTINUE
      
      DO 200 I = 1, 3
         DEV = 0
         DO 20 J = 1, 3
            DEV = DEV + DADB1(I,J)*SGB(J)
 20      CONTINUE
         HLSMR(I) = HLSMR(I) + DEV
 200  CONTINUE
      
      DO 300 I = 1, 2
         DEV = 0
         DO 30 J = 1, 2
            DEV = DEV + DADB2(I,J)*SGB(J+3)
 30      CONTINUE
         HLSMR(I+3) = HLSMR(I+3) + DEV
 300  CONTINUE
      
C  Calculate error matrix.
      
      CALL VZERO(ERROR,30)
      N = 0
      DO 400 J = 1, 3
         DO 40 I = 1, J
            N = N + 1
            DO 4 M = 1, 3
               ERROR(N) = ERROR(N)
     &              + DADB1(I,M)*SGB(M)**2*DADB1(J,M)
 4          CONTINUE
 40      CONTINUE
 400  CONTINUE
      
      DO 500 J = 1, 2
         N = N + 3
         DO 50 I = 1, J
            N = N + 1
            DO 5 M = 1, 2
               ERROR(N) = ERROR(N)
     &              + DADB2(I,M)*SGB(M+3)**2*DADB2(J,M)
 5          CONTINUE
 50      CONTINUE
 500  CONTINUE
ccccc CALL UCOPY(ERROR(1),HLSMR(9),30)
      
C  Set other information.
      
c      HLSMR(39)  = 0
c      HLSMR(40)  = 0
      IDATA(1) = 2*NHIT - 5
      IDATA(2) = 0
      IDATA(3) = 5
      IDATA(4) = 0
      IDATA(5) = 0
      IDATA(6) = 0
ccccc CALL UCOPY(IDATA,HLSMR(41),6)
ccccc HLSMR(47)  = R1
ccccc HLSMR(48)  = R2
      
C  OK.
      
      IRET = 0
      
C  That's it.
      
      RETURN
      END
