      SUBROUTINE FTLINE( N, XHIT, YHIT, SIGF, Y0, DYDX, ERRMAT,
     &                   RESI, CHISQ )
*.
*...FTLINE   subroutine to fit straight line.
*.
*.  INPUT    : N      number of hits
*.           : XHIT   x value of hit points
*.           : YHIT   y value of hit points
*.           : SIGF   standard deviation of y hit positions
*.  OUTPUT   : Y0     a intercept of the fitted straight line
*.           : DYDX   a slope the of fitted straight line
*.           : ERRM error matrix of the parameters
*.           :   ERRM(i,j). i and j are the parameter
*.           :        1--Zmin, 2--dYdX
*.           :   The error matrix is the invert matrix of
*.           :   ERRM(1,1) =           = d2(chisq)/d(Zmin)d(Zmin)
*.           :   ERRM(1,2) = ERRM(2,1) = d2(chisq)/d(Zmin)d(dYdX)
*.           :   ERRM(2,2) =           = d2(chisq)/d(dYdX)d(dYdX)
*.           : RESI   Residual for the points
*.           : CHISQ  Chi squared
*.
*. AUTHOR    : J. Kanzaki
*. VERSION   : 1.00
*. CREATED   : 06-Mar-1996
*. LAST MOD  : 06-Mar-1996
*.
*. HISTORY   :
*. 20-Jul-1982  Created by H. Boerner (KEK)
*. 20-Feb-1986  Changed by  Takashi Oyama (TMU)
*. 20-Jan-1988  Entry NLINFER is added
*. 21-Mar-1996  Array size of XHIT, YHIT, SIGF, REST changed from N to 
*.              NHMAX=1000, since that in ctalal is 1000.    Y.Fujii
*.
      IMPLICIT NONE
      INTEGER N 
      INTEGER NHMAX 
      PARAMETER (NHMAX=3000)
C      REAL XHIT(N),YHIT(N),SIGF(N),RESI(N),CHISQ,Y0,DYDX,ERRM(2,2)
      REAL XHIT(NHMAX),YHIT(NHMAX),SIGF(NHMAX),RESI(NHMAX),
C     &     CHISQ,Y0,DYDX,ERRM(2,2)
     &     CHISQ,Y0,DYDX
      DOUBLE PRECISION ERRMAT(2,2)
      DOUBLE PRECISION X, XX, XY, Y, W
      DOUBLE PRECISION DET, A11, A12, A21, A22, DDYDX, DY0, DY
      DOUBLE PRECISION DS2, DOX, DOY

      REAL BIG
      PARAMETER( BIG = 1.0E+12 )
      DOUBLE PRECISION DSMALL
      PARAMETER (DSMALL = 1.0D-6)

      INTEGER I


      X  = 0.0D0
      XX = 0.0D0
      XY = 0.0D0
      Y  = 0.0D0
      W  = 0.0D0
      CHISQ = 0.0D0
      DO 10, I = 1,N
      DS2 = 1.0D0/DBLE( SIGF( I ))**2
      DOX = DBLE( XHIT( I ))
      DOY = DBLE( YHIT( I ))
      X   = X  + DOX        *DS2
      XX  = XX + DOX**2     *DS2
      XY  = XY + DOX   *DOY *DS2
      Y   = Y  +        DOY *DS2
      W   = W  +             DS2
   10 CONTINUE
      DET = XX*W - X ** 2
      IF (DABS(DET).GT.DSMALL) THEN
         A11 =  W/DET
         A12 = -X/DET
         A21 = -X/DET
         A22 = XX/DET
         DDYDX = A11*XY + A12*Y
         DY0   = A21*XY + A22*Y
         DO 20, I = 1,N
         DY = DDYDX * XHIT( I ) + DY0 - YHIT( I )
         RESI( I ) = DY/SIGF(I)
         CHISQ     = CHISQ + DY**2/( DDYDX**2+1 )/SIGF(I)**2
   20    CONTINUE
         DYDX = DDYDX
         Y0   = DY0
      ELSE
         DYDX = 0.0E0
         Y0   = 0.0E0
         CHISQ = BIG
      ENDIF
      ERRMAT(1,1)=A22
      ERRMAT(1,2)=A12
      ERRMAT(2,1)=A21
      ERRMAT(2,2)=A11
C      ERRM(1,1)=ERRMAT(1,1)
C      ERRM(2,1)=ERRMAT(2,1)
C      ERRM(1,2)=ERRMAT(1,2)
C      ERRM(1,2)=ERRMAT(2,2)
      RETURN
      END
