C   03/08/85 511021406 MEMBER NAME  GMCIR2   (S)           FORTRAN
************************************************************************
*     ------------------------------------------------------
      SUBROUTINE GMCIR2(X1,Y1,DX1,DY1,X2,Y2,XC,YC,RC,IERR)
*     ------------------------------------------------------
*
*     (Purpose)
*        Calculate the coordinate of center and radius of circle
*        when two points on the circle and the tangent of the
*        circle at one of these points are given.
*
*     (Input)
*        X1,Y1  : Coordinate of point on the circle.
*        DX1    : X componet of tanget of circle at  (X1,Y1).
*        DY1    : Y componet of tanget of circle at  (X1,Y1).
*        X2,Y2  : Coordinate of other point on the circle.
*
*     (Outputs)
*        XC,YC  : Center of the circle.
*        RC     : Radius of the circle.
*        IERR=0 : Normal end.
*        IERR=1 : No solution found - tanget of given two point has
*                                     value equal to tangent of circle.
*
*     (Creation Date)
*        1985.08.03 : First version by  K.Amako.
*
************************************************************************
      IMPLICIT NONE

      REAL X1,Y1,DX1,DY1,X2,Y2,XC,YC,RC
      INTEGER IERR

      DOUBLE PRECISION X1D,Y1D,DX1D,DY1D,X2D,Y2D
      DOUBLE PRECISION DX,DY,DD,DAX,DAY

***** In the case DYDX is not given.

      IF(DX1.EQ.0. .AND. DY1.EQ.0.) THEN
        IERR=1
        RETURN

***** In the case DYDX is not zero nor infinite

       ELSE

         X1D=X1
         Y1D=Y1
         DX1D=DX1
         DY1D=DY1
         X2D=X2
         Y2D=Y2

         DX=X2D-X1D
         DY=Y2D-Y1D

         DD=DX*DY1D-DY*DX1D

         IF(DD.EQ.0.) THEN
            IERR=1
            RETURN
         END IF

         IERR=0

         DAX=0.5D0*DY1D*(DX**2+DY**2)/DD
         DAY=0.5D0*DX1D*(DX**2+DY**2)/DD
         RC=DSQRT(DAX**2+DAY**2)
         XC=X1D+DAX
         YC=Y1D-DAY
      END IF

      RETURN
      END
